/****************************************
 *  COPYRIGHT (C) 2008-2017
 *  Holger Graf
 ****************************************/
package sitool;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import sitool.components.Accountable;
import sitool.components.Fleet;
import sitool.components.Planet;
import sitool.components.ProductionBonus;
import sitool.components.Specialization;
import sitool.components.Technology;
import sitool.components.Version;
import sitool.i18n.Language;

/**
 * @author graf
 *
 */
public class Account implements Accountable, Comparable<Account>
{
    private String id;
    private boolean defaultAccount;

    private List<Planet> myPlanets = new ArrayList<Planet>();
    private Fleet totalShips;
    private Technology myTech = new Technology();
    private Language language;
    private ResourceBundle resources;
    private double speedFactor = 1.0;
    private double speedUp = 1.0;
    private double fleetSpeed = 1.0;
    private boolean spectralMode = false;
    private Version version = Version.retro;
    private int orbitTicks = 0;
    private int galaxyTicks = 0;
    private boolean sync = false;
    private final Specialization specialization;
    private final ProductionBonus productionBonus;
    private boolean mayImportPlanetaryOverview = false;
    private boolean mayImportSpyReports = false;
    private boolean isOutpost = false;

    /**
     * @param id
     */
    public Account( String id, boolean isDefault )
    {
        this.id = id;
        this.defaultAccount = isDefault;
        this.specialization = new Specialization();
        this.productionBonus = new ProductionBonus();
   }

    public Account( Account other )
    {
        this.id = other.id;
        this.defaultAccount = other.defaultAccount;
        this.myPlanets = other.myPlanets;
        this.totalShips = other.totalShips;
        this.myTech = other.myTech;
        this.language = other.language;
        this.speedFactor = other.speedFactor;
        this.speedUp = other.speedUp;
        this.fleetSpeed = other.fleetSpeed;
        this.specialization = other.specialization;
        this.productionBonus = other.productionBonus;
        this.spectralMode = other.spectralMode;
        this.version = other.version;
        this.resources = other.resources;
        this.orbitTicks = other.orbitTicks;
        this.galaxyTicks = other.galaxyTicks;
        this.sync = other.sync;
        this.mayImportPlanetaryOverview = other.mayImportPlanetaryOverview;
        this.mayImportSpyReports = other.mayImportSpyReports;
        this.isOutpost = other.isOutpost;
    }

    public String getId()
    {
        return this.id;
    }

    public void setId( String id )
    {
        this.id = id;
    }

    public boolean isDefault()
    {
        return this.defaultAccount;
    }

    public void setDefault( boolean isDefault )
    {
        this.defaultAccount = isDefault;
    }

    /**
     * @return the myTech
     */
    public Technology getMyTech()
    {
        return myTech;
    }

    /**
     * @param myTech the myTech to set
     */
    public void setMyTech( Technology myTech )
    {
        this.myTech = myTech;
    }

    /**
     * @return the myPlanets
     */
    public List<Planet> getMyPlanets()
    {
        return myPlanets;
    }

    public boolean hasTotalShips() {
        return totalShips != null;
    }

    public Fleet getTotalShips() {
        Fleet rv = new Fleet();
        if(totalShips != null) {
            rv.addShips(totalShips.getShips());
        }
        else {
            for(Planet planet : myPlanets ) {
                for(boolean isAstro : new boolean[] { false, true } ) {
                    rv.addShips(planet.getFleet(isAstro).getShips());
                }
            }
        }
        return rv;
    }

    public void setTotalShips(Fleet totalShips) {
        this.totalShips = totalShips;
    }

    /**
     * @return the lookupPlanets
     */
    public Map<Planet, Planet> getLookupPlanets()
    {
        Map<Planet, Planet> lookupPlanets = new HashMap<Planet, Planet>();
        for( Planet planet : myPlanets )
        {
            lookupPlanets.put( planet, planet );
        }
        return lookupPlanets;
    }

    /**
     * @return the language
     */
    public Language getLanguage()
    {
        return language;
    }

    /**
     * @param language the language to set
     */
    public void setLanguage( Language language )
    {
        this.language = language;
        // reload resource bundle
        try
        {
            resources = ResourceBundle.getBundle( ConfigData.resourcePath, language.getLocale() );
        }
        catch( NullPointerException ex )
        {
            resources = null;
        }
        catch( MissingResourceException ex )
        {
            resources = null;
        }
    }

    public ResourceBundle getResourceBundle()
    {
        return resources;
    }

    public double getSpeedFactor()
    {
        return speedFactor;
    }

    public void setSpeedFactor( double speedFactor )
    {
        this.speedFactor = speedFactor;
    }

    public double getSpeedUp()
    {
        return speedUp;
    }

    public void setSpeedUp( double speedUp )
    {
        this.speedUp = speedUp;
    }

    public double getFleetSpeed()
    {
        return fleetSpeed;
    }

    public void setFleetSpeed( double fleetSpeed )
    {
        this.fleetSpeed = fleetSpeed;
    }

    public Version getVersion()
    {
        return version;
    }

    public void setVersion( Version version )
    {
        this.version = version;
    }

    public int getTicks()
    {
        return orbitTicks;
    }

    public void setTicks( int ticks )
    {
        this.orbitTicks = ticks;
    }

    public int getGalaxyTicks()
    {
        return galaxyTicks;
    }

    public void setGalaxyTicks( int ticks )
    {
        this.galaxyTicks = ticks;
    }

    public boolean isSync()
    {
        return sync;
    }

    public void setSync( boolean sync )
    {
        this.sync = sync;
    }

    public boolean isSpectralMode() {
        return spectralMode;
    }

    public void setSpectralMode( boolean spectralMode ) {
        this.spectralMode = spectralMode;
    }

    @Override
    public Specialization getSpecialization() {
        return specialization;
    }

    @Override
    public ProductionBonus getProductionBonus() {
        return productionBonus;
    }

    public boolean isMayImportPlanetaryOverview() {
        return mayImportPlanetaryOverview;
    }

    public void setMayImportPlanetaryOverview( boolean mayImportPlanetaryOverview ) {
        this.mayImportPlanetaryOverview = mayImportPlanetaryOverview;
    }

    public boolean isMayImportSpyReports() {
        return mayImportSpyReports;
    }

    public void setMayImportSpyReports( boolean mayImportSpyReports ) {
        this.mayImportSpyReports = mayImportSpyReports;
    }

    public boolean isOutpost() {
        return isOutpost;
    }

    public void setOutpost( boolean isOutpost ) {
        this.isOutpost = isOutpost;
    }

    /**
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString()
    {
        return id;
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals( Object obj )
    {
        boolean rv = false;
        if( obj instanceof Account )
        {
            rv = id.equals( ( (Account)obj ).id );
        }
        return rv;
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode()
    {
        return id.hashCode();
    }

    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo( Account o )
    {
        return id.compareTo( o.id );
    }
}
