/****************************************
 *  COPYRIGHT (C) 2008-2022
 *  Holger Graf
 ****************************************/
package sitool;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.text.DecimalFormat;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.TreeSet;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import sitool.components.Building;
import sitool.components.Coordinates;
import sitool.components.Defense;
import sitool.components.EntryComparator;
import sitool.components.Fleet;
import sitool.components.FleetSpeed;
import sitool.components.LaFStyle;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.Research;
import sitool.components.Resource;
import sitool.components.Ship;
import sitool.components.SiToolFileFilter;
import sitool.components.Specialization;
import sitool.components.Stage;
import sitool.components.Technology;
import sitool.components.TotalPointDisplay;
import sitool.components.Version;
import sitool.i18n.Language;
import sitool.planets.Coordinator;
import sitool.resources.ImportExportPane;
import sitool.savecalc.Calculator;

public class ConfigData
{
    private static final String fileName = ".si_tool";
    private static final String planetsName = "/planets-SI2-U1.xml";
    private File configFile;
    private File accountDir;

    protected final static int defaultBufferSize = 4096;
    public static DecimalFormat speedFormat = new DecimalFormat("#0.0");

    public static final char auml = '\u00e4';
    public static final char ouml = '\u00f6';
    public static final char uuml = '\u00fc';
    public static final char szlig = '\u00df';
    public static final String resourcePath = "sitool.i18n.Messages";
    public static final long MSECPERDAY = 1000 * 60 * 60 * 24;

    private Parser xmlParser = null;
    private List<Account> accounts = new ArrayList<Account>();

    private FleetSpeed velocity = new FleetSpeed();
    private Account activeAccount;
    private ResourceBundle defaultResources;
    private LaFStyle currentLaF;

    private Map<Position, Coordinates> zeroPointMap = new HashMap<>();
    private Map<Position, Coordinates> translationCache = new HashMap<Position,Coordinates>();
    private EntryComparator entryComparator = new EntryComparator();

    private static final int defaultWidth = 940;
    private static final int defaultHeight = 715;
    private Dimension size = new Dimension();

    /**
     *
     */
    public ConfigData()
    {
        try
        {
            xmlParser = new Parser();
        }
        catch( ParserConfigurationException e )
        {
        }

        configFile = findConfigFile( fileName );
        accountDir = configFile.getParentFile();
        velocity.setAcceleration( 0 );
        velocity.setPreptime( 0 );
        velocity.setVelocity( 0 );

        // default fallback is english
        try
        {
            Locale locale = Language.english.getLocale();
            defaultResources = ResourceBundle.getBundle( ConfigData.resourcePath, locale );
        }
        catch( NullPointerException ex )
        {}
        catch( MissingResourceException ex )
        {
            ex.printStackTrace();
        }
    }

    public Dimension getSize()
    {
        return this.size;
    }

    public void readConfigData()
    {
        currentLaF = LaFStyle.defaultLaF;
        accounts.clear();

        Document doc;
        try(InputStream is = getClass().getResourceAsStream(planetsName)) {
            doc = xmlParser.getDocument( is );
            if( doc != null )
            {
                importCoordinates( doc );
            }
        }
        catch(IOException e) {
            e.printStackTrace();
        }
        doc = readFile( configFile, true );
        if( doc != null )
        {
            Element rootEl = ( Element )doc.getFirstChild();
            int i = parseIntegerAttribute( rootEl.getAttribute( Parser.Attribute_Width ) );
            size.width = ( i > 0 ) ? i : defaultWidth;
            i = parseIntegerAttribute( rootEl.getAttribute( Parser.Attribute_Height ) );
            size.height = ( i > 0 ) ? i : defaultHeight;
            currentLaF = parseLaFAttribute( rootEl.getAttribute( Parser.Attribute_LaF ) );
            accounts.addAll( readAccountsFromConfig( doc ) );
        }
        else
        {
            size.width = defaultWidth;
            size.height = defaultHeight;
        }

    }

    public void saveConfigData()
    {
        Document doc = xmlParser.getDocument();
        addAccounts2Config( doc, accounts );
        saveFile( configFile, doc, true );
        doc = xmlParser.getDocument();
    }

    public void adjustCoordinates()
    {
        // account has been resynced, clear translation cache
        this.translationCache.clear();
        Account current = getActiveAccount();
        if( activeAccount != null && activeAccount.getVersion() == Version.v20 )
        {
            List<Planet> myPlanets = current.getMyPlanets();
            int galaxyTicks = current.getGalaxyTicks();
            int ticks = current.getTicks();
            Map<Position, Coordinates> map = getCoordinates();
            for( Planet planet: myPlanets )
            {
                Position position = planet.getPosition();
                Coordinates coordinates = map.get( position );
                if( coordinates != null )
                {
                    planet.setCoordinates( translate( coordinates, galaxyTicks, ticks ) );
                }
                else
                {
                    planet.setCoordinates( new Coordinates( "" ) );
                }
            }
        }
    }

    public void importCoordinates( Element element, Map<Position, Coordinates> coordinates )
    {
        NodeList planetList = element.getElementsByTagName( Parser.Tag_Planet );
        Position position = null;
        if( planetList != null )
        {
            for( int j = 0; j < planetList.getLength(); j++  )
            {
                try
                {
                    Element planetNode = ( Element )planetList.item( j );
                    position = new Position( planetNode.getAttribute( Parser.Attribute_Position ) );
                    int orbit = parseIntegerAttribute( planetNode.getAttribute( Parser.Attribute_Orbit ) );
                    int start = parseIntegerAttribute( planetNode.getAttribute( Parser.Attribute_Start ) );
                    Coordinates c = new Coordinates( position, start, orbit );
                    coordinates.put( position, c );
                }
                catch( Exception ex ) {
                    ex.printStackTrace(System.err);
                    if( JOptionPane.showConfirmDialog( null, ex.toString(), "Error", JOptionPane.OK_CANCEL_OPTION, JOptionPane.ERROR_MESSAGE ) == JOptionPane.CANCEL_OPTION )
                    {
                        coordinates.clear();
                        break;
                    }
                }
            }
        }
    }

    private void importCoordinates( Document doc )
    {
        NodeList accountList = doc.getElementsByTagName( Parser.Tag_Account );
        if( accountList != null )
        {
            for( int i = 0; i < accountList.getLength(); i++  )
            {
                Element accountNode = ( Element )accountList.item( i );
                importCoordinates( accountNode, zeroPointMap );
            }
        }
    }

    //    private String printPosition(Position position) {
    //        StringBuilder sb = new StringBuilder();
    //        sb.append( position.getPlanet() );
    //        while(sb.length() < 2) {
    //            sb.insert( 0, '0' );
    //        }
    //        sb.insert( 0, ':' );
    //        sb.insert( 0, position.getSystem() );
    //        while(sb.length() < 6) {
    //            sb.insert( 0, '0' );
    //        }
    //        sb.insert( 0, ':' );
    //        sb.insert( 0, position.getGalaxy() );
    //        while(sb.length() < 10) {
    //            sb.insert( 0, '0' );
    //        }
    //        return sb.toString();
    //    }

    public void clearTranslationCache()
    {
        translationCache.clear();
    }

    /**
     * Due to possible collisions, the entire system must be considered when moving a planet
     * @param base
     * @param galaxyTicks
     * @param ticks
     * @return
     */
    public Coordinates translate( Coordinates base, int galaxyTicks, int ticks )
    {
        return translate( base, galaxyTicks, ticks, true );
    }

    /**
     * Due to possible collisions, the entire system must be considered when moving a planet
     * @param base
     * @param galaxyTicks
     * @param ticks
     * @return
     */
    public Coordinates translate( Coordinates base, int galaxyTicks, int ticks, boolean useCache )
    {
        Position position = base.getPosition();
        Coordinates translated = null;

        if( useCache )
        {
            translated = translationCache.get( position );
        }

        if(translated == null)
        {
            int galaxy = position.getGalaxy();
            int system = position.getSystem();
            Map<Position, Coordinates> systemMap = new HashMap<Position, Coordinates>();

            // shift the galaxy first
            Point galaxyPoint = Coordinates.getGalaxyPoint( position, galaxyTicks );
            Point systemPoint = Coordinates.getSystemPoint( position );

            if( ticks > 0 )
            {
                // collect all planets of system
                // the system sizes are not yet integrated, assume an upper limit of 20 planets per system
                Map<Position, Coordinates> map = getCoordinates();

                for( int planet = 1; planet <= Position.MAX_PLANETS; planet++ )
                {
                    Position key = new Position( galaxy, system, planet );
                    Coordinates coordinates = map.get( key );
                    if( coordinates == null )
                        break;

                    // make copies, we'll shift them into the new orbital positions
                    coordinates = new Coordinates( galaxyPoint, systemPoint, coordinates.getPlanet(), coordinates.getOrbit() );
                    coordinates.setPosition( key );
                    systemMap.put( key, coordinates );
                }

                shift( ticks, systemMap );

                // add the translated system to the cache
                if( useCache )
                {
                    translationCache.putAll( systemMap );
                }
            }

            translated = systemMap.get( position );
            if( translated == null )
            {
                translated = new Coordinates( galaxyPoint, systemPoint, base.getPlanet(), base.getOrbit() );
                translated.setPosition( position );
            }
        }
        return translated;
    }

    /**
     * Shift the Coordinates in <code>systemMap</code> forward by <code>ticks</code> positions
     * @param ticks
     * @param systemMap
     */
    public void shift( int ticks, Map<Position, Coordinates> systemMap )
    {
        // we assume systemMap is initially collision free, since it represents a real state
        for( int i = 0; i < ticks; i++ )
        {
            // shift
            for(Coordinates coordinates : systemMap.values() )
            {
                coordinates.shift( ticks );
            }
            resolveCollisions( systemMap, ticks );
        }
    }

    private void resolveCollisions( Map<Position, Coordinates> systemMap, int sign )
    {
        Map<Point, List<Position>> collisionMap = new HashMap<Point, List<Position>>();
        for(Coordinates coordinates : systemMap.values() )
        {
            Point point = new Point(coordinates.getX(), coordinates.getY());
            List<Position> positions = collisionMap.get( point );
            if( positions == null )
            {
                positions = new ArrayList<Position>();
                collisionMap.put( point, positions );
            }
            positions.add( coordinates.getPosition() );
        }
        // check for collisions
        boolean collided = false;
        for( List<Position> positions : collisionMap.values() )
        {
            if( positions.size() > 1 )
            {
                collided = true;
                for( Position position : positions )
                {
                    systemMap.get( position ).shift( sign );
                }
            }
        }
        // try again
        if( collided )
        {
            resolveCollisions( systemMap, sign );
        }
    }

    private Element createAppRoot( Document doc )
    {
        Element rootEl = doc.createElement( Parser.Tag_Application );
        doc.appendChild( rootEl );
        rootEl.setAttribute( Parser.Attribute_Width, Integer.toString( size.width ) );
        rootEl.setAttribute( Parser.Attribute_Height, Integer.toString( size.height ) );
        rootEl.setAttribute( Parser.Attribute_LaF, currentLaF.name() );
        rootEl.setAttribute( Parser.Attribute_Time, Long.toString( System.currentTimeMillis() ) );
        return rootEl;
    }

    void addAccounts2Config( Document doc, List<Account> accounts )
    {
        if( doc != null && accounts != null )
        {
            Element rootEl = createAppRoot( doc );
            for( Account account : accounts )
            {
                Element accountNode = doc.createElement( Parser.Tag_Account );
                rootEl.appendChild( accountNode );
                accountNode.setAttribute( Parser.Attribute_Name, account.getId() );
                accountNode.setAttribute( Parser.Attribute_Default, Boolean.toString( account.isDefault() ) );
                accountNode.setAttribute( Parser.Attribute_Speed, Double.toString( account.getSpeedFactor() ) );
                accountNode.setAttribute( Parser.Attribute_FleetSpeed, Double.toString( account.getFleetSpeed() ) );
                accountNode.setAttribute( Parser.Attribute_SpeedUp, Double.toString( account.getSpeedUp() ) );
                accountNode.setAttribute( Parser.Attribute_SpectralMode, Boolean.toString( account.isSpectralMode() ) );
                accountNode.setAttribute( Parser.Attribute_Specialization_Fleeter, Integer.toString( account.getSpecialization().getFleeter() ) );
                accountNode.setAttribute( Parser.Attribute_Specialization_Miner, Integer.toString( account.getSpecialization().getMiner() ) );
                accountNode.setAttribute( Parser.Attribute_Specialization_Researcher, Integer.toString( account.getSpecialization().getResearcher() ) );
                accountNode.setAttribute( Parser.Attribute_Productionbonus_Iron, Integer.toString( account.getProductionBonus().getIron() ) );
                accountNode.setAttribute( Parser.Attribute_Productionbonus_Metal, Integer.toString( account.getProductionBonus().getMetal() ) );
                accountNode.setAttribute( Parser.Attribute_Productionbonus_Kryptonite, Integer.toString( account.getProductionBonus().getKryptonite() ) );
                accountNode.setAttribute( Parser.Attribute_Productionbonus_Spice, Integer.toString( account.getProductionBonus().getSpice() ) );
                accountNode.setAttribute( Parser.Attribute_Productionbonus_Energy, Integer.toString( account.getProductionBonus().getEnergy() ) );
                accountNode.setAttribute( Parser.Attribute_ImportPlanets, Boolean.toString( account.isMayImportPlanetaryOverview() ) );
                accountNode.setAttribute( Parser.Attribute_ImportSpyReports, Boolean.toString( account.isMayImportSpyReports() ) );
                accountNode.setAttribute( Parser.Attribute_Outpost, Boolean.toString( account.isOutpost() ) );
                Language language = account.getLanguage();
                if( language != null )
                {
                    accountNode.setAttribute( Parser.Attribute_Language, language.name() );
                }
                Version version = account.getVersion();
                if( version != null )
                {
                    accountNode.setAttribute( Parser.Attribute_Version, version.name() );
                }
                int ticks = account.getTicks();
                if( ticks != 0 )
                {
                    accountNode.setAttribute( Parser.Attribute_Time, Integer.toString( ticks ) );
                }
                ticks = account.getGalaxyTicks();
                if( ticks != 0 )
                {
                    accountNode.setAttribute( Parser.Attribute_GalaxyTime, Integer.toString( ticks ) );
                }

                Element techNode = doc.createElement( Parser.Tag_Module_Technology );
                accountNode.appendChild( techNode );

                Technology myTech = account.getMyTech();
                if( myTech == null )
                {
                    myTech = new Technology();
                }
                Element ecoTechNode = doc.createElement( Parser.Tag_EcoTech );
                techNode.appendChild( ecoTechNode );
                ecoTechNode.setAttribute( Research.mining.name(), Integer.toString( myTech.getMining() ) );
                ecoTechNode.setAttribute( Research.energy.name(), Integer.toString( myTech.getEnergy() ) );

                Element driveTechNode = doc.createElement( Parser.Tag_DriveTech );
                techNode.appendChild( driveTechNode );
                driveTechNode.setAttribute( Research.drive.name(), Integer.toString( myTech.getDrive() ) );
                driveTechNode.setAttribute( Research.rocket.name(), Integer.toString( myTech.getRocket() ) );
                driveTechNode.setAttribute( Research.diffusion.name(), Integer.toString( myTech.getDiffusion() ) );
                driveTechNode.setAttribute( Research.warp.name(), Integer.toString( myTech.getWarp() ) );

                Element weaponsTechNode = doc.createElement( Parser.Tag_WeaponTech );
                techNode.appendChild( weaponsTechNode );
                weaponsTechNode.setAttribute( Research.weapons.name(), Integer.toString( myTech.getWeapons() ) );
                weaponsTechNode.setAttribute( Research.shields.name(), Integer.toString( myTech.getShields() ) );
                weaponsTechNode.setAttribute( Research.structure.name(), Integer.toString( myTech.getStructure() ) );

                Element otherTechNode = doc.createElement( Parser.Tag_OtherTech );
                techNode.appendChild( otherTechNode );
                otherTechNode.setAttribute( Research.espionage.name(), Integer.toString( myTech.getEspionage() ) );
                otherTechNode.setAttribute( Research.target.name(), Integer.toString( myTech.getTarget() ) );
                otherTechNode.setAttribute( Research.particle.name(), Integer.toString( myTech.getParticle() ) );
                otherTechNode.setAttribute( Research.teleportation.name(), Integer.toString( myTech.getTeleportation() ) );
                otherTechNode.setAttribute( Research.graviton.name(), Integer.toString( myTech.getGraviton() ) );
                otherTechNode.setAttribute( Research.blackhole.name(), Integer.toString( myTech.getBlackhole() ) );

                // fleet - total ships
                if(account.hasTotalShips()) {
                    Element fleetNode = doc.createElement( Parser.Tag_TotalFleet );
                    accountNode.appendChild( fleetNode );
                    for( Map.Entry<Ship, Long> entry : account.getTotalShips().getShips().entrySet() )
                    {
                        if( entry.getValue() > 0 )
                        {
                            fleetNode.setAttribute( entry.getKey().name(), Long.toString( entry.getValue() ) );
                        }
                    }
                }

                List<Planet> myPlanets = account.getMyPlanets();

                Element planetsNode = doc.createElement( Parser.Tag_Module_Planets );
                accountNode.appendChild( planetsNode );

                if( myPlanets != null )
                {
                    for( Planet planet : myPlanets )
                    {
                        Element planetNode = doc.createElement( Parser.Tag_Planet );
                        planetsNode.appendChild( planetNode );

                        Position position = planet.getPosition();
                        planetNode.setAttribute( Parser.Attribute_Name, planet.getName() );
                        planetNode.setAttribute( Parser.Attribute_Galaxy, Integer.toString( position.getGalaxy() ) );
                        planetNode.setAttribute( Parser.Attribute_System, Integer.toString( position.getSystem() ) );
                        planetNode.setAttribute( Parser.Attribute_Planet, Integer.toString( position.getPlanet() ) );
                        planetNode.setAttribute( Parser.Attribute_AstroSize, Integer.toString( planet.getAstroSize() ) );
                        planetNode.setAttribute( Parser.Attribute_Size, Integer.toString( planet.getSize() ) );
                        planetNode.setAttribute( Parser.Attribute_Mine, Boolean.toString( planet.isMine() ) );

                        if( planet.isMine() )
                        {
                            Fleet fleet = planet.getFleet( false );
                            // fleet - ships
                            Element fleetNode = doc.createElement( Parser.Tag_Fleet );
                            planetNode.appendChild( fleetNode );
                            for( Ship ship : Ship.values() )
                            {
                                long count = fleet.getCount( ship );
                                if( count > 0 )
                                {
                                    fleetNode.setAttribute( ship.name(), Long.toString( count ) );
                                }
                            }
                            // fleet - defenses
                            fleetNode = doc.createElement( Parser.Tag_Defense );
                            planetNode.appendChild( fleetNode );
                            for( Defense defense : Defense.values() )
                            {
                                long count = fleet.getCount( defense );
                                if( count > 0 )
                                {
                                    fleetNode.setAttribute( defense.name(), Long.toString( count ) );
                                }
                            }

                            fleet = planet.getFleet( true );
                            // astro fleet - ships
                            fleetNode = doc.createElement( Parser.Tag_AstroFleet );
                            planetNode.appendChild( fleetNode );
                            for( Ship ship : Ship.values() )
                            {
                                long count = fleet.getCount( ship );
                                if( count > 0 )
                                {
                                    fleetNode.setAttribute( ship.name(), Long.toString( count ) );
                                }
                            }
                            // astro fleet - defenses
                            fleetNode = doc.createElement( Parser.Tag_AstroDefense );
                            planetNode.appendChild( fleetNode );
                            for( Defense defense : Defense.values() )
                            {
                                long count = fleet.getCount( defense );
                                if( count > 0 )
                                {
                                    fleetNode.setAttribute( defense.name(), Long.toString( count ) );
                                }
                            }
                        }
                        Element buildingNode = doc.createElement( Parser.Tag_Buildings );
                        planetNode.appendChild( buildingNode );
                        Map<Building, Stage> buildings;
                        buildings = planet.getBuildings( false );
                        for( Map.Entry<Building, Stage> entry : buildings.entrySet() )
                        {
                            buildingNode.setAttribute( entry.getKey().name(), Integer.toString( entry.getValue().getLevel() ) );
                        }
                        buildingNode = doc.createElement( Parser.Tag_AstroBuildings );
                        planetNode.appendChild( buildingNode );
                        buildings = planet.getBuildings( true );
                        for( Map.Entry<Building, Stage> entry : buildings.entrySet() )
                        {
                            buildingNode.setAttribute( entry.getKey().name(), Integer.toString( entry.getValue().getLevel() ) );
                        }
                    }
                }
            }
        }
    }

    List<Account> readAccountsFromConfig( Document doc )
    {
        List<Account> result = new ArrayList<Account>();
        // cut right through
        NodeList accountList = doc.getElementsByTagName( Parser.Tag_Account );
        if( accountList != null )
        {
            for( int i = 0; i < accountList.getLength(); i++  )
            {
                Element accountNode = ( Element )accountList.item( i );
                Version version = parseVersionAttribute( accountNode.getAttribute( Parser.Attribute_Version ) );
                String id = accountNode.getAttribute( Parser.Attribute_Name );
                boolean isDefault = parseBooleanAttribute( accountNode.getAttribute( Parser.Attribute_Default ) );
                Language language = parseLanguageAttribute( accountNode.getAttribute( Parser.Attribute_Language ) );
                double speed = parseSpeedAttribute( accountNode.getAttribute( Parser.Attribute_Speed ) );
                double fleetSpeed = parseSpeedAttribute( accountNode.getAttribute( Parser.Attribute_FleetSpeed ) );
                double speedUp = parseSpeedAttribute( accountNode.getAttribute( Parser.Attribute_SpeedUp ) );
                int fleeter = parseIntegerAttribute( accountNode.getAttribute( Parser.Attribute_Specialization_Fleeter ) );
                int miner = parseIntegerAttribute( accountNode.getAttribute( Parser.Attribute_Specialization_Miner ) );
                int researcher = parseIntegerAttribute( accountNode.getAttribute( Parser.Attribute_Specialization_Researcher ) );
                int bonusIron = parseIntegerAttribute( accountNode.getAttribute( Parser.Attribute_Productionbonus_Iron ) );
                int bonusMetal = parseIntegerAttribute( accountNode.getAttribute( Parser.Attribute_Productionbonus_Metal ) );
                int bonusKryptonite = parseIntegerAttribute( accountNode.getAttribute( Parser.Attribute_Productionbonus_Kryptonite ) );
                int bonusSpice = parseIntegerAttribute( accountNode.getAttribute( Parser.Attribute_Productionbonus_Spice ) );
                int bonusEnergy = parseIntegerAttribute( accountNode.getAttribute( Parser.Attribute_Productionbonus_Energy ) );
                boolean spectralMode = parseBooleanAttribute( accountNode.getAttribute( Parser.Attribute_SpectralMode ) );
                boolean mayImportPlanets = parseBooleanAttribute( accountNode.getAttribute( Parser.Attribute_ImportPlanets ) );
                boolean mayImportSpyReports = parseBooleanAttribute( accountNode.getAttribute( Parser.Attribute_ImportSpyReports ) );
                boolean isOutpost = parseBooleanAttribute( accountNode.getAttribute( Parser.Attribute_Outpost ) );
                Account account = new Account( id, isDefault );
                account.setLanguage( language );
                account.setVersion( version );
                account.setSpeedFactor( speed );
                account.setFleetSpeed( fleetSpeed );
                account.setSpeedUp( speedUp );
                account.setSpectralMode(spectralMode);
                account.setMayImportPlanetaryOverview(mayImportPlanets);
                account.setMayImportSpyReports(mayImportSpyReports);
                account.getSpecialization().setFleeter(fleeter);
                account.getSpecialization().setMiner(miner);
                account.getSpecialization().setResearcher(researcher);
                account.getProductionBonus().setIron(bonusIron);
                account.getProductionBonus().setMetal(bonusMetal);
                account.getProductionBonus().setKryptonite(bonusKryptonite);
                account.getProductionBonus().setSpice(bonusSpice);
                account.getProductionBonus().setEnergy(bonusEnergy);
                account.setOutpost(isOutpost);
                if( version == Version.v20 )
                {
                    int ticks = parseIntegerAttribute( accountNode.getAttribute( Parser.Attribute_Time ) );
                    account.setTicks( ticks );
                    int galaxyTicks = parseIntegerAttribute( accountNode.getAttribute( Parser.Attribute_GalaxyTime ) );
                    account.setGalaxyTicks( galaxyTicks );
                    try {
                        Coordinator coordinator = new Coordinator(this, true);
                        boolean sync = galaxyTicks == coordinator.getCurrentGalaxyTicks() && ticks == coordinator.getCurrentTicks();
                        account.setSync(sync);
                    }
                    catch(IOException e) {
                        e.printStackTrace();
                    }
                }
                result.add( account );
                if( isDefault )
                {
                    setActiveAccount( account );
                }
                Technology myTech = new Technology();
                NodeList list = accountNode.getElementsByTagName( Parser.Tag_DriveTech );
                Element tech = ( Element )list.item( 0 );
                if( tech != null )
                {
                    myTech.setDrive( parseIntegerAttribute( tech.getAttribute( Research.drive.name() ) ) );
                    myTech.setRocket( parseIntegerAttribute( tech.getAttribute( Research.rocket.name() ) ) );
                    myTech.setDiffusion( parseIntegerAttribute( tech.getAttribute( Research.diffusion.name() ) ) );
                    myTech.setWarp( parseIntegerAttribute( tech.getAttribute( Research.warp.name() ) ) );
                }

                list = accountNode.getElementsByTagName( Parser.Tag_EcoTech );
                tech = ( Element )list.item( 0 );
                if( tech != null )
                {
                    myTech.setEnergy( parseIntegerAttribute( tech.getAttribute( Research.energy.name() ) ) );
                    myTech.setMining( parseIntegerAttribute( tech.getAttribute( Research.mining.name() ) ) );
                }

                list = accountNode.getElementsByTagName( Parser.Tag_WeaponTech );
                tech = ( Element )list.item( 0 );
                if( tech != null )
                {
                    myTech.setWeapons( parseIntegerAttribute( tech.getAttribute( Research.weapons.name() ) ) );
                    myTech.setShields( parseIntegerAttribute( tech.getAttribute( Research.shields.name() ) ) );
                    myTech.setStructure( parseIntegerAttribute( tech.getAttribute( Research.structure.name() ) ) );
                }

                list = accountNode.getElementsByTagName( Parser.Tag_OtherTech );
                tech = ( Element )list.item( 0 );
                if( tech != null )
                {
                    myTech.setEspionage( parseIntegerAttribute( tech.getAttribute( Research.espionage.name() ) ) );
                    myTech.setTarget( parseIntegerAttribute( tech.getAttribute( Research.target.name() ) ) );
                    myTech.setParticle( parseIntegerAttribute( tech.getAttribute( Research.particle.name() ) ) );
                    myTech.setTeleportation( parseIntegerAttribute( tech.getAttribute( Research.teleportation.name() ) ) );
                    myTech.setGraviton( parseIntegerAttribute( tech.getAttribute( Research.graviton.name() ) ) );
                    myTech.setBlackhole( parseIntegerAttribute( tech.getAttribute( Research.blackhole.name() ) ) );
                }

                account.setMyTech( myTech );

                NodeList totalFleetList = accountNode.getElementsByTagName( Parser.Tag_TotalFleet );
                Element totalFleetNode;
                if( totalFleetList != null && totalFleetList.getLength() > 0 )
                {
                    Fleet totalFleet = new Fleet();
                    totalFleetNode = ( Element )totalFleetList.item( 0 );
                    for( Ship ship : Ship.values() )
                    {
                        long count = parseIntegerAttribute( totalFleetNode.getAttribute( ship.name() ) );
                        if( count > 0 )
                        {
                            totalFleet.setCount( ship, Long.valueOf( count ) );
                        }
                    }
                    account.setTotalShips(totalFleet);
                }

                NodeList myPlanets = accountNode.getElementsByTagName( Parser.Tag_Planet );
                List<Planet> planets = account.getMyPlanets();
                for( int j = 0; j < myPlanets.getLength(); j++ )
                {
                    Element planetNode = ( Element )myPlanets.item( j );
                    String name = planetNode.getAttribute( Parser.Attribute_Name );
                    int galaxy = parseIntegerAttribute( planetNode.getAttribute( Parser.Attribute_Galaxy ) );
                    int system = parseIntegerAttribute( planetNode.getAttribute( Parser.Attribute_System ) );
                    int planet = parseIntegerAttribute( planetNode.getAttribute( Parser.Attribute_Planet ) );
                    int size = parseIntegerAttribute( planetNode.getAttribute( Parser.Attribute_Size ) );
                    int astroSize = parseIntegerAttribute( planetNode.getAttribute( Parser.Attribute_AstroSize ) );
                    boolean isMine = Boolean.parseBoolean( planetNode.getAttribute( Parser.Attribute_Mine ) );
                    Position position = new Position( galaxy, system, planet );
                    Planet p = new Planet( name, position );
                    p.setMine( isMine );
                    p.setSize( size );
                    p.setAstroSize( astroSize );
                    planets.add( p );

                    // old style fleet
                    String s = planetNode.getAttribute( Parser.Attribute_Ship );
                    if( s != null )
                    {
                        Ship ship = null;
                        try
                        {
                            ship = Ship.valueOf( s );
                        }
                        catch( IllegalArgumentException e )
                        {}
                        if( ship != null )
                        {
                            p.getFleet( false ).setCount( ship, Long.valueOf( 1 ) );
                        }
                    }
                    // new style fleet
                    NodeList fleetList = planetNode.getElementsByTagName( Parser.Tag_Fleet );
                    Element fleetNode;
                    if( fleetList != null && fleetList.getLength() > 0 )
                    {
                        Fleet fleet = p.getFleet( false );
                        fleetNode = ( Element )fleetList.item( 0 );
                        for( Ship ship : Ship.values() )
                        {
                            int count = parseIntegerAttribute( fleetNode.getAttribute( ship.name() ) );
                            if( count > 0 )
                            {
                                fleet.setCount( ship, Long.valueOf( count ) );
                            }
                        }
                    }
                    // astro fleet
                    fleetList = planetNode.getElementsByTagName( Parser.Tag_AstroFleet );
                    if( fleetList != null && fleetList.getLength() > 0 )
                    {
                        Fleet fleet = p.getFleet( true );
                        fleetNode = ( Element )fleetList.item( 0 );
                        for( Ship ship : Ship.values() )
                        {
                            int count = parseIntegerAttribute( fleetNode.getAttribute( ship.name() ) );
                            if( count > 0 )
                            {
                                fleet.setCount( ship, Long.valueOf( count ) );
                            }
                        }
                    }
                    // defenses
                    fleetList = planetNode.getElementsByTagName( Parser.Tag_Defense );
                    if( fleetList != null && fleetList.getLength() > 0 )
                    {
                        Fleet fleet = p.getFleet( false );
                        fleetNode = ( Element )fleetList.item( 0 );
                        for( Defense defense : Defense.values() )
                        {
                            int count = parseIntegerAttribute( fleetNode.getAttribute( defense.name() ) );
                            if( count > 0 )
                            {
                                fleet.setCount( defense, Long.valueOf( count ) );
                            }
                        }
                    }
                    // astro defenses
                    fleetList = planetNode.getElementsByTagName( Parser.Tag_AstroDefense );
                    if( fleetList != null && fleetList.getLength() > 0 )
                    {
                        Fleet fleet = p.getFleet( true );
                        fleetNode = ( Element )fleetList.item( 0 );
                        for( Defense defense : Defense.values() )
                        {
                            int count = parseIntegerAttribute( fleetNode.getAttribute( defense.name() ) );
                            if( count > 0 )
                            {
                                fleet.setCount( defense, Long.valueOf( count ) );
                            }
                        }
                    }
                    if( isDefault && account.getVersion() == Version.v20 )
                    {
                        Coordinates coordinates = getCoordinates().get( p.getPosition() );
                        if( coordinates != null )
                        {
                            // enter current coordinates
                            p.setCoordinates( translate( coordinates, account.getGalaxyTicks(), account.getTicks() ) );
                        }
                        else
                        {
                            p.setCoordinates( new Coordinates( "" ) );
                        }
                    }

                    NodeList myBuildings = planetNode.getElementsByTagName( Parser.Tag_Buildings );
                    Element buildingNode;
                    version = account.getVersion();

                    if( myBuildings != null && myBuildings.getLength() > 0 )
                    {
                        buildingNode = ( Element )myBuildings.item( 0 );
                        int level;
                        Stage stage;
                        for( Building b : Building.planetBuildings() )
                        {
                            String value = buildingNode.getAttribute( b.name() );
                            if( value != null && value.trim().length() > 0 )
                            {
                                level = parseIntegerAttribute( value );
                                stage = new Stage(b, level);
                                p.addBuilding( b, stage, false );
                            }
                        }
                    }

                    myBuildings = planetNode.getElementsByTagName( Parser.Tag_AstroBuildings );
                    if( myBuildings != null && myBuildings.getLength() > 0 )
                    {
                        buildingNode = ( Element )myBuildings.item( 0 );
                        int level;
                        Stage stage;
                        for( Building b : Building.astroBuildings() )
                        {
                            String value = buildingNode.getAttribute( b.name() );
                            if( value != null && value.trim().length() > 0 )
                            {
                                level = parseIntegerAttribute( value );
                                stage = new Stage(b, level);
                                p.addBuilding( b, stage, true );
                            }
                        }
                    }
                }
                clearTranslationCache();
            }
        }

        return result;
    }

    public Document readData( ImportExportPane view )
    {
        Document doc = null;
        File inFile = getFile( view, false );
        if( inFile != null )
        {
            doc = readFile( inFile, true );
        }
        return doc;
    }

    public void saveData( ImportExportPane view )
    {
        File outFile = getFile( view, true );
        if( outFile != null )
        {
            Document doc = view.getData( xmlParser );
            if( doc != null )
            {
                saveFile( outFile, doc, true );
            }
        }
    }

    public String getI18nText( String key, Object... args )
    {
        String rv;
        String template = null;
        ResourceBundle resources = null;
        Locale locale = null;
        if( activeAccount != null )
        {
            resources = activeAccount.getResourceBundle();
        }
        if( resources == null )
        {
            resources = defaultResources;
        }
        if( resources != null )
        {
            locale = resources.getLocale();
            try
            {
                // check for version specific values first
                template = resources.getString( getVersion().name() + "." + key );
            }
            catch( MissingResourceException mre )
            {
                try
                {
                    template = resources.getString( key );
                }
                catch( Exception ex )
                {}
            }
            catch( NullPointerException ex )
            {}
        }
        if( template == null )
        {
            // last ditch defense
            template = key;
            locale = Language.english.getLocale();
        }
        if( args != null && args.length > 0 )
        {
            MessageFormat formatter = new MessageFormat("");
            formatter.setLocale(locale);

            formatter.applyPattern( template );
            rv = formatter.format( args );
        }
        else
        {
            rv = template;
        }
        return rv;
    }

    public FleetSpeed getVelocity()
    {
        return velocity;
    }

    /**
     * @param maxSpeed
     * @return
     */
    public String getUnitFromMaxSpeed( FleetSpeed maxSpeed )
    {
        String rv = null;

        if( maxSpeed.getVelocity() > 0 )
        {
            Set<String> shipNames = new TreeSet<String>();
            for( Ship ship : maxSpeed.getShips() )
            {
                if( ship != null )
                {
                    shipNames.add( getI18nText( ship.name() ) );
                }
            }
            if( !shipNames.isEmpty() )
            {
                Iterator<String> it = shipNames.iterator();
                StringBuilder sb = new StringBuilder();
                while( it.hasNext() )
                {
                    sb.append( it.next() );
                    if( it.hasNext() )
                    {
                        sb.append( "; " );
                    }
                }
                rv = sb.toString();
            }
        }

        if( rv == null )
        {
            rv = getVelocityText( maxSpeed );
        }
        return rv;
    }

    private String getVelocityText( FleetSpeed velocity )
    {
        String text;
        if( getVersion() != Version.v20 )
        {
            text = ( velocity.getVelocity() > 0 ) ? Long.toString( velocity.getVelocity() ) : "";
        }
        else
        {
            StringBuilder sb = new StringBuilder();
            if( velocity.getVelocity() > 0 )
            {
                sb.append( ConfigData.formatTime( velocity.getPreptime() ) ).append( "; " );
                sb.append( TotalPointDisplay.format( velocity.getAcceleration() ) ).append( "; " );
                sb.append( TotalPointDisplay.format( velocity.getVelocity() ) );
            }
            text = sb.toString();
        }
        return text;
    }

    /**
     * @return the activeAccount
     */
    public Account getActiveAccount()
    {
        return activeAccount;
    }

    public Account lookupAccount( String id )
    {
        Account rv = null;
        for( Account account : getAccounts() )
        {
            if( account.getId().equals( id ) )
            {
                rv = account;
                break;
            }
        }
        return rv;
    }

    public Version getVersion()
    {
        Version version = Version.retro;
        Account current = getActiveAccount();
        if( current != null )
        {
            version = current.getVersion();
        }
        return version;
    }

    public boolean isOutpost()
    {
        boolean isOutpost = false;
        Account current = getActiveAccount();
        if( current != null )
        {
            isOutpost = current.isOutpost();
        }
        return isOutpost;
    }

    public double getSpeedFactor(Class<?> kind) {
        double speedFactor = 1.0;
        Account current = getActiveAccount();
        if( current != null )
        {
            speedFactor = current.getSpeedFactor();
            // speedup in SI2 is done by ramping up utilization
            if(current.getVersion() != Version.v20) {
                speedFactor *= current.getSpeedUp();
            }
            if(kind != null && current.isSpectralMode()) {
                double decrease = 1.0;
                if(kind == Resource.class) {
                    decrease = 4.0;
                }
                else if (kind == Building.class) {
                    decrease = 2.0;
                }
                else if (kind == Ship.class || kind == Defense.class) {
                    decrease = 10.0;
                }
                speedFactor /= decrease;
            }
        }
        return speedFactor;
    }

    public double getUtilizationSpeedUp() {
        double speedUp = 1.0;
        Account account = getActiveAccount();
        if(account != null && account.getVersion() == Version.v20) {
            // speedup in SI2 is done by ramping up utilization
            speedUp = account.getSpeedUp();
        }
        return speedUp;
    }

    public boolean isSpectralMode()
    {
        boolean isSpectralMode = false;
        Account current = getActiveAccount();
        if( current != null )
        {
            isSpectralMode = current.isSpectralMode();
        }
        return isSpectralMode;
    }

    public Map<Position, Coordinates> getCoordinates()
    {
        return zeroPointMap;
    }

    /**
     * Map a position to absolute Coordinates
     * @param position
     * @return
     */
    public Coordinates getCoordinates( Position position )
    {
        Account current = getActiveAccount();
        return getCoordinates( position, current );
    }

    /**
     * Map a position to absolute Coordinates
     * @param position
     * @return
     */
    public Coordinates getCoordinates( Position position, Account current )
    {
        Coordinates coordinates = null;
        coordinates = zeroPointMap.get( position );
        if( coordinates != null )
        {
            coordinates = translate( coordinates, current.getGalaxyTicks(), current.getTicks() );
        }
        // approximate from solar system
        if( coordinates == null )
        {
            coordinates = getStarCoordinates( position );
        }
        return coordinates;
    }

    public LaFStyle getCurrentStyle()
    {
        return currentLaF;
    }

    public void setCurrentStyle( LaFStyle style )
    {
        currentLaF = style;
    }

    /**
     * @param activeAccount the activeAccount to set
     */
    public void setActiveAccount( Account activeAccount )
    {
        this.activeAccount = activeAccount;
        this.translationCache.clear();
    }

    public List<Account> getAccounts()
    {
        return accounts;
    }

    public void setAccounts( List<Account> accounts )
    {
        this.accounts.clear();
        this.translationCache.clear();
        for( Account account : accounts )
        {
            this.accounts.add( account );
            if( account.isDefault() )
            {
                activeAccount = account;
                defaultResources = activeAccount.getResourceBundle();
            }
        }
        if( !this.accounts.contains( activeAccount ) )
        {
            activeAccount = null;
        }
    }

    public void setVelocity(FleetSpeed velocity)
    {
        this.velocity = velocity;
    }

    public EntryComparator getEntryComparator()
    {
        return entryComparator;
    }

    public Planet getPlanetOnPosition( Position position )
    {
        Planet planet = new Planet( null, position );
        if( activeAccount != null )
        {
            Map<Planet, Planet> lookupPlanets = activeAccount.getLookupPlanets();

            // is it one of ours?
            if( lookupPlanets.containsKey( planet ) )
            {
                Planet myPlanet = lookupPlanets.get( planet );
                planet.setName( myPlanet.getName() );
                planet.setPosition( new Position( myPlanet.getPosition(), !position.isRealPosition() ) );
            }
            else
            {
                if( activeAccount.getVersion() == Version.v20 )
                {
                    Coordinates coordinates = getCoordinates().get( position );
                    // only add planets if in sync
                    if( activeAccount.isSync() && coordinates != null )
                    {
                        planet.setPosition( coordinates.getPosition() );
                        planet.setCoordinates( coordinates );
                    }
                    else
                    {
                        // create sun position
                        coordinates = getStarCoordinates( position );
                        planet.setPosition( coordinates.getPosition() );
                        planet.setCoordinates( coordinates );
                    }
                }
            }
        }
        return planet;
    }

    public List<Planet> getMyPlanets()
    {
        return getMyPlanets( true, false );
    }

    public List<Planet> getMyAsteroids()
    {
        return getMyPlanets( false, true );
    }

    public List<Planet> getMyPlanetsAndAsteroids()
    {
        return getMyPlanets( true, true );
    }

    public List<Planet> getMyPlanets( boolean addPlanets, boolean addAsteroids )
    {
        List<Planet> rv = new ArrayList<Planet>();
        List<Planet> planets = new ArrayList<Planet>();
        List<Planet> asteroids = new ArrayList<Planet>();

        if( activeAccount != null )
        {
            List<Planet> myPlanets = activeAccount.getMyPlanets();
            for( Planet planet : myPlanets )
            {
                if( planet.isMine() )
                {
                    planets.add( planet );
                    if( addAsteroids )
                    {
                        Planet asteroid = planet.makeAsteroid( this );
                        if( asteroid != null )
                        {
                            asteroids.add( asteroid );
                        }
                    }
                }
            }
        }
        if( addPlanets )
        {
            rv.addAll( planets );
        }
        if( addAsteroids )
        {
            rv.addAll( asteroids );
        }
        return rv;
    }

    public List<Planet> getPlanets()
    {
        if( activeAccount != null )
        {
            return activeAccount.getMyPlanets();
        }

        return new ArrayList<Planet>();
    }

    public void setPlanets( List<Planet> planets )
    {
        if( activeAccount != null )
        {
            List<Planet> myPlanets = activeAccount.getMyPlanets();

            myPlanets.clear();
            myPlanets.addAll( planets );
        }
    }

    public boolean hasPlanet( Planet planet )
    {
        if( activeAccount != null )
        {
            return activeAccount.getLookupPlanets().containsKey( planet );
        }

        return false;
    }

    public Planet getPlanet( Planet planet )
    {
        if( activeAccount != null )
        {
            return activeAccount.getLookupPlanets().get( planet );
        }

        return null;
    }

    /**
     * @return the myTech
     */
    public Technology getMyTech()
    {
        if( activeAccount != null )
        {
            return activeAccount.getMyTech();
        }
        return new Technology();
    }

    public Specialization getSpecialization()
    {
        if( activeAccount != null )
        {
            return activeAccount.getSpecialization();
        }
        return new Specialization();
    }

    /**
     * @param myTech the myTech to set
     */
    public void setMyTech( Technology myTech )
    {
        if( activeAccount != null )
        {
            activeAccount.setMyTech( myTech );
        }
    }

    public List<Coordinates> getTargets( Coordinates originCoordinates, long distance )
    {
        // find all planets/systems on the circle  with radius distance around originCoordinates
        List<Coordinates> rv = new ArrayList<Coordinates>();
        int ticks = getActiveAccount().getTicks();
        int galaxyTicks = getActiveAccount().getGalaxyTicks();
        for(int phi = 0; phi < 360; phi += 2)
        {
            double rad = phi * Math.PI / 180.0;
            int dx = (int)( distance * Math.cos( rad ) / Coordinates.scale + 0.5 );
            int dy = (int)( distance * Math.sin( rad ) / Coordinates.scale + 0.5 );
            int x = originCoordinates.getX() + dx;
            int y = originCoordinates.getY() + dy;
            int galaxy = -1;
            // this is the exact target spot, its most likely empty
            // find something in the vicinity
            Coordinates absoluteTarget = new Coordinates( x, y );
            Point galaxyPoint = absoluteTarget.getGalaxy();
            if( galaxyPoint.x > 0 && galaxyPoint.x <= Coordinates.sysScale
             && galaxyPoint.y > 0 && galaxyPoint.y <= Coordinates.sysScale )
            {
                // find the galaxy
                galaxy = Coordinates.getGalaxy( galaxyPoint, galaxyTicks );
            }
            if( galaxy != -1 )
            {
                int system = Coordinates.getSystem( galaxy, absoluteTarget.getSystem() );
                if( system != -1 )
                {
                    // now we have at least a target system, lets look at the planets
                    List<Coordinates> candidates = getKnownPlanetsOfSystem( galaxy, system );
                    List<Coordinates> targets = new ArrayList<Coordinates>();
                    for( Coordinates target : candidates )
                    {
                        // get current position
                        Coordinates real = translate( target, galaxyTicks, ticks );
                        long realdistance = originCoordinates.distance( real );
                        long diff = Math.abs( realdistance - distance );
                        // consider a match at less than 15% deviation
                        // consider in system matches, exclude origin
                        if( ( originCoordinates.getPosition().getPlanet() != real.getPosition().getPlanet() )
                         && ( ( 7 * diff / distance < 1 )
                         || ( realdistance < Coordinates.sysScale * Coordinates.scale / 2 ) ) )
                        {
                            if( !targets.contains( real ) )
                                targets.add( real );
                        }
                    }

                    // add at least the system itself
                    if( targets.size() == 0 )
                    {
                        Position position = new Position(galaxy, system, -1 );
                        galaxyPoint = Coordinates.getGalaxyPoint( position, galaxyTicks );
                        Point systemPoint = Coordinates.getSystemPoint( position );
                        Point planetPoint = new Point( 26, 26 );
                        Coordinates target = new Coordinates( galaxyPoint, systemPoint, planetPoint, -1 );
                        target.setPosition( position );
                        // exclude origin
                        if( !originCoordinates.getPosition().equals( position ) )
                        {
                            if( !rv.contains( target ) )
                                rv.add( target );
                        }
                    }
                    else
                    {
                        rv.addAll( targets );
                    }

                }
            }
        }
        return rv;
    }

    private List<Coordinates> getKnownPlanetsOfSystem(int galaxy, int system) {
        // this is just brute force, however, only add planets if in sync
        List<Coordinates> rv = new ArrayList<Coordinates>();
        Account account = getActiveAccount();
        if( account != null && account.isSync() )
        {
            Map<Position, Coordinates> map = getCoordinates();
            for(int planet = 1; planet <= Position.MAX_PLANETS; planet++ )
            {
                Position position = new Position(galaxy, system, planet);
                Coordinates target = map.get( position );
                if( target != null )
                {
                    rv.add( target );
                }
            }
        }
        return rv;
    }

    private Coordinates getStarCoordinates( Position position )
    {
        int galaxy = position.getGalaxy();
        int galaxyTicks = getActiveAccount().getGalaxyTicks();
        Point galaxyXy = Coordinates.getGalaxyPoint( position, galaxyTicks );
        Point systemXy = Coordinates.getSystemPoint( position );
        Point centerXy = new Point( Coordinates.sysScale / 2 + 1, Coordinates.sysScale / 2 + 1 );
        int orbit = -1;
        Coordinates coordinates = new Coordinates( galaxyXy, systemXy, centerXy, orbit );
        coordinates.setPosition( new Position( galaxy, position.getSystem(), 0 ) );
        return coordinates;
    }


    private Document readFile( File inFile, boolean pack )
    {
        Document doc = null;
        InputStream is = null;
        if( !inFile.exists() && inFile.getName().endsWith( ".gz" ) )
        {
            pack = false;
            String fileName = inFile.getAbsolutePath();
            inFile = new File( fileName.substring( 0, fileName.indexOf( ".gz" ) ) );
        }
        if( xmlParser != null && inFile.exists() )
        {
            try
            {
                is = new FileInputStream( inFile );
                if( pack )
                {
                    is = new GZIPInputStream( is, defaultBufferSize );
                }
                doc = xmlParser.getDocument( is );
                is.close();
            }
            catch( Exception ex )
            {
                ex.printStackTrace();
            }
            finally
            {
                if( is != null )
                {
                    try
                    {
                        is.close();
                    }
                    catch( IOException e )
                    {}
                }
            }
        }
        return doc;
    }

    private void saveFile( File outFile, Document doc, boolean pack )
    {
        if( xmlParser != null )
        {
            try
            {
                OutputStream os = new FileOutputStream( outFile );
                if( pack )
                {
                    os = new GZIPOutputStream( os, defaultBufferSize );
                }

                xmlParser.writeDocument( os, doc );

                if( pack )
                {
                    ((GZIPOutputStream)os).finish();
                }
                // close the stream
                os.close();
            }
            catch (FileNotFoundException e)
            {}
            catch (IOException e)
            {}
        }
    }

    private File findConfigFile( String name )
    {
        File file;
        // find a suitable config file
        // start with command line option
        String configDir = System.getProperty( "sitool.configdir" );
        file = new File( configDir + File.separator + name  );

        if( !file.exists() )
        {
            // try current dir (USB stick use case)
            configDir = System.getProperty( "user.dir" );
            file = new File( configDir + File.separator + name  );
        }

        if( !file.exists() )
        {
            // finally user home
            configDir = System.getProperty( "user.home" );
            file = new File( configDir + File.separator + name  );
        }
        System.out.print( "config file: " );
        System.out.println( file.getAbsolutePath() );
        return file;
    }

    public static LaFStyle parseLaFAttribute( String value )
    {
        LaFStyle rv = LaFStyle.defaultLaF;

        if( value != null && value.trim().length() > 0 )
        {
            try
            {
                rv = LaFStyle.valueOf( value );
            }
            catch( Exception e )
            {}
        }
        return rv;
    }

    public static boolean parseBooleanAttribute( String value )
    {
        boolean rv = false;

        if( value != null && value.trim().length() > 0 )
        {
            rv = Boolean.parseBoolean( value );
        }
        return rv;
    }

    public static int parseIntegerAttribute( String value )
    {
        int rv = 0;

        if( value != null && value.trim().length() > 0 )
        {
            try {
                rv = Integer.parseInt( value );
            }
            catch( NumberFormatException e)
            {}
        }
        return rv;
    }

    public static long parseLongAttribute( String value )
    {
        long rv = 0;

        if( value != null && value.trim().length() > 0 )
        {
            try {
                rv = Long.parseLong( value );
            }
            catch( NumberFormatException e)
            {}
        }
        return rv;
    }

    public static double parseSpeedAttribute( String value )
    {
        double rv = 1.0;

        if( value != null && value.trim().length() > 0 )
        {
            try {
                rv = Double.parseDouble( value );
            }
            catch( NumberFormatException e)
            {}
        }
        return rv;
    }

    public static Version parseVersionAttribute( String value )
    {
        Version rv = Version.retro;

        if( value != null && value.trim().length() > 0 )
        {
            try
            {
                rv = Version.valueOf( value );
            }
            catch( IllegalArgumentException ex )
            {}
        }
        return rv;
    }

    public static Research parseResearchAttribute( String value )
    {
        Research rv = null;

        if( value != null && value.trim().length() > 0 )
        {
            try
            {
                rv = Research.valueOf( value );
            }
            catch( IllegalArgumentException ex )
            {}
        }
        return rv;
    }

    public static Language parseLanguageAttribute( String value )
    {
        Language rv = Language.english;

        if( value != null && value.trim().length() > 0 )
        {
            try
            {
                rv = Language.valueOf( value );
            }
            catch( IllegalArgumentException ex )
            {
                // universal default is english
            }
        }
        return rv;
    }

    public static Icon getIcon( String path )
    {
        ImageIcon rv = null;
        URL	url = null;
        try
        {
            url = ConfigData.class.getResource( path );
        }
        catch( Exception ex )
        {}
        if( url != null )
        {
            rv = new ImageIcon( url, path );
        }
        return rv;
    }

    public static Icon getIcon(String path, Dimension size) {
        ImageIcon imageIcon = (ImageIcon)getIcon(path);
        if(imageIcon != null && size != null) {
            Image image = imageIcon.getImage(); // transform it
            imageIcon = new ImageIcon(image.getScaledInstance(size.width, size.height,  java.awt.Image.SCALE_SMOOTH));
        }
        return imageIcon;
    }

    public static String formatTime( long duration )
    {
        StringBuilder sb = new StringBuilder();

        if( duration >= Calculator.secondsPerHour )
        {
            long h = duration / Calculator.secondsPerHour;
            if( h < 10 )
            {
                sb.append( '0' );
            }
            sb.append( h );
            sb.append( ':' );
            duration %= Calculator.secondsPerHour;
        }
        if( duration >= Calculator.secondsPerMinute )
        {
            long m = duration / Calculator.secondsPerMinute;
            if( m < 10 )
            {
                sb.append( '0' );
            }
            sb.append( m );
            duration %= Calculator.secondsPerMinute;
        }
        else
        {
            sb.append( "00" );
        }
        sb.append( ':' );
        if( duration < 10 )
        {
            sb.append( '0' );
        }
        sb.append( duration );

        return sb.toString();
    }

    public File getAccountDir()
    {
        return accountDir;
    }

    public void setAccountDir( File accountDir )
    {
        this.accountDir = accountDir;
    }

    protected File getFile( ImportExportPane view, boolean save )
    {
        File rv = null;
        File currentDir = view.getCurrentDir();
        String extension = view.getExtension();
        MainFrame mainFrame = view.getMainFrame();

        JFileChooser chooser = new JFileChooser( currentDir );
        chooser.setAcceptAllFileFilterUsed( false );
        FileFilter filter = new SiToolFileFilter( "SI Tool files", extension );
        chooser.addChoosableFileFilter( filter );
        if( save )
        {
            chooser.setDialogTitle( getI18nText( "save", "SI Tool" ) );
        }
        else
        {
            chooser.setDialogTitle( getI18nText( "load", "SI Tool" ) );
        }
        chooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
        chooser.setMultiSelectionEnabled( false );
        int nOption;
        if( save )
        {
            chooser.setSelectedFile( new File( currentDir, view.getFileName() ) );
            nOption = chooser.showSaveDialog( mainFrame );
        }
        else
        {
            nOption = chooser.showOpenDialog( mainFrame );
        }
        if( nOption == JFileChooser.APPROVE_OPTION )
        {
            view.setCurrentDir( chooser.getCurrentDirectory().getAbsoluteFile() );
            if( save )
            {
                try
                {
                    String outputFileName = chooser.getSelectedFile().getAbsolutePath();
                    if( !outputFileName.endsWith( extension ) )
                    {
                        outputFileName += extension;
                    }
                    File outputFile = new File( outputFileName );
                    nOption = JOptionPane.YES_OPTION;
                    if( outputFile.exists() )
                    {
                        String fileExists = getI18nText( "file.exists" );
                        String confirm = getI18nText( "confirm.overwrite" );
                        nOption = JOptionPane.showConfirmDialog( mainFrame, fileExists, confirm, JOptionPane.YES_NO_OPTION );
                    }
                    if( nOption == JOptionPane.YES_OPTION )
                    {
                        rv = outputFile;
                    }
                }
                catch( Exception ex )
                {
                    JOptionPane pane = new JOptionPane( ex.getMessage(),
                                    JOptionPane.ERROR_MESSAGE,
                                    JOptionPane.DEFAULT_OPTION );
                    mainFrame.showModalBox( pane.createDialog( mainFrame, ex.getMessage() ) );

                }
            }
            else
            {
                rv = chooser.getSelectedFile().getAbsoluteFile();
            }
        }
        return rv;
    }

    //    // TODO remove
    //    private Map<Position, Coordinates> getTranslatedCoordinates( Account account )
    //    {
    //        translationCache.clear();
    //        Map<Position, Coordinates> map = getCoordinates( account );
    //        int galaxyTicks = account.getGalaxyTicks();
    //        int ticks = account.getTicks();
    //        for(int galaxy = 1; galaxy < 19; galaxy++) {
    //            for(int system = 1; system < 401; system++) {
    //                Coordinates base = map.get( new Position(galaxy, system, 1) );
    //                translate( base, galaxyTicks, ticks, true );
    //            }
    //        }
    //        return translationCache.size() > 0 ? translationCache : map;
    //    }
}
