/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.components;

import java.util.ArrayList;
import java.util.List;

public class Distance
{
    public final static int baseBetweenSystems = 800;
    public final static int incrementBetweenSystems = 50;
    public final static int incrementBetweenGalaxies = 5000;

    public final static int lowerPlanetBound = 1;
    public final static int lowerSystemBound = 1;
    public final static int lowerGalaxyBound = 1;

    public final static int lower = 0;
    public final static int upper = 1;

    public static int getUpperGalaxyBound(Version version) {
        return version.getUpperGalaxyBound();
    }

    public static int getUpperSystemBound(Version version) {
        return version.getUpperSystemBound();
    }

    public static int getUpperPlanetBound() {
        return 16;
    }

    public static long[] getDistance( Version version, Position a, Position b )
    {
        long[] rv;

        // special case TF & asteroid
        if( a.equals( b ) )
        {
            rv = new long[] { 50 };
        }
        else if( a.getGalaxy() != b.getGalaxy() )
        {
            rv = new long[] { Distance.incrementBetweenGalaxies * galaxyDistance( version, a, b ) };
        }
        else if( a.getSystem() != b.getSystem() )
        {
            rv = new long[2];
            rv[lower] = Distance.baseBetweenSystems + Distance.incrementBetweenSystems * Math.abs( a.getSystem() - b.getSystem() );
            rv[upper] = Distance.baseBetweenSystems + Distance.incrementBetweenSystems * ( getUpperSystemBound(version) - Math.abs( a.getSystem() - b.getSystem() ) );
        }
        else
        {
            rv = new long[1];
            PlanetData start = SystemData.getPlanetData( version, a );
            PlanetData end = SystemData.getPlanetData( version, b );
            // well up to here its simply math
            double d = Math.sqrt( ( end.getX() - start.getX() ) * ( end.getX() - start.getX() ) + ( end.getY() - start.getY() ) * ( end.getY() - start.getY() ) );
            rv[lower] = (long)( d );
            if( d > rv[lower] )
            {
                // however, fractions are always rounded up to next full
                rv[lower]++;
            }
            // but, hazzit says, there is another secret ingredient!
            rv[lower] += 200;
        }
        return rv;
    }

    public static List<Position>  getNearestPositions( Version version, Position source, PositionSet localSystem, int distance )
    {
        int delta;
        ArrayList<Position> list = new ArrayList<Position>();

        // in-system distances
        {
            // there are probably formulas to determine whether a point lies inside a ring
            // but there are only so many other planets
            for(Position target : localSystem)
            {
                if(source.equals(target)) {
                    continue;
                }
                delta = Math.abs( distance - (int)getDistance( version, source, target )[lower] );
                // permissible delta?
                if( delta < distance / 15 )
                {
                    list.add( target );
                }
            }

        }
        // add systems
        if( (distance - Distance.baseBetweenSystems) >= 0 )
        {
            delta = (distance - Distance.baseBetweenSystems) / Distance.incrementBetweenSystems;
            if( delta > 0 )
            {
                boolean exactMatch = ( (distance - Distance.baseBetweenSystems) % Distance.incrementBetweenSystems == 0 );
                int galaxy = source.getGalaxy();
                int system = source.getSystem();
                addPossiblePosition(list, version, galaxy, system, delta);
                if( !exactMatch )
                {
                    if(delta > 1) {
                        addPossiblePosition(list, version, galaxy, system, delta - 1);
                    }
                    addPossiblePosition(list, version, galaxy, system, delta + 1);
                }
            }
        }
        // add galaxies, if applicable
        if( (distance / Distance.incrementBetweenGalaxies > 0) && (distance % Distance.incrementBetweenGalaxies <= Distance.incrementBetweenSystems))
        {
            delta = distance / Distance.incrementBetweenGalaxies;
            int galaxy = source.getGalaxy();
            if( galaxy - delta >= Distance.lowerGalaxyBound )
            {
                list.add( new Position(galaxy - delta, 0, 0) );
            }
            if( galaxy + delta <= Distance.getUpperGalaxyBound(version) )
            {
                list.add( new Position(galaxy + delta, 0, 0) );
            }
        }

        return list;
    }

    private static void addPossiblePosition(List<Position> list, Version version, int galaxy, int system, int delta)
    {
        if( system - delta >= Distance.lowerSystemBound )
        {
            list.add( new Position(galaxy, system - delta, 0) );
        }
        // wrap around at most one galaxy
        else if( delta < Distance.getUpperSystemBound(version))
        {
            list.add( new Position(galaxy, Distance.getUpperSystemBound(version) + system - delta, 0) );
        }
        if( system + delta <= Distance.getUpperSystemBound(version) )
        {
            list.add( new Position(galaxy, system + delta, 0) );
        }
        // wrap around at most one galaxy
        else if( delta < Distance.getUpperSystemBound(version))
        {
            list.add( new Position(galaxy, system + delta - Distance.getUpperSystemBound(version), 0) );
        }
    }

    public static int galaxyDistance( Version version, Position a, Position b )
    {
        int rv = Math.abs( a.getGalaxy() - b.getGalaxy());
        // galaxies wrap around
        if( rv > getUpperGalaxyBound(version) / 2 )
        {
            rv = getUpperGalaxyBound(version) - rv;
        }
        return rv;
    }
}
