/****************************************
 *  COPYRIGHT (C) 2008, 2018
 *  Holger Graf
 ****************************************/
package sitool.components;

import java.util.HashMap;
import java.util.Map;

import sitool.ConfigData;



public class Planet
{
    public static final int astroTemp = -20;
    private String name;
    private boolean mine;
    private int size;
    private int astroSize;
    private Position position;
    private Map<Building, Stage> buildings = new HashMap<Building, Stage>();
    private Map<Building, Stage> astroBuildings = new HashMap<Building, Stage>();
    private Fleet fleet = new Fleet();
    private Fleet astroFleet = new Fleet();
    private Coordinates coordinates;

    public Planet(String name, Position position)
    {
        super();
        this.name = name;
        this.position = position;
    }

    public String getName()
    {
        return name;
    }

    /**
     * @param name the name to set
     */
    public void setName( String name )
    {
        this.name = name;
    }

    public Position getPosition()
    {
        return position;
    }

    /**
     * @param position the position to set
     */
    public void setPosition( Position position )
    {
        this.position = position;
    }

    /**
     * @return the mine
     */
    public boolean isMine()
    {
        return mine;
    }

    /**
     * @param mine the mine to set
     */
    public void setMine( boolean mine )
    {
        this.mine = mine;
    }

    public int getTemperature(Version version)
    {
        int temperature = 0;
        if( position.isRealPosition() )
        {
            if( version != Version.v20 )
            {
                temperature = ( position.isAsteroidPosition() ) ? astroTemp : SystemData.getTemperature( version, position );
            }
            else if( coordinates.isValid() )
            {
                temperature = SystemData.getTemperature( coordinates );
            }
        }
        return temperature;
    }

    public int getSize()
    {
        return size;
    }

    public void setSize( int size )
    {
        this.size = size;
    }

    public int getAstroSize()
    {
        return astroSize;
    }

    public void setAstroSize( int astroSize )
    {
        this.astroSize = astroSize;
    }

    public Coordinates getCoordinates()
    {
        return coordinates;
    }

    public void setCoordinates( Coordinates coordinates )
    {
        this.coordinates = coordinates;
    }

    public int getUsedFields()
    {
        int rv = 0;
        for( Stage stage : buildings.values() )
        {
            rv += stage.getLevel();
        }
        return rv;
    }

    public int getUsedAstroFields()
    {
        int rv = 0;
        for( Stage stage : astroBuildings.values() )
        {
            rv += stage.getLevel();
        }
        return rv;
    }

    /**
     *
     * @param building
     * @param stage
     * @param astro
     * @throws NullPointerException if the building map is unset
     */
    public void addBuilding( Building building, Stage stage, boolean astro )
    {
        if( astro )
            astroBuildings.put( building, stage );
        else
            buildings.put( building, stage );
    }

    /**
     *
     * @param building
     * @param astro
     * @throws NullPointerException if the building map is unset
     */
    public void removeBuilding( Building building, boolean astro )
    {
        if( astro )
            astroBuildings.remove( building );
        else
            buildings.remove( building );
    }

    public void resetBuildings( boolean isAstro ) {
        if(isAstro)
            astroBuildings = null;
        else
            buildings = null;
    }

    public Stage getStage( Building building, boolean astro )
    {
        Stage rv;
        if( astro )
            rv = astroBuildings.get( building);
        else
            rv = buildings.get( building );
        return rv;
    }

    public Map<Building, Stage> getBuildings( boolean astro )
    {
        Map<Building, Stage> rv;
        if( astro )
            rv = astroBuildings;
        else
            rv = buildings;
        return rv;
    }

    public Fleet getFleet()
    {
        return getFleet( getPosition().isAsteroidPosition() );
    }

    public Fleet getFleet( boolean isAstro )
    {
        Fleet rv;
        if( isAstro )
        {
            rv = astroFleet;
        }
        else
        {
            rv = fleet;
        }
        return rv;
    }

    public void resetFleet( boolean isAstro ) {
        if( isAstro)
            astroFleet = null;
        else
            fleet = null;
    }

    /**
     * make an asteroid on this planets position if the planet may have one
     * @param configData
     * @return
     */
    public Planet makeAsteroid( ConfigData configData )
    {
        Planet rv = null;
        if( getAstroSize() > 0 )
        {
            Position aPosition = new Position( this.position, false );
            aPosition.setAsteroidPosition( true );
            String name = configData.getI18nText( "Asteroid" );
            rv = new Planet( name, aPosition );
            // add all references to the asteroid, too
            // so we can get at them regardless
            rv.astroBuildings = this.astroBuildings;
            rv.buildings = this.buildings;
            rv.astroSize = this.astroSize;
            rv.astroFleet = astroFleet;
            rv.fleet = this.fleet;
        }
        return rv;
    }

    /**
     * Update this planet with data from the given one.
     *
     * @param planet
     */
    public void update(Planet planet) {
        // short-circuit validation, just assign the values
        this.name = planet.name;
        this.size = planet.size;
        this.astroSize = planet.astroSize;
        if(planet.buildings != null) {
            this.buildings.clear();
            this.buildings.putAll(planet.buildings);
        }
        if(planet.astroBuildings != null) {
            this.astroBuildings.clear();
            this.astroBuildings.putAll(planet.astroBuildings);
        }
        if(planet.fleet != null) {
            if(planet.fleet.hasShips()) {
                this.fleet.setShips(planet.fleet.getShips());
            }
            if(planet.fleet.hasDefenses()) {
                this.fleet.setDefenses(planet.fleet.getDefenses());
            }
        }
        if(planet.astroFleet != null) {
            if(planet.astroFleet.hasShips()) {
                this.astroFleet.setShips(planet.astroFleet.getShips());
            }
            if(planet.astroFleet.hasDefenses()) {
                this.astroFleet.setDefenses(planet.astroFleet.getDefenses());
            }
        }
    }

    /**
     * for two planets to be equal it is sufficient that they occupy the same point in space
     */
    @Override
    public boolean equals( Object obj )
    {
        boolean rv = false;
        if( obj instanceof Planet )
        {
            rv = this.position.equals( ((Planet)obj).getPosition() );
        }
        return rv;
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode()
    {
        return position.hashCode();
    }

    @Override
    public String toString()
    {
        StringBuilder sb = new StringBuilder();

        sb.append( position.toString() );
        if( name != null )
        {
            sb.append( ' ' );
            sb.append( name );
        }
        return sb.toString();
    }

}
