/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.components;

import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class Position implements Comparable<Position>
{
    public static final int MAX_PLANETS = 32;
    // '0' == 'x'
    private int galaxy;
    private int system;
    private int planet;

    // when one of our planet positions acts a a place holder for a generic position
    // we still want to know it was generic
    private boolean explicitelyUnreal = false;
    // mark asteroid positions different from their planets
    private boolean asteroidPosition = false;

    private static Pattern pattern = Pattern.compile( "(\\d+):([\\d+|x]+):([\\d+|x]+)" );

    public Position( String position )
    {
        Matcher m = pattern.matcher( position );
        if( m.find() )
        {
            int galaxy = Integer.parseInt( m.group( 1 ) );
            int system = ( "x".equals( m.group( 2 ) ) ) ? 0 : Integer.parseInt( m.group( 2 ) );
            int planet = ( "x".equals( m.group( 3 ) ) ) ? 0 : Integer.parseInt( m.group( 3 ) );
            init( galaxy, system, planet, false );
        }
    }

    public Position( int galaxy, int system, int planet )
    {
        this( galaxy, system, planet, false );
    }

    public Position( int galaxy, int system, int planet, boolean markUnreal )
    {
        init( galaxy, system, planet, markUnreal );
    }

    protected void init( int galaxy, int system, int planet, boolean markUnreal )
    {
        this.galaxy = galaxy;
        this.system = system;
        this.planet = planet;
        explicitelyUnreal = markUnreal;
    }

    public Position( Position that, boolean markUnreal )
    {
        this(that.galaxy, that.system, that.planet, markUnreal);
    }

    public int getGalaxy()
    {
        return galaxy;
    }

    public int getSystem()
    {
        return system;
    }

    public int getPlanet()
    {
        return planet;
    }

    public boolean isAsteroidPosition()
    {
        return asteroidPosition;
    }

    public void setAsteroidPosition( boolean asteroidPosition )
    {
        this.asteroidPosition = asteroidPosition;
    }

    public boolean isRealPosition()
    {
        boolean rv;

        if( explicitelyUnreal )
        {
            rv = false;
        }
        else
        {
            rv = ( galaxy >= Distance.lowerGalaxyBound );
            rv &= ( system >= Distance.lowerSystemBound );
            rv &= ( planet >= Distance.lowerPlanetBound );
        }
        return rv;
    }

    @Override
    public int compareTo( Position o ) {
        long id0 = galaxy * 1_000_000 + system * 1000 + planet;
        long id1 = o.galaxy * 1_000_000 + o.system * 1000 + o.planet;
        return Long.signum(id0 - id1);
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals( Object obj )
    {
        boolean rv = false;

        if( obj instanceof Position )
        {
            Position that = ( Position )obj;
            rv = ( this.asteroidPosition == that.asteroidPosition ) &&
                     ( this.galaxy == that.galaxy ) &&
                     ( this.system == that.system );
            // treat 'x' as wildcard
            rv &= ( this.planet == that.planet || this.planet == 0 || that.planet == 0 );
        }
        return rv;
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode()
    {
        int rv = ( ( galaxy & 0xF ) << 16 ) + ( system & 0x1FF );
        return rv;
    }

    @Override
    public String toString()
    {
        StringBuilder sb = new StringBuilder();
        if( galaxy == 0 && system == 0 && planet == 0 )
        {
            return sb.toString();
        }
        sb.append( '[' );
        sb.append( pad( galaxy ) );
        sb.append( ':' );
        sb.append( pad( system ) );
        sb.append( ':' );
        sb.append( pad( planet ) );
        sb.append( ']' );
        return sb.toString();
    }

    static String pad( int value )
    {
        StringBuilder sb = new StringBuilder();
        if( value > 0 )
        {
            if( value < 10 )
            {
                sb.append( '0' );
            }
            sb.append( value );
        }
        else
        {
            sb.append( 'x' );
        }
        return sb.toString();
    }
}
