/****************************************
 *  COPYRIGHT (C) 2021
 *  Holger Graf
 ****************************************/
package sitool.datatransfer;


import java.awt.Component;
import java.io.IOException;
import java.util.Date;
import java.util.List;

import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;

import com.fasterxml.jackson.jr.ob.JSON;

import datatransfer.RequestType;
import datatransfer.json.InfoBean;
import datatransfer.json.ResponseBean;
import sitool.AboutBox;
import sitool.Account;
import sitool.MainFrame;
import sitool.components.Fleet;
import sitool.components.Planet;
import sitool.components.Specialization;
import sitool.components.Technology;
import sitool.components.Version;
import sitool.fleet.FleetPane;
import sitool.json.IntergalacticOverviewBean;
import sitool.json.SpyReportBean;
import sitool.planets.PlanetPane;
import sitool.resources.BuildingSummaryPane;
import sitool.resources.ResourcePane;
import sitool.resources.ResourceSummaryPane;
import sitool.technology.TechPane;

/**
 * @author graf
 *
 */
public class DataServer extends datatransfer.DataServer {


    private final MainFrame parent;

    public DataServer(final MainFrame parent) {
        this.parent = parent;
    }


    @Override
    protected Integer getAccountVersionID() {
        Account account = parent.getConfigData().getActiveAccount();
        return account != null ? account.getVersion().getId() : null;
    }


    @Override
    protected String getIdentifier() {
        return AboutBox.getIdentifier();
    }


    @Override
    protected String getProductName() {
        return AboutBox.getProductName();
    }


    @Override
    protected String getProductVersion() {
        return AboutBox.getVersion();
    }


    @Override
    protected void updateResponseBean(ResponseBean responseBean, RequestType type, String json) throws IOException  {
        // call base to process reequest types we are not interested in
        super.updateResponseBean(responseBean, type, json);
        switch(type) {
            case planetOverview:
            {
                final IntergalacticOverviewBean overview = JSON.std.beanFrom(IntergalacticOverviewBean.class, json);
                // first check, does the universe type match?
                if(checkAccess(overview.getInfo()) && isIntergalacticOverviewImportAllowed()) {
                    SwingUtilities.invokeLater(new Runnable() {
                        @Override
                        public void run() {
                            importFromIntergalacticOverview(overview);
                        }
                    });
                }
            }
            break;
            case spyreport:
                {
                    final SpyReportBean reportBean = JSON.std.beanFrom(SpyReportBean.class, json);
                    // first check, does the universe type match?
                    if(checkAccess(reportBean.getInfo()) && isSpyReportImportAllowed()) {
                        SwingUtilities.invokeLater(new Runnable() {
                            @Override
                            public void run() {
                                importFromSpyReport(reportBean);
                            }
                        });
                    }
                }
                break;
            default:
                break;
        }
    }

    private boolean isIntergalacticOverviewImportAllowed() {
        Account account = parent.getConfigData().getActiveAccount();
        return (account != null && account.isMayImportPlanetaryOverview());
    }

    private boolean isSpyReportImportAllowed() {
        Account account = parent.getConfigData().getActiveAccount();
        return (account != null && account.isMayImportSpyReports());
    }

    private void importFromIntergalacticOverview( IntergalacticOverviewBean overview ) {
        // import from intergalactic planet overview is only possible from planet pane and resource pane
        JTabbedPane outer = parent.getTabbedPane();
        Component c = outer.getSelectedComponent();
        boolean isOutpost = parent.getConfigData().isOutpost();
        Version version = parent.getConfigData().getVersion();
        if( c instanceof PlanetPane) {
            final List<Planet> planets = DataImport.createPlanetsFromJSON(version, overview, isOutpost,true, true);
            Technology newTech = DataImport.createTechnologyFromJSON(overview.getResearches());
            Specialization specialization = DataImport.createSpecializationFromJSON(overview.getSpecs());
            Fleet totalShips = DataImport.importTotalShipsFromJSON(overview.getShips(), isOutpost);
            ((PlanetPane)c).updateConfiguration(planets, newTech, specialization, totalShips);
        }
        else if( c instanceof FleetPane ) {
            PlanetPane planetPane = (PlanetPane)outer.getComponentAt(0);
            final List<Planet> planets = DataImport.createPlanetsFromJSON(version, overview, isOutpost, false, true);
            Technology newTech = DataImport.createTechnologyFromJSON(overview.getResearches());
            Specialization specialization = DataImport.createSpecializationFromJSON(overview.getSpecs());
            Fleet totalShips = DataImport.importTotalShipsFromJSON(overview.getShips(), isOutpost);
            planetPane.updateConfiguration(planets, newTech, specialization, totalShips);
            ((FleetPane)c).update();
        }
        else if( c instanceof TechPane) {
            Technology tech = DataImport.createTechnologyFromJSON(overview.getResearches());
            Account account = parent.getConfigData().getActiveAccount();
            DataImport.updateConfiguration(account, null, tech, null, null);
            ((TechPane)c).update();
        }
        else if( c instanceof BuildingSummaryPane ) {
            PlanetPane planetPane = (PlanetPane)outer.getComponentAt(0);
            final List<Planet> planets = DataImport.createPlanetsFromJSON(version, overview, isOutpost, true, false);
            Technology newTech = DataImport.createTechnologyFromJSON(overview.getResearches());
            Specialization specialization = DataImport.createSpecializationFromJSON(overview.getSpecs());
            planetPane.updateConfiguration(planets, newTech, specialization, null);
            ((BuildingSummaryPane)c).update();
        }
        else if( c instanceof ResourceSummaryPane )
        {
            ResourceSummaryPane summary = ( ResourceSummaryPane )c;
            ResourcePane resourcePane = summary.addTab(getDate(overview.getInfo()));
            DataImport.importResourcesFromJSON(resourcePane, overview);
        }

    }

    private Date getDate( InfoBean info ) {
        return new Date(info.getTs());
    }


    private void importFromSpyReport( SpyReportBean reportBean ) {
        // import from spy report is possible from planet pane, fleet pane and resource pane
        JTabbedPane outer = parent.getTabbedPane();
        Component c = outer.getSelectedComponent();
        if( c instanceof PlanetPane) {
            final Planet planet = DataImport.createPlanetFromJSON(parent.getConfigData().getVersion(), reportBean, true, true);
            final Technology newTech = DataImport.createTechnologyFromJSON(reportBean.getPlanet().getResearches());
            ((PlanetPane)c).addPlanet(planet, newTech);
        }
        else if( c instanceof FleetPane ) {
            PlanetPane planetPane = (PlanetPane)outer.getComponentAt(0);
            final Planet planet = DataImport.createPlanetFromJSON(parent.getConfigData().getVersion(), reportBean, false, true);
            final Technology newTech = DataImport.createTechnologyFromJSON(reportBean.getPlanet().getResearches());
            planetPane.addPlanet(planet, newTech);
            ((FleetPane)c).update();
        }
        else if( c instanceof BuildingSummaryPane ) {
            PlanetPane planetPane = (PlanetPane)outer.getComponentAt(0);
            final Planet planet = DataImport.createPlanetFromJSON(parent.getConfigData().getVersion(), reportBean, true, false);
            final Technology newTech = DataImport.createTechnologyFromJSON(reportBean.getPlanet().getResearches());
            planetPane.addPlanet(planet, newTech);
            ((BuildingSummaryPane)c).update();
        }
        else if( c instanceof ResourceSummaryPane )
        {
            ResourceSummaryPane summary = ( ResourceSummaryPane )c;
            String title = summary.getTitle(new Date(reportBean.getTimestamp() * 1000));
            JTabbedPane inner = summary.getTabbedPane();
            int index = inner.getSelectedIndex();
            if(index >= 0) {
                inner.setTitleAt( index, title );
                ResourcePane resourcePane = (ResourcePane)inner.getComponentAt(index);
                DataImport.importResourcesFromJSON(resourcePane, reportBean);
                resourcePane.update();
            }
        }
    }
}
