/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool.resources;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;

import com.fasterxml.jackson.jr.ob.JSON;

import sitool.ConfigData;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.Resource;
import sitool.components.Version;
import sitool.datatransfer.DataImport;
import sitool.json.IntergalacticOverviewBean;
import sitool.json.JSONTags;
import sitool.json.SpyReportBean;
import sitool.table.StockpileTableModel;

/**
 *
 * @author graf
 *
 */
public class ResourceBox extends JDialog implements ActionListener
{
    private static final long serialVersionUID = -4586875011135823902L;

    private JTextArea textArea = new JTextArea();

    private final ResourcePane resourcePane;
    private final StockpileTableModel stockPileModel;

    // these Scanboxes are all pretty much the same, they just need a different scanner...
    public ResourceBox( ResourcePane parent, StockpileTableModel stockPileModel )
    {
	    super( parent.getMainFrame() );
	    this.resourcePane = parent;
        this.stockPileModel = stockPileModel;
        ConfigData configData = resourcePane.getMainFrame().getConfigData();

        enableEvents( AWTEvent.WINDOW_EVENT_MASK );
        this.setTitle( configData.getI18nText( "ResourceBox.title" ) );
        JPanel contentPane = ( JPanel )getContentPane();
        contentPane.setLayout( new BorderLayout() );
        contentPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel center = createCenterPanel();
        center.setPreferredSize( new Dimension( 640, 520 ) );

        JPanel lower = new JPanel( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
        JButton button = new JButton( configData.getI18nText( "OK" ) );
        button.addActionListener( this );
        lower.add( button );
        getRootPane().setDefaultButton( button );

        contentPane.add( center, BorderLayout.CENTER );
        contentPane.add( lower, BorderLayout.SOUTH );
        pack();

        // by popular demand, add paste clipboard to RMB
        MouseAdapter l = new MouseAdapter()
        {

            @Override
            public void mouseClicked( MouseEvent e )
            {
                if( SwingUtilities.isRightMouseButton( e ) )
                {
                    String input = "";
                    Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
                    Transferable contents = clipboard.getContents( null );
                    boolean hasTransferableText = ( ( contents != null )
                                    && contents.isDataFlavorSupported( DataFlavor.stringFlavor ) );
                    if( hasTransferableText )
                    {
                        try
                        {
                            input = ( String )contents.getTransferData( DataFlavor.stringFlavor );
                        }
                        catch( UnsupportedFlavorException ex )
                        {
                            System.out.println( ex );
                            ex.printStackTrace();
                        }
                        catch( IOException ex )
                        {
                            System.out.println( ex );
                            ex.printStackTrace();
                        }
                    }
                    textArea.setText( input );
                }
            }

        };

        textArea.addMouseListener( l );
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );
        center.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10) );

        JScrollPane scrollPane = new JScrollPane( textArea,
                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                        JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        center.add( scrollPane, BorderLayout.CENTER );

        return center;
    }

    // Overridden so we can exit when window is closed
    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    protected void processWindowEvent( WindowEvent e )
    {
        if( e.getID() == WindowEvent.WINDOW_CLOSING )
        {
            cancel();
        }
        super.processWindowEvent( e );
    }

    // Close the dialog

    /***************************************************************************
     * Description of the Method
     **************************************************************************/
    void cancel()
    {
        dispose();
    }

    // Close the dialog on a button event

    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    public void actionPerformed( ActionEvent e )
    {
        String input = textArea.getText();

        Date updated = null;
        ConfigData configData = resourcePane.getMainFrame().getConfigData();
        Version version = configData.getVersion();
        if( version != Version.v20 )
        {
            if( input.indexOf(JSONTags.reportType_SpyReport) > -1 ) {
                updated = scanSpyReportJSON(input);
            }
            else if( input.indexOf(JSONTags.reportType_PlanetOverview) > -1 ) {
                updated = scanIntergalacticPlanetOverviewJSON(input);
            }
            else if( input.indexOf( configData.getI18nText( "PlanetBox.spyMarker" ) ) > -1 )
            {
                updated = scanSpyReport( input );
            }
            else
            {
                updated = scanIntergalacticPlanetOverview( input );
            }
        }
        else
        {
            if( input.indexOf( configData.getI18nText( "PlanetBox.spyMarker2" ) ) > -1 )
            {
                updated = scanSpyReport2( input );
            }
            else
            {
                updated = scanIntergalacticPlanetOverview2( input );
            }
        }


        if( updated != null )
        {
            stockPileModel.fireTableDataChanged();
            JTabbedPane outer = resourcePane.getMainFrame().getTabbedPane();
            Component c = outer.getSelectedComponent();
            if( c instanceof ResourceSummaryPane )
            {
                ResourceSummaryPane summary = ( ResourceSummaryPane )c;
                String title = summary.getTitle(updated);
                JTabbedPane inner = summary.getTabbedPane();
                int index = inner.getSelectedIndex();
                inner.setTitleAt( index, title );
            }
        }

        dispose();
    }

    private Date scanSpyReport( String input )
    {
        boolean updated = false;
        ConfigData configData = resourcePane.getMainFrame().getConfigData();

        try
        {
            List<Planet> myPlanetsAndAsteroids = configData.getMyPlanetsAndAsteroids();

            Pattern basePattern = Pattern.compile( ".*" + configData.getI18nText( "PlanetBox.spyMarker" ) + "\\s+(\\S+.*)\\[(\\d+):(\\d+):(\\d+)\\s*\\](.*)", Pattern.DOTALL );
            Matcher m = basePattern.matcher( input );
            if( m.matches() )
            {
                // get planet
                String name = m.group( 1 ).trim();
                int gal = Integer.parseInt( m.group( 2 ) );
                int sys = Integer.parseInt( m.group( 3 ) );
                int pl = Integer.parseInt( m.group( 4 ) );
                Position position = new Position( gal, sys, pl );
                boolean astro = configData.getI18nText( "Asteroid" ).equals( name );
                position.setAsteroidPosition( astro );
                Planet planet = new Planet( name, position );

                input = m.group( 5 );
                BufferedReader br = new BufferedReader( new StringReader( input ) );
                StringBuilder sb = new StringBuilder();
                sb.append( "\\s*" );
                for( Resource resource : Resource.values() )
                {
                    sb.append( configData.getI18nText( resource.name() ).toLowerCase() );
                    sb.append( "\\s+" );
                }
                sb.append( configData.getI18nText( "Energy" ).toLowerCase() );
                sb.append( ".*" );
                Pattern resourcePattern = Pattern.compile( sb.toString() );
                String line;
                while( ( line = br.readLine() ) != null )
                {
                    m = resourcePattern.matcher( line.toLowerCase() );
                    if( m.matches() )
                    {
                        line = br.readLine();
                        Pattern numPattern = Pattern.compile( "\\s*(\\S+)\\s+(\\S+)\\s+(\\S+)\\s+(\\S+)\\s+.*" );
                        m = numPattern.matcher( line );
                        if( m.matches() )
                        {
                            int row = myPlanetsAndAsteroids.indexOf( planet );
                            for( int index = 1; index <= 4; index++ )
                            {
                                stockPileModel.setValueAt( m.group( index ), row, 2 * index - 1 );
                            }
                            updated = !astro;
                            break;
                        }
                    }
                }

                br.close();
            }
        }
        catch( Exception e1 ) {
            e1.printStackTrace();
        }
        return updated ? new Date() : null;
    }

    private Date scanSpyReport2( String input )
    {
        boolean updated = false;
        ConfigData configData = resourcePane.getMainFrame().getConfigData();

        BufferedReader br = new BufferedReader( new StringReader( input ) );
        try
        {
            Pattern pattern = Pattern.compile( "=>\\s+(\\d+):(\\d+):(\\d+)" );
            String line;
            Matcher m;
            while( ( line = br.readLine() ) != null )
            {
                m = pattern.matcher( line );
                if( !m.find() )
                {
                    continue;
                }
                int gal = Integer.parseInt( m.group( 1 ) );
                int sys = Integer.parseInt( m.group( 2 ) );
                int pl = Integer.parseInt( m.group( 3 ) );
                Position position = new Position( gal, sys, pl );

                List<Planet> myPlanetsAndAsteroids = configData.getMyPlanetsAndAsteroids();
                Planet planet = new Planet( "", position );
                int row = myPlanetsAndAsteroids.indexOf( planet );
                if( row > -1 )
                {
                    int index = 0;
                    for( Resource resource : Resource.values() )
                    {
                        pattern = Pattern.compile( configData.getI18nText( resource.name() ) + "\\s+(\\d+\\S*\\d?)" );
                        while( ( line = br.readLine() ) != null )
                        {
                            m = pattern.matcher( line );
                            if( !m.find() )
                            {
                                continue;
                            }
                            stockPileModel.setValueAt( m.group( 1 ), row, 2 * index + 1 );
                            index++;
                            break;
                        }
                    }
                    updated = true;
                }
            }
            br.close();
        }
        catch( Exception e1 ) {
            e1.printStackTrace();
        }

        return updated ? new Date() : null;
    }

    private Date scanSpyReportJSON( String input ) {
        Date updated = null;
        try
        {
            SpyReportBean reportBean = JSON.std.beanFrom(SpyReportBean.class, input);
            updated = DataImport.importResourcesFromJSON(resourcePane, reportBean);
        }
        catch( Exception e1 ) {
            e1.printStackTrace();
        }
        return updated;
    }

    private Date scanIntergalacticPlanetOverview( String input )
    {
        BufferedReader br = new BufferedReader( new StringReader( input ) );
        String line;
        boolean updated = false;
        try
        {
            List<Planet> myPlanets = resourcePane.getMainFrame().getConfigData().getMyPlanets();
            HashMap<String, Planet> keys = new HashMap<String, Planet>();
            for( Planet planet : myPlanets )
            {
                keys.put( planet.getPosition().toString(), planet );
            }
            Planet planet;
            line = br.readLine();
            while( line != null )
            {
                // check for coords
                final String key = "[" + line.trim() + "]";
                // a resource line starts with the position
                if( keys.containsKey( key ) )
                {
                    planet = keys.get( key );
                    // row is offset by 1
                    int row = myPlanets.indexOf( planet ) + 1;
                    int column;
                    // values follow on subsequent lines
                    line = br.readLine();
                    if( line != null )
                    {
                        // iron
                        column = 1;
                        stockPileModel.setValueAt( line.trim(), row, column );

                        line = br.readLine();
                        if( line != null )
                        {
                            // metal
                            column = 3;
                            stockPileModel.setValueAt( line.trim(), row, column );

                            line = br.readLine();
                            if( line != null )
                            {
                                // kryptonite
                                column = 5;
                                stockPileModel.setValueAt( line.trim(), row, column );

                                line = br.readLine();
                                if( line != null )
                                {
                                    // spice
                                    column = 7;
                                    stockPileModel.setValueAt( line.trim(), row, column );
                                    updated = true;

                                    line = br.readLine();
                                }
                            }
                        }
                    }
                }
                else
                {
                    line = br.readLine();
                }
            }
            br.close();
        }
        catch( Exception e1 ) {
            e1.printStackTrace();
        }
        return updated ? new Date() : null;
    }

    private Date scanIntergalacticPlanetOverview2( String input )
    {
        boolean updated = false;
        BufferedReader br = new BufferedReader( new StringReader( input ) );
        try
        {
            StringBuilder patternString = new StringBuilder();
            patternString.append( "\\[(\\d+):(\\d+):(\\d+)\\]" );
            patternString.append( ".+?\\s+" );
            patternString.append( "(\\d+\\S*\\d?)" ).append( "\\s+" );
            patternString.append( "(\\d+\\S*\\d?)" ).append( "\\s+" );
            patternString.append( "(\\d+\\S*\\d?)" ).append( "\\s+" );
            patternString.append( "(\\d+\\S*\\d?)$" );
            Pattern pattern = Pattern.compile( patternString.toString() );

            List<Planet> myPlanets = resourcePane.getMainFrame().getConfigData().getMyPlanets();

            String line;
            Matcher m;
            while( ( line = br.readLine() ) != null )
            {
                m = pattern.matcher( line.trim() );
                if( !m.find() )
                {
                    continue;
                }
                int gal = Integer.parseInt( m.group( 1 ) );
                int sys = Integer.parseInt( m.group( 2 ) );
                int pl = Integer.parseInt( m.group( 3 ) );
                Position position = new Position( gal, sys, pl );

                Planet planet = new Planet( "", position );
                // row is offset by 1
                int row = myPlanets.indexOf( planet ) + 1;
                if( row > 0 )
                {
                    for( int index = 0; index < 4; index++ )
                    {
                        stockPileModel.setValueAt( m.group( index + 4 ), row, 2 * index + 1 );
                    }
                    updated = true;
                }
            }
            br.close();
        }
        catch( Exception e1 ) {
            e1.printStackTrace();
        }

        return updated ? new Date() : null;
    }

    private Date scanIntergalacticPlanetOverviewJSON( String input ) {
        Date updated = null;
        try
        {
            IntergalacticOverviewBean overview = JSON.std.beanFrom(IntergalacticOverviewBean.class, input);
            updated = DataImport.importResourcesFromJSON(resourcePane, overview);
        }
        catch( Exception e1 ) {
            e1.printStackTrace();
        }
        return updated;
    }
}
