/****************************************
 *  COPYRIGHT (C) 2008-2017
 *  Holger Graf
 ****************************************/
package sitool.resources;

import sitool.components.Building;
import sitool.components.Cost;
import sitool.components.Version;

public class ResourceCalculator
{
    /**
     * Returns build time in ms
     * @param cost
     * @return
     */
    public static double buildTime( Cost cost, boolean isConstructorEnabled, Version version)
    {
        double rv = 0;

        rv += 15000.0 * cost.getIron();       //  5/6
        rv += 15000.0 * cost.getMetal();      //  5/6
        rv += 20000.0 * cost.getKryptonite(); // 10/9
        // spice doesn't count
        rv /= 18.0;
        if( isConstructorEnabled )
        {
            //int factor = (version == Version.v20) ? 90 : 95;
            int factor = 90;
            rv = factor * rv / 100;
        }
        rv = Math.floor( rv );
        return rv;
    }

    public static double calculateProduction( Building building, int level, int temperature, Version version )
    {
        double base = 0.0;
        double rv = 0;

        switch( building )
        {
            case ironMine:
                base = 24.0;
                break;
            case furnace:
                base = 10.5;
                break;
            case kryptoniteMine:
                base = 12.25;
                break;
            case spiceMine:
                if( version == Version.v20 )
                {
                    if( temperature > 300 ) temperature = 300;
                }
                base = (6.0 - temperature / 80.0);
                break;
            case fusionPlant:
                base = 10.0;
                if( version == Version.v20) base *= 0.9;
                break;
            case thermosolarPlant:
                base = 8.0;
                if( version == Version.v20) base *= 1.75;
                break;
            default:
                base = 0;
        }

        if( base != 0 )
        {
            rv = base * level * Math.pow( 1.15, level );
        }
        return rv;
    }

    public static double calculateConsumption( Building building, int level )
    {
        double base = 0.0;
        double rv = 0;

        switch( building )
        {
            case furnace:
                base = 21;
                break;
            case thermosolarPlant:
                base = 2.0;
                break;
            default:
                base = 0;
        }

        if( base != 0 )
        {
            rv = base * level * Math.pow( 1.15, level );
        }
        return rv;
    }

    public static long calculateCapacity( Building building, int level )
    {
        double base;
        long rv = 0;

        switch( building )
        {
            case ironStorage:
            case metalStorage:
            case kryptoniteStorage:
            case spiceStorage:
                base = 2.0;
                break;
            default:
                base = 0.0;
                break;
        }
        if( base != 0 )
        {
            rv = ( long ) ( 100000 * Math.pow( base, level ) );
        }
        return rv;
    }

    public static double calculateEnergyConsumption( Building building, int level )
    {
        double base = 0.0;
        double rv = 0;

        switch( building )
        {
            case ironMine:
                base = 4.0;
                break;
            case furnace:
                base = 2.0;
                break;
            case kryptoniteMine:
                base = 4.5;
                break;
            case spiceMine:
                base = 8.0;
                break;
            default:
                base = 0;
        }

        if( base != 0 )
        {
            rv = base * level * Math.pow( 1.15, level );
        }
        return rv;
    }


    public static Cost calculateCost( Building building, int level )
    {
        double base = 0.0;
        Cost rv;

        switch( building )
        {
            case ironMine:
            case furnace:
            case kryptoniteMine:
            case spiceMine:
            case fusionPlant:
            case thermosolarPlant:
                base = 1.51;
                break;
            case tunnel:
                base = 1.08;
                break;
            case superComputer:
                base = 1.0;
                break;
            default:
                base = 2.0;
        }
        long iron = 0;
        long metal = 0;
        long kryptonite = 0;
        long spice = 0;

        boolean overflow = false;
        Cost in = building.getCost();
        double d = Math.pow( base, (level - 1) ) * in.getIron();
        if( d > Long.MAX_VALUE )
            overflow = true;
        iron = (long)Math.ceil(d);
        d = Math.pow( base, (level - 1) ) * in.getMetal();
        if( d > Long.MAX_VALUE )
            overflow = true;
        metal = (long)Math.ceil(d);
        d = Math.pow( base, (level - 1) ) * in.getKryptonite();
        if( d > Long.MAX_VALUE )
            overflow = true;
        kryptonite = (long)Math.ceil(d);
        d = Math.pow( base, (level - 1) ) * in.getSpice();
        if( d > Long.MAX_VALUE )
            overflow = true;
        spice = (long)Math.ceil(d);

        if( overflow )
        {
            iron = -1;
            metal = -1;
            kryptonite = -1;
            spice = -1;
        }

        rv = new Cost( iron, metal, kryptonite, spice );
        return rv;
    }

    public static Cost calculateTotalCost( Building building, int level )
    {
        Cost rv = new Cost( 0, 0, 0, 0);
        for( int i = 1; i <= level; i++ )
        {
            Cost c = calculateCost( building, i );
            if( c.isOverflow() )
            {
                rv = c;
                break;
            }
            else
            {
                rv.add( c );
            }
        }
        return rv;
    }

    public static long calculateSolarSatProduction( double temperature, Version version )
    {
        long rv;
        if( version != Version.v20 )
        {
            rv = Math.max( (long)( 30 + temperature / 4 ), 10l );
        }
        else
        {
            double energie;
            if( temperature < 0 )
            {
                energie = 30.0 * ( 1 - ( temperature/-120 ) );
            }
            else
            {
                energie = 30.0 * temperature / 240.0 + 30.0;
            }
            rv = (long)Math.floor( energie );
            if( rv < 10 ) rv = 10;
            if( rv > 60 ) rv = 60;
        }

    	return rv;
    }
}
