/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool.resources;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.Parser;
import sitool.components.Building;
import sitool.components.Cost;
import sitool.components.EnumWrapper;
import sitool.components.Percent;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.Resource;
import sitool.components.Unit;
import sitool.components.Version;
import sitool.table.AlienTableCellRenderer;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.BlindCheckBoxTableCellRenderer;
import sitool.table.BlindTableCellRenderer;
import sitool.table.CheckBoxTableCellRenderer;
import sitool.table.ResourceTableCellRenderer;
import sitool.table.ResourceTableModel;
import sitool.table.SpinnerCellEditor;
import sitool.table.StockpileTableModel;
import sitool.table.TooltipTable;

/**
 * @author graf
 *
 */
public class ResourcePane extends JPanel implements ResourceModulePane, ImportExportPane
{
    private static final long serialVersionUID = 297701847420927691L;

    private MainFrame mainFrame;
    private ResourceSummaryPane parent;
    private ConfigData configData;
    private JPanel center;

    private JSpinner time = new JSpinner();
    private JComboBox<EnumWrapper<Unit>> units = new JComboBox<>();
    private JComboBox<Percent> steps = new JComboBox<>();
    private JCheckBox u_mod = new JCheckBox();
    private JLabel labelProduction = new JLabel();
    private JButton importButton = new JButton();

    protected JTable table;
    private ResourceTableModel model;
    protected JTable stockPile;
    private StockpileTableModel stockPileModel;

    public ResourcePane( ResourceSummaryPane parent )
    {
        this.parent = parent;
        mainFrame = parent.getMainFrame();
        configData = mainFrame.getConfigData();
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );

        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        updateLabel();
        updateTable();
    }

    public boolean updateStock( Planet planet, List<Long> resources) {
        final List<Planet> myPlanetsAndAsteroids = configData.getMyPlanetsAndAsteroids();
        boolean updated = false;
        // row 0 in stockpile is in flight resources
        int row = myPlanetsAndAsteroids.indexOf(planet) + 1;
        if( row >= 0) {
            int index = 0;
            for(final Iterator<Long> it = resources.iterator(); it.hasNext(); index++) {
                stockPileModel.setValueAt( it.next(), row, 2 * index + 1 );
            }
            updated = true;
        }
        return updated;
    }

    public void updateStock( List<Long> resources) {
        Cost cost = stockPileModel.getFlightResources();
        boolean[] use = stockPileModel.getFlightSettings();
        for(Resource resource : Resource.values()) {
            cost.setResource(resource, resources.get(resource.ordinal()));
            use[resource.ordinal()] = true;
        }
    }

    public void getResourceData( Document doc )
    {
        Element rootEl = doc.createElement( Parser.Tag_Application );
        doc.appendChild( rootEl );
        rootEl.setAttribute( Parser.Attribute_Time, getTitle() );
        rootEl.setAttribute( Parser.Tag_Account, configData.getActiveAccount().getId() );
        Element flightResourcesNode = doc.createElement( Parser.Tag_FlightResources );
        Element flightUseNode = doc.createElement( Parser.Tag_FlightUse );
        rootEl.appendChild( flightResourcesNode );
        rootEl.appendChild( flightUseNode );
        Cost cost = stockPileModel.getFlightResources();
        boolean[] u = stockPileModel.getFlightSettings();
        for( Resource res : Resource.values() )
        {
            flightResourcesNode.setAttribute( res.name(), Long.toString( cost.getResource( res ) ) );
            flightUseNode.setAttribute( res.name(), Boolean.toString( u[res.ordinal()] ) );
        }

        Map<Planet,Boolean> aliens = model.getAliens();
        Map<Planet, boolean[]> use = stockPileModel.getUse();
        Map<Planet, Cost> values = stockPileModel.getValues();

        List<Planet> myPlanets = configData.getMyPlanetsAndAsteroids();
        for( Map.Entry<Planet, Cost> entry : values.entrySet() )
        {
            Planet planet = entry.getKey();
            // save only current account
            if( myPlanets.contains( planet ) )
            {
                Element planetNode = doc.createElement( Parser.Tag_Planet );
                rootEl.appendChild( planetNode );
                Position position = planet.getPosition();
                planetNode.setAttribute( Parser.Attribute_Name, planet.getName() );
                planetNode.setAttribute( Parser.Attribute_Galaxy, Integer.toString( position.getGalaxy() ) );
                planetNode.setAttribute( Parser.Attribute_System, Integer.toString( position.getSystem() ) );
                planetNode.setAttribute( Parser.Attribute_Planet, Integer.toString( position.getPlanet() ) );

                Element resourceNode = doc.createElement( Parser.Tag_Resources );
                planetNode.appendChild( resourceNode );
                Element useNode = doc.createElement( Parser.Tag_Use );
                planetNode.appendChild( useNode );
                Element utilizationNode = doc.createElement( Parser.Tag_Utilization );
                if( !position.isAsteroidPosition() )
                {
                    planetNode.appendChild( utilizationNode );
                }

                cost = entry.getValue();
                u = use.get( planet );
                int[] util = model.getUtilization( planet );
                for( Resource res : Resource.values() )
                {
                    resourceNode.setAttribute( res.name(), Long.toString( cost.getResource( res ) ) );
                    useNode.setAttribute( res.name(), Boolean.toString( u[res.ordinal()] ) );
                    if( !position.isAsteroidPosition() )
                    {
                        utilizationNode.setAttribute( res.name(), Integer.toString( util[res.ordinal()] ) );
                    }
                }
                if( !position.isAsteroidPosition() )
                {
                    utilizationNode.setAttribute( Building.thermosolarPlant.name(), Integer.toString( util[Resource.values().length] ) );
                }
                Boolean alien = aliens.get( planet );
                if( alien != null )
                {
                    utilizationNode.setAttribute( Parser.Attribute_Alien, alien.toString() );
                }
            }
        }
    }

    private String getTitle()
    {
        JTabbedPane pane = ( JTabbedPane )getParent();
        int index = pane.getSelectedIndex();
        return pane.getTitleAt( index );
    }

    public String setResourceData( Document doc )
    {
        NodeList planets = doc.getElementsByTagName( Parser.Tag_Planet );

        Map<Planet,int[]> utilization = new HashMap<Planet, int[]>();
        Map<Planet,Boolean> aliens = new HashMap<Planet, Boolean>();
        Map<Planet, boolean[]> use = new HashMap<Planet, boolean[]>();
        Map<Planet, Cost> values = new HashMap<Planet, Cost>();
        Account newAccount = null;
        Element root = ( Element )doc.getFirstChild();
        String accountId = root.getAttribute( Parser.Tag_Account );
        String title = root.getAttribute( Parser.Attribute_Time );
        String asteroid = configData.getI18nText( "Asteroid" );

        // set target account
        if( accountId != null )
        {
            for( Account account : configData.getAccounts() )
            {
                if( accountId.equals( account.getId() ) && !account.equals( configData.getActiveAccount() ) )
                {
                    newAccount = account;
                    configData.setActiveAccount( newAccount );
                    break;
                }
            }
        }

        for( int i = 0; i < planets.getLength(); i++  )
        {
            Element planetNode = ( Element )planets.item( i );
            String name = planetNode.getAttribute( Parser.Attribute_Name );
            int galaxy = ConfigData.parseIntegerAttribute( planetNode.getAttribute( Parser.Attribute_Galaxy ) );
            int system = ConfigData.parseIntegerAttribute( planetNode.getAttribute( Parser.Attribute_System ) );
            int p = ConfigData.parseIntegerAttribute( planetNode.getAttribute( Parser.Attribute_Planet ) );
            // look if we have a planet on this position
            Planet myPlanet = configData.getPlanet( new Planet( name, new Position( galaxy, system, p ) ) );
            // do we need an asteroid?
            if( myPlanet != null && asteroid.equals( name ) )
            {
                myPlanet = myPlanet.makeAsteroid( configData );
            }
            if( myPlanet != null )
            {
                NodeList list = planetNode.getElementsByTagName( Parser.Tag_Resources );
                Element resourceNode = ( Element )list.item( 0 );
                list = planetNode.getElementsByTagName( Parser.Tag_Use );
                Element useNode = ( Element )list.item( 0 );
                list = planetNode.getElementsByTagName( Parser.Tag_Utilization );
                Element utilizationNode = null;
                if( list != null && list.getLength() > 0 )
                {
                    utilizationNode = ( Element )list.item( 0 );
                }

                Cost cost = new Cost( 0, 0, 0, 0 );
                boolean[] u = new boolean[Resource.values().length];
                int[] util = new int[u.length + 1];
                for( Resource res : Resource.values() )
                {
                    cost.setResource( res, ConfigData.parseLongAttribute( resourceNode.getAttribute( res.name() ) ) );
                    u[res.ordinal()] = ConfigData.parseBooleanAttribute( useNode.getAttribute( res.name() ) );
                    if( utilizationNode != null )
                    {
                        util[res.ordinal()] = ConfigData.parseIntegerAttribute( utilizationNode.getAttribute( res.name() ) );
                    }
                }
                if( utilizationNode != null )
                {
                    util[Resource.values().length] = ConfigData.parseIntegerAttribute( utilizationNode.getAttribute( Building.thermosolarPlant.name() ) );
                    utilization.put( myPlanet, util );
                    aliens.put( myPlanet, ConfigData.parseBooleanAttribute( utilizationNode.getAttribute( Parser.Attribute_Alien ) ) );
                }
                use.put( myPlanet, u );
                values.put( myPlanet, cost );
            }
        }

        NodeList list = root.getElementsByTagName( Parser.Tag_FlightResources );
        Element resourceNode = null;
        if(list != null && list.getLength() > 0)
            resourceNode = ( Element )list.item( 0 );
        list = root.getElementsByTagName( Parser.Tag_FlightUse );
        Element useNode = null;
        if(list != null && list.getLength() > 0)
            useNode = ( Element )list.item( 0 );
        if(resourceNode != null && useNode != null) {
            Cost cost = stockPileModel.getFlightResources();
            boolean[] u = stockPileModel.getFlightSettings();
            for( Resource res : Resource.values() )
            {
                cost.setResource( res, ConfigData.parseLongAttribute( resourceNode.getAttribute( res.name() ) ) );
                u[res.ordinal()] = ConfigData.parseBooleanAttribute( useNode.getAttribute( res.name() ) );
            }
        }

        model.setUtilization( utilization );
        model.setAliens( aliens );
        stockPileModel.setData( use, values );
        if( newAccount != null )
        {
            mainFrame.updateAccounts();
        }
        return title;
    }

    private JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );

        JPanel upper = new JPanel( new BorderLayout() );
        JPanel left = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
        upper.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        left.add( labelProduction );
        left.add( time );
        left.add( units );
        left.add( u_mod );
        upper.add( left, BorderLayout.WEST );
        center.add( upper, BorderLayout.NORTH );
        time.setModel( new SpinnerNumberModel( 0, 0, Integer.MAX_VALUE, 1 ) );
        for( Unit unit : Unit.values() )
        {
            units.addItem( new EnumWrapper<Unit>( configData, unit ) );
        }
        units.setSelectedIndex( 1 );

        JPanel tablePanel = new JPanel();
        tablePanel.setLayout( new BoxLayout( tablePanel, BoxLayout.Y_AXIS ) );
        model = new ResourceTableModel( mainFrame );
        final BlindTableCellRenderer colR0Renderer = new AlienTableCellRenderer();
        final BlindTableCellRenderer colS0Renderer = new BlindTableCellRenderer();
        final BlindTableCellRenderer rowRenderer = new BlindTableCellRenderer();
        final BlindCheckBoxTableCellRenderer linkRenderer = new BlindCheckBoxTableCellRenderer();
        final TableCellEditor columnEditor = new DefaultCellEditor( steps ) ;
        final TableCellEditor bonusEditor = new SpinnerCellEditor( new SpinnerNumberModel( 0, 0, 100, 3 ) );
        JCheckBox link = new JCheckBox( ConfigData.getIcon( "/link-open.png" ) );
        link.setHorizontalAlignment( SwingConstants.CENTER );
        final TableCellEditor linkEditor = new DefaultCellEditor( link ) ;
        rowRenderer.setHorizontalAlignment( SwingConstants.CENTER );
        rowRenderer.setFont( importButton.getFont().deriveFont( Font.BOLD ) );
        //rowRenderer.setForeground( Color.green.darker().darker() );
        linkRenderer.setHorizontalAlignment( SwingConstants.RIGHT );
        table = new TooltipTable( model ) {
            private static final long serialVersionUID = 7149497989459592130L;
            public TableCellRenderer getCellRenderer(int row, int column)
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                int planetCount = mainFrame.getConfigData().getMyPlanets().size();
                if( row >= planetCount )
                {
                    if( modelColumn > 0 && modelColumn % 2 == 0 && row == planetCount )
                    {
                        rv = linkRenderer;
                    }
                    else
                    {
                        rv = rowRenderer;
                    }
                }
                else if( modelColumn == 0 )
                {
                    rv = colR0Renderer;
                }
                else
                {
                    rv = super.getCellRenderer( row, column );
                }
                return rv;
            }

            public TableCellEditor getCellEditor(int row, int column)
            {
                TableCellEditor rv = super.getCellEditor( row, column );
                // convert to model
                int modelRow =  convertRowIndexToModel( row );
                int modelColumn =  convertColumnIndexToModel( column );
                if( modelRow == getRowCount() - 1 )
                {
                    if( modelColumn > 0 && modelColumn % 2 == 0 && modelColumn / 2 <= Resource.values().length
                     || modelColumn == getModel().getColumnCount() - 1 )
                    {
                        rv = bonusEditor;
                    }
                }
                else if( modelColumn > 0 && modelColumn % 2 == 0 )
                {
                    if( row == mainFrame.getConfigData().getMyPlanets().size() )
                    {
                        rv = linkEditor;
                    }
                    else
                    {
                        rv = columnEditor;
                    }
                }
                return rv;
            }

        };
        table.setDefaultRenderer( Object.class, new ResourceTableCellRenderer() );
        table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
        table.getTableHeader().setReorderingAllowed( false );
        table.addMouseListener( new MouseAdapter()
        {
            public void mouseClicked( MouseEvent e )
            {
                if( configData.getVersion() == Version.v20 )
                {
                    int row = table.rowAtPoint(e.getPoint());
                    int col = table.columnAtPoint(e.getPoint());
                    // convert to model
                    row = table.convertRowIndexToModel( row );
                    col = table.convertColumnIndexToModel( col );
                    if ( SwingUtilities.isLeftMouseButton(e) && e.getClickCount() == 1 && col == 0 && row < table.getRowCount() - 2)
                    {
                        model.toggleAlien( row );
                    }
                }
            }
        } );

        JScrollPane scrollPane = new JScrollPane( table,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        scrollPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        tablePanel.add( scrollPane );

        stockPileModel = new StockpileTableModel( mainFrame );
        // link the resource table
        model.setStocks( stockPileModel );
        stockPileModel.addTableModelListener( table );

        JTextField editor = new JTextField();
        editor.setBorder( new EmptyBorder( 0, 2, 0, 2 ) );
        editor.setHorizontalAlignment( SwingConstants.RIGHT );
        JCheckBox checkbox = new JCheckBox();
        checkbox.setHorizontalAlignment( SwingConstants.CENTER );
        final DefaultCellEditor textEditor = new DefaultCellEditor( editor );
        textEditor.setClickCountToStart( 1 );
        final TableCellEditor checkEditor = new DefaultCellEditor( checkbox );
        final CheckBoxTableCellRenderer renderer = new CheckBoxTableCellRenderer();
        renderer.setHorizontalAlignment( SwingConstants.CENTER );
        stockPile = new TooltipTable( stockPileModel ) {
            private static final long serialVersionUID = 5092225737602828765L;
            public TableCellRenderer getCellRenderer(int row, int column)
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                if( modelColumn == 0 )
                {
                    rv = colS0Renderer;
                }
                else if( modelColumn % 2 == 0 )
                {
                    rv = renderer;
                }
                else
                {
                    rv = super.getCellRenderer( row, column );
                }
                return rv;
            }
            public TableCellEditor getCellEditor(int row, int column)
            {
                TableCellEditor rv;
                // convert to model
                int modelColumn =  convertColumnIndexToModel( column );
                if( modelColumn > 0 )
                {
                    if( modelColumn % 2 == 0 )
                    {
                        rv = checkEditor;
                    }
                    else
                    {
                        rv = textEditor;
                    }
                }
                else
                {
                    rv = super.getCellEditor( row, column );
                }
                return rv;
            }
        };
        stockPile.addMouseListener( stockPileModel );
        stockPile.getTableHeader().setReorderingAllowed( false );
        stockPile.getTableHeader().addMouseListener( new MouseAdapter()
        {
            public void mouseClicked( MouseEvent e )
            {
                if ( SwingUtilities.isLeftMouseButton(e) && e.getClickCount() == 1 )
                {
                    Object o = e.getSource();
                    if( o instanceof JTableHeader )
                    {
                        JTableHeader header = ((JTableHeader)o);
                        int col = header.columnAtPoint(e.getPoint());
                        if( col % 2 == 0 )
                        {
                            boolean uValue = !getArrayValue( col / 2 );

                            if( col == 0 )
                            {
                                setAllUValues( uValue );
                            }
                            else
                            {
                                setUValue( col / 2, uValue );
                            }

                            stockPileModel.fireTableDataChanged();
                        }
                    }
                }
            }

            private boolean getArrayValue( int column )
            {
                boolean rv = false;
                Set<boolean[]> use = new HashSet<>(stockPileModel.getUse().values());
                use.add(stockPileModel.getFlightSettings());
                for( boolean[] uArray : use )
                {
                    if( column == 0 )
                    {
                        for( int i = 0; !rv && i < uArray.length; i++ )
                        {
                            rv |= uArray[i];
                        }
                    }
                    else
                    {
                        rv |= uArray[column - 1];
                    }
                }
                return rv;
            }

            private void setAllUValues( boolean uValue )
            {
                List<boolean[]> use = new ArrayList<>(stockPileModel.getUse().values());
                use.add(stockPileModel.getFlightSettings());
                for( boolean[] uArray : use )
                {
                    Arrays.fill( uArray, uValue );
                }
            }

            private void setUValue( int column, boolean uValue )
            {
                List<boolean[]> use = new ArrayList<>(stockPileModel.getUse().values());
                use.add(stockPileModel.getFlightSettings());
                for( boolean[] uArray : use )
                {
                    uArray[column - 1] = uValue;
                }
            }

        } );

        stockPile.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
        AlternateLineTableCellRenderer defaultCellRenderer = new AlternateLineTableCellRenderer();
        defaultCellRenderer.setHorizontalAlignment( SwingConstants.RIGHT );
        stockPile.setDefaultRenderer( Object.class, defaultCellRenderer );

        JPanel resourcePanel = new JPanel( new BorderLayout() );
        resourcePanel.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        scrollPane = new JScrollPane( stockPile,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        scrollPane.setBorder( BorderFactory.createEmptyBorder( 0, 0, 0, 0 ) );
        resourcePanel.add( scrollPane, BorderLayout.CENTER );

        JPanel south = new JPanel( new FlowLayout( FlowLayout.CENTER ) );
        importButton.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionRead();
            }
        });
        south.add( importButton );
        resourcePanel.add( south, BorderLayout.SOUTH );
        tablePanel.add( resourcePanel );

        center.add( tablePanel, BorderLayout.CENTER );

        // add listeners and initialize
        time.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                setMultiplier();
            }
        });
        units.addItemListener( new ItemListener()
        {
            double oldMultiplier;

            @SuppressWarnings("unchecked")
            public void itemStateChanged( ItemEvent e )
            {
                if( e.getStateChange() == ItemEvent.DESELECTED )
                {
                    oldMultiplier = ((EnumWrapper<Unit>)e.getItem()).getBase().getMultiplier();
                }
                else
                {
                    double newMultiplier = ((EnumWrapper<Unit>)e.getItem()).getBase().getMultiplier();
                    double value = ((Number)time.getValue()).doubleValue();
                    int newValue = (int) Math.round( value * oldMultiplier / newMultiplier );
                    time.setValue( Integer.valueOf( newValue ) );
                }
            }
        });
        u_mod.addItemListener( new ItemListener()
        {
            public void itemStateChanged( ItemEvent e )
            {
                setSteps();
                setMultiplier();
            }
        });

        setMultiplier();
        setSteps();

        return center;
    }

    protected void actionRead()
    {
        JDialog dlg = new ResourceBox( this, stockPileModel );
        mainFrame.showModalBox( dlg );
    }

    private void setSteps()
    {
        Version version = configData.getVersion();
        int step = 10;
        double speedUp = model.powerUtilizationFactor(version);
        if(version == Version.v20) {
            step = 5 * (int)Math.floor(speedUp);
        }

        steps.removeAllItems();
        for( int i = (int)(100 * speedUp); i >= 0; i-= step )
        {
            if( u_mod.isSelected() && i > 25 )
            {
                continue;
            }
            steps.addItem( new Percent( i ) );
        }
        steps.setMaximumRowCount( 25 );
    }

    @SuppressWarnings("unchecked")
    private void setMultiplier()
    {
        double multiplier = 1.0;
        multiplier *= ((Number)time.getValue()).doubleValue();
        multiplier *= ((EnumWrapper<Unit>)units.getSelectedItem()).getBase().getMultiplier();
        model.setMultiplier( multiplier, u_mod.isSelected() );
        adjustColumns();
    }

    private void updateLabel()
    {
        labelProduction.setText( configData.getI18nText( "Resources.production" ) );
        u_mod.setVisible( configData.getVersion() == Version.v20 );
        u_mod.setText( configData.getI18nText( "Resources.vacationMode" ) );
        importButton.setText( configData.getI18nText( "Resources.import" ) );
    }

    private void updateTable()
    {
        model.update();
        stockPileModel.update();
        setSteps();
        // free up real estate by hiding the unused column
        TableColumnModel columnModel = table.getColumnModel();
        TableColumn column = columnModel.getColumn( 9 );
        columnModel.removeColumn( column );
        adjustColumns();
    }

    private void adjustColumns()
    {
        int usageWidth = 58;
        int energyWidth = 68;

        int width = 160;
        int i = 0;
        TableColumnModel columnModel = table.getColumnModel();
        TableColumn column = columnModel.getColumn( i );
        TableColumnModel stockpileColumnModel = stockPile.getColumnModel();
        TableColumn stockpileColumn = stockpileColumnModel.getColumn( i );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
        stockpileColumn.setPreferredWidth( width );
        stockpileColumn.setMinWidth( width );
        stockpileColumn.setMaxWidth( width );
        width = usageWidth;
        for( i = 2; i < 9; i += 2 )
        {
            column = columnModel.getColumn( i );
            column.setPreferredWidth( width );
            column.setMinWidth( width );
            column.setMaxWidth( width );
            stockpileColumn = stockpileColumnModel.getColumn( i );
            stockpileColumn.setPreferredWidth( width );
            stockpileColumn.setMinWidth( width );
            stockpileColumn.setMaxWidth( width );
        }
        i = 9;
        column = columnModel.getColumn( i );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
        i = 10;
        width = energyWidth;
        column = columnModel.getColumn( i );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
    }

    @Override
    public void setMultipleProduction( double multipleProduction, int productionBonus, int allianceSpecialization ) {
        model.setMultipleProduction(multipleProduction, productionBonus, allianceSpecialization);
        setSteps();
    }

    @Override
    public String getFileName() {
        return parent.getFileName(this);
    }

    @Override
    public String getExtension() {
        return parent.getExtension();
    }

    @Override
    public File getCurrentDir() {
        return parent.getCurrentDir();
    }

    @Override
    public void setCurrentDir( File currentDir ) {
        parent.setCurrentDir(currentDir);
    }

    @Override
    public Document getData( Parser xmlParser ) {
        Document doc = null;
        if( xmlParser != null ) {
            doc = xmlParser.getDocument();
            getResourceData( doc );
        }
        return doc;
    }

    @Override
    public MainFrame getMainFrame() {
        return mainFrame;
    }
}
