/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool.savecalc;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.SoftBevelBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.Parser;
import sitool.components.Accountable;
import sitool.components.Distance;
import sitool.components.Fleet;
import sitool.components.FleetSpeed;
import sitool.components.MultiLineButton;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.PositionField;
import sitool.components.Ship;
import sitool.components.Specialization;
import sitool.components.Technology;
import sitool.components.TimeField;
import sitool.components.TimePanel;
import sitool.components.Version;
import sitool.resources.ImportExportPane;
import sitool.table.CheckBoxTableCellRenderer;
import sitool.table.PlanetTableCellEditor;
import sitool.table.SaveTargetComparator;
import sitool.table.SaveTargetComparatorMode;
import sitool.table.SaveTargetTableCellRenderer;
import sitool.table.SaveTargetTableModel;
import sitool.table.SortableTableCellHeaderRenderer;
import sitool.table.SortableTableHeaderMouseListener;
import sitool.table.TooltipTable;

/**
 *
 * @author graf
 *
 */
public class SaveCalc extends AbstractDurationPanel implements SaveCalcPane, ImportExportPane
{
	private static final long serialVersionUID = -1878874776754387489L;

	protected static final int buttonWidth = 189;
    protected static final Color off = UIManager.getColor("control");
    protected static final Color on = new Color( 0xFAAF00 );
    protected static final String extension = ".svz";
    private static SimpleDateFormat fileFormat = new SimpleDateFormat( "yyyyMMddHHmm" );

    private static final String spacer = "   ";

    private JComboBox<Planet> myPlanets = new JComboBox<>();
    private JTextField velocity = new JTextField( 16 );
    private PositionField launchPosition = new PositionField();
    private TimeField saveTime;

    private TimePanel timePanel;
    private JTable summaryTable;
    private JTable selectedTargets;
    private SaveTargetTableModel currentSaves;
    private SaveTargetTableModel allSaves;

    private MainFrame mainFrame;
    private Calculator calculator;
    private Account current;

    private JButton speedButton = new JButton();
    private JButton timeButton = new JButton();
    private JButton calculateButton = new JButton();
    private JLabel labelSpeed = new JLabel();
    private JLabel labelCoords = new JLabel();
    private JLabel labelTime = new JLabel();
    private JLabel labelCurrent = new JLabel();
    private JLabel labelSaved = new JLabel();
    private MultiLineButton selectSaveflight;
    private JButton addSaveflight = new JButton();
    private MultiLineButton addAllSaveflights;
    private JButton deleteSelected = new JButton();
    private JButton deleteAll = new JButton();
    private JButton loadFlights = new JButton();
    private JButton saveFlights = new JButton();

    /**
     * @throws HeadlessException
     */
    public SaveCalc( MainFrame parent ) throws HeadlessException
    {
        super(parent.getConfigData());

        this.mainFrame = parent;
        current = configData.getActiveAccount();
        calculator = new Calculator( mainFrame.getConfigData() );

        setLayout( new BorderLayout() );
        JPanel centerPanel = createCenterPanel();
        add( centerPanel, BorderLayout.CENTER );
    }

    public MainFrame getMainFrame()
    {
        return mainFrame;
    }

    public void setVelocity( FleetSpeed velocity )
    {
        String unitText = configData.getUnitFromMaxSpeed( velocity );
        this.velocity.setText( unitText );
        this.velocity.setToolTipText( unitText );
    }

    public void setDuration( long duration )
    {
        long readBack = 2 * duration;
        saveTime.setDay( Long.toString( readBack / Calculator.secondsPerDay ) );
        readBack %= Calculator.secondsPerDay;
        saveTime.setHour( Long.toString( readBack / Calculator.secondsPerHour ) );
        readBack %= Calculator.secondsPerHour;
        saveTime.setMinute( Long.toString( readBack / Calculator.secondsPerMinute ) );
    }

    /**
     *
     */
    public void update()
    {
        mainFrame.getRootPane().setDefaultButton( calculateButton );
        updateMyPlanets();
        updateLabel();
        updateTable();
    }


    private void updateTable()
    {
        if( current != configData.getActiveAccount() )
        {
            currentSaves.removeAll();
            current = configData.getActiveAccount();
        }
        currentSaves.fireTableStructureChanged();
        allSaves.fireTableStructureChanged();

        adjustTableColumns();
    }

    private void updateLabel()
    {
        speedButton.setText( configData.getI18nText( "Savecalc.calculate" ) );
        timeButton.setText( configData.getI18nText( "Savecalc.calculate" ) );
        calculateButton.setText( configData.getI18nText( "Savecalc.calculateSaveflight" ) );
        labelSpeed.setText( configData.getI18nText( "Savecalc.speed" ) );
        labelCoords.setText( configData.getI18nText( "Savecalc.coords" ) );
        labelTime.setText( configData.getI18nText( "Savecalc.flightTime" ) );
        labelCurrent.setText( configData.getI18nText( "Savecalc.currentFlights" ) );
        labelSaved.setText( configData.getI18nText( "Savecalc.savedFlights" ) );
        selectSaveflight.setText( configData.getI18nText( "Savecalc.select1" ), configData.getI18nText( "Savecalc.select2" ) );
        addSaveflight.setText( configData.getI18nText( "Savecalc.add" ) );
        addAllSaveflights.setText( configData.getI18nText( "Savecalc.addAll1" ), configData.getI18nText( "Savecalc.addAll2" ) );
        deleteSelected.setText( configData.getI18nText( "Savecalc.delete" ) );
        deleteAll.setText( configData.getI18nText( "Savecalc.deleteAll" ) );
        loadFlights.setText( configData.getI18nText( "Load" ) );
        saveFlights.setText( configData.getI18nText( "Save" ) );

        timePanel.updateLabel();
        saveTime.update();

        addAllSaveflights.setEnabled( false );
        velocity.setEditable( false );
    }

    protected void actionFleetSpeed()
    {
        JDialog dlg = new FleetSpeedBox( mainFrame );
        showModalBox( dlg );
        setVelocity( configData.getVelocity() );
    }

    protected void actionSaveTime()
    {
        JDialog dlg = new DateBox( this );
        showModalBox( dlg );
    }

    protected void showModalBox( JDialog dialog )
    {
    	mainFrame.showModalBox( dialog );
    }

    protected JPanel createCenterPanel()
    {
        JPanel panel = new JPanel( new BorderLayout() );
        Dimension size;

        updateMyPlanets();
        myPlanets.addItemListener( new ItemListener() {

            public void itemStateChanged( ItemEvent e )
            {
                actionComboBoxChanged( e );
            }

        });

        JPanel northPanel = new JPanel( new FlowLayout( FlowLayout.CENTER ) );
        northPanel.setBorder( new EtchedBorder( EtchedBorder.LOWERED ) );

        GridBagConstraints gridConstraints = new GridBagConstraints();
        gridConstraints.gridx = 0;
        gridConstraints.gridy = 0;
        gridConstraints.gridheight = 1;
        gridConstraints.gridwidth = 2;
        gridConstraints.fill = GridBagConstraints.NONE;
        gridConstraints.anchor = GridBagConstraints.WEST;
        gridConstraints.weightx = 0.0;
        gridConstraints.weighty = 0.0;
        gridConstraints.insets = new Insets( 2, 2, 2, 2 );

        JPanel businessPanel = new JPanel( new GridBagLayout() );
        businessPanel.setBorder( new EmptyBorder( 2, 2, 2, 2) );
        businessPanel.add( labelCoords, gridConstraints );
        gridConstraints.gridx = 3;
        businessPanel.add( labelSpeed, gridConstraints );
        gridConstraints.gridx = 6;
        businessPanel.add( labelTime, gridConstraints );
        gridConstraints.gridwidth = 1;
        gridConstraints.fill = GridBagConstraints.BOTH;
        gridConstraints.weightx = 1.0;
        gridConstraints.gridx = 0;
        gridConstraints.gridy = 1;
        businessPanel.add( launchPosition, gridConstraints );
        gridConstraints.gridx = 1;
        businessPanel.add( myPlanets, gridConstraints );
        gridConstraints.gridx = 2;
        businessPanel.add( new JLabel( spacer ), gridConstraints );
        gridConstraints.gridx = 3;
        velocity.setHorizontalAlignment( SwingConstants.RIGHT );
        //velocity.addFocusListener( new TextFieldFocusListener() );
        businessPanel.add( velocity, gridConstraints );
        speedButton.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionFleetSpeed();
            }
        } );
        gridConstraints.gridx = 4;
        businessPanel.add( speedButton, gridConstraints );
        gridConstraints.gridx = 5;
        businessPanel.add( new JLabel( spacer ), gridConstraints );
        gridConstraints.gridx = 6;
        saveTime = new TimeField( configData );
        businessPanel.add( saveTime, gridConstraints );
        timeButton.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionSaveTime();
            }
        } );
        gridConstraints.gridx = 7;
        businessPanel.add( timeButton, gridConstraints );

        gridConstraints.anchor = GridBagConstraints.WEST;
        gridConstraints.fill = GridBagConstraints.BOTH;
        gridConstraints.gridx = 1;
        gridConstraints.gridy = 2;
        gridConstraints.gridwidth = 5;
        gridConstraints.insets.top = 6;
        calculateButton.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionCalculate();
            }
        } );
        businessPanel.add( calculateButton, gridConstraints );
        northPanel.add( businessPanel );

        panel.add( northPanel, BorderLayout.NORTH );

        MouseListener mouseListener = new SortableTableHeaderMouseListener();
        currentSaves = new SaveTargetTableModel( mainFrame, false );
        allSaves = new SaveTargetTableModel( mainFrame, true );

        size = myPlanets.getPreferredSize();
//        size.width += 24;
        size.width = buttonWidth;
        size.height = 2 * size.height - 4;

        JTable rowHeader;
        JScrollPane scrollPane;
        JPanel summaryPanel = new JPanel( new BorderLayout() );
        summaryPanel.setBorder( new EmptyBorder( 2, 2, 0, 2) );
        labelCurrent.setBorder( new SoftBevelBorder( SoftBevelBorder.RAISED ) );
        summaryPanel.add( labelCurrent, BorderLayout.NORTH);
        final CheckBoxTableCellRenderer col2Renderer = new CheckBoxTableCellRenderer();
        summaryTable = new TooltipTable( currentSaves ) {
            private static final long serialVersionUID = -8314337443532116923L;

            @Override
            public TableCellRenderer getCellRenderer( int row, int column ) {
                if(convertColumnIndexToModel(column) == 2 ) {
                    return col2Renderer;
                }
                return super.getCellRenderer(row, column);
            }
        };
        summaryTable.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        TableCellRenderer delegate = summaryTable.getTableHeader().getDefaultRenderer();
        summaryTable.getTableHeader().setDefaultRenderer( new SortableTableCellHeaderRenderer( delegate ) );
        summaryTable.getTableHeader().addMouseListener( mouseListener );
        summaryTable.setDefaultRenderer( Object.class, new SaveTargetTableCellRenderer() );
        scrollPane = new JScrollPane( summaryTable,
                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                        JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        scrollPane.setBorder( BorderFactory.createEmptyBorder( 0, 0, 0, 0 ) );
        rowHeader = currentSaves.getRowHeaderTable();
        scrollPane.setRowHeaderView( rowHeader );
        scrollPane.setCorner( ScrollPaneConstants.UPPER_LEFT_CORNER, rowHeader.getTableHeader() );
        summaryPanel.add( scrollPane, BorderLayout.CENTER );

        JPanel actionPanel = new JPanel( new FlowLayout() );
        JPanel buttonPanel = new JPanel( new GridLayout( 3, 1, 5, 5) );
        selectSaveflight = new MultiLineButton( configData.getI18nText( "Savecalc.select1" ), configData.getI18nText( "Savecalc.select2" ) );
        selectSaveflight.setHorizontalAlignment( SwingConstants.CENTER );
        selectSaveflight.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionSelectClosest();
            }
        });
        selectSaveflight.setPreferredSize( size );
        buttonPanel.add( selectSaveflight );
        addSaveflight.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                // since we limited selection, there can be only one
                int selectedRow = summaryTable.getSelectedRow();
                SaveTarget selectedTarget = currentSaves.getRow( selectedRow );
                Planet target = selectedTarget.getPlanet();
                Planet savePlanet = new Planet( target.getName(), new Position( target.getPosition(), false ) );
                SaveTarget newTarget = new SaveTarget( selectedTarget.getOrigin(), savePlanet, selectedTarget.isLongDistance(), selectedTarget.getSpeed(), selectedTarget.getVelocity(), selectedTarget.getUnit(), selectedTarget.getDuration(), selectedTarget.getRequestedDuration(), selectedTarget.getAccount() );
                allSaves.addRow( newTarget );
            }
        });
        addSaveflight.setPreferredSize( size );
        buttonPanel.add( addSaveflight );
        addAllSaveflights = new MultiLineButton( configData.getI18nText( "Savecalc.addAll1" ), configData.getI18nText( "Savecalc.addAll2" ) );
        addAllSaveflights.setHorizontalAlignment( SwingConstants.CENTER );
        addAllSaveflights.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionAddAll();
            }
        });
        addAllSaveflights.setPreferredSize( size );
        buttonPanel.add( addAllSaveflights );
        actionPanel.add( buttonPanel );
        summaryPanel.add( actionPanel, BorderLayout.EAST );
        addSaveflight.setEnabled( false );
        addAllSaveflights.setEnabled( false );
        summaryTable.getSelectionModel().addListSelectionListener( new ListSelectionListener()
        {
            public void valueChanged( ListSelectionEvent e )
            {
                if( !e.getValueIsAdjusting() )
                {
                    boolean enable = ( summaryTable.getSelectedRow() >= 0 );
                    addSaveflight.setEnabled( enable );
                    if( enable )
                    {
                        int rowIndex = summaryTable.getSelectedRow();
                        SaveTarget target = currentSaves.getRow( rowIndex );
                        if( configData.getVersion() != Version.v20 )
                        {
                            enable = !target.getPlanet().getPosition().isRealPosition();
                            enable &= !target.getOrigin().getPosition().isAsteroidPosition();
                        }
                        else
                        {
                            enable = target.getPlanet().getPosition().isRealPosition();
                        }
                    }
                    addAllSaveflights.setEnabled( enable );
                }
            }
        });

        JPanel southPanel = new JPanel( new BorderLayout() );
        southPanel.setBorder( new EmptyBorder( 2, 2, 0, 2) );
        labelSaved.setBorder( new SoftBevelBorder( SoftBevelBorder.RAISED ) );
        southPanel.add( labelSaved , BorderLayout.NORTH );
        final TableCellEditor columnEditor = new PlanetTableCellEditor();
        selectedTargets = new TooltipTable( allSaves )
        {
            private static final long serialVersionUID = 4730075221929609521L;

            @Override
            public TableCellRenderer getCellRenderer( int row, int column ) {
                if(convertColumnIndexToModel(column) == 2 ) {
                    return col2Renderer;
                }
                return super.getCellRenderer(row, column);
            }

            @Override
            public TableCellEditor getCellEditor(int row, int column)
            {
                TableCellEditor rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                if( modelColumn == 1 )
                {
                    rv = columnEditor;
                }
                else
                {
                    rv = super.getCellEditor( row, column );
                }
                return rv;
            }
        };
        delegate = selectedTargets.getTableHeader().getDefaultRenderer();
        selectedTargets.getTableHeader().setDefaultRenderer( new SortableTableCellHeaderRenderer( delegate ) );
        selectedTargets.getTableHeader().addMouseListener( mouseListener );
        selectedTargets.setDefaultRenderer( Object.class, new SaveTargetTableCellRenderer() );
        scrollPane = new JScrollPane( selectedTargets,
                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                        JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        scrollPane.setBorder( BorderFactory.createEmptyBorder( 0, 0, 0, 0 ) );
        rowHeader = allSaves.getRowHeaderTable();
        scrollPane.setRowHeaderView( rowHeader );
        scrollPane.setCorner( ScrollPaneConstants.UPPER_LEFT_CORNER, rowHeader.getTableHeader() );
        southPanel.add( scrollPane, BorderLayout.CENTER );

        JPanel savePanel = new JPanel( new BorderLayout() );
        actionPanel = new JPanel( new FlowLayout() );
        //actionPanel.setBackground(Color.green);
        buttonPanel = new JPanel( new GridLayout( 5, 1, 5, 5) );
        size = new Dimension( size );
        size.height = size.height / 2 + 4;
        deleteSelected.setPreferredSize( size );
        deleteSelected.setEnabled( false );
        deleteSelected.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionDeleteSaves();
            }

        });
        buttonPanel.add( deleteSelected );
        selectedTargets.getSelectionModel().addListSelectionListener( new ListSelectionListener()
        {
            public void valueChanged( ListSelectionEvent e )
            {
                if( !e.getValueIsAdjusting() )
                {
                    boolean enable = ( e.getFirstIndex() >= 0 );
                    deleteSelected.setEnabled( enable );
                }
            }
        });
        selectedTargets.getInputMap(WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0), "deleteSaves");
        selectedTargets.getActionMap().put("deleteSaves", new AbstractAction() {
            private static final long serialVersionUID = 5999113634471333070L;

            @Override
            public void actionPerformed( ActionEvent e ) {
                if(deleteSelected.isEnabled()) {
                    actionDeleteSaves();
                }
            }
        });
        deleteAll.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                allSaves.removeAll();
            }

        });
        deleteAll.setPreferredSize( size );
        buttonPanel.add( deleteAll );

        buttonPanel.add( new JLabel( " " ) );

        loadFlights.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed( ActionEvent e )
            {
                actionLoad();
            }
        } );
        loadFlights.setPreferredSize( size );
        buttonPanel.add( loadFlights );
        saveFlights.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed( ActionEvent e )
            {
                actionSave();
            }
        } );
        saveFlights.setPreferredSize( size );
        buttonPanel.add( saveFlights );

        actionPanel.add( buttonPanel );
        savePanel.add( actionPanel, BorderLayout.NORTH );
        // add some padding
        savePanel.add( new JLabel( "  " ), BorderLayout.CENTER );
        timePanel = new TimePanel( configData, selectedTargets );
        savePanel.add( timePanel, BorderLayout.SOUTH );
        southPanel.add( savePanel, BorderLayout.EAST );

        JSplitPane splitPane = new JSplitPane( JSplitPane.VERTICAL_SPLIT, false, summaryPanel, southPanel );
        splitPane.setResizeWeight( 0.38 );
        splitPane.setBorder( new EtchedBorder( EtchedBorder.LOWERED ) );
        panel.add( splitPane, BorderLayout.CENTER );

        return panel;
    }

    /**
     *
     */
    protected void actionAddAll()
    {
        if( configData.getVersion() != Version.v20 )
        {
            addAll1();
        }
        else
        {
            addAll2();
        }
    }

    private void addAll1()
    {
        // if target is out-system create a save target for every one of my planets
        // try to prefer systems with known planets
        // since we limited selection, there can be only one
        int selectedRow = summaryTable.getSelectedRow();
        SaveTarget selectedTarget = currentSaves.getRow( selectedRow );

        Position position = selectedTarget.getPlanet().getPosition();
        if( !position.isRealPosition() )
        {
            Position originPosition = selectedTarget.getOrigin().getPosition();
            int galaxyDiff = Math.abs( position.getGalaxy() - originPosition.getGalaxy() );
            int systemDiff = position.getSystem() - originPosition.getSystem();
            List<Planet> myPlanets = mainFrame.getConfigData().getMyPlanets();
            List<Planet> knownPlanets = mainFrame.getConfigData().getPlanets();
            Version version = mainFrame.getConfigData().getVersion();

            SaveTarget newTarget;
            for( Planet origin : myPlanets )
            {
                Planet savePlanet = null;
                if( origin.equals( selectedTarget.getOrigin() ) )
                {
                    Planet target = selectedTarget.getPlanet();
                    savePlanet = new Planet( target.getName(), new Position( target.getPosition(), false ) );
                    newTarget = new SaveTarget( origin, savePlanet, selectedTarget.isLongDistance(), selectedTarget.getSpeed(), selectedTarget.getVelocity(), selectedTarget.getUnit(), selectedTarget.getDuration(), selectedTarget.getRequestedDuration(), selectedTarget.getAccount() );
                }
                else
                {
                    if( galaxyDiff > 0 )
                    {
                        for( Planet target : knownPlanets )
                        {
                            if( Math.abs( origin.getPosition().getGalaxy() - target.getPosition().getGalaxy() )  == galaxyDiff )
                            {
                                savePlanet = target;
                                break;
                            }
                        }
                        if( savePlanet == null )
                        {
                            int newgalaxy = origin.getPosition().getGalaxy() + galaxyDiff;
                            if( newgalaxy > Distance.getUpperGalaxyBound(version) )
                            {
                                newgalaxy = origin.getPosition().getGalaxy() - galaxyDiff;
                            }
                            savePlanet = new Planet( null, new Position( newgalaxy, 0, 0) );
                        }
                    }
                    else
                    {
                        originPosition = origin.getPosition();
                        Planet placeHolder = null, fav = null;
                        // look the other way first in case we find a 'real' planet there
                        int newSystem = originPosition.getSystem() - systemDiff;
                        // check for wrap around
                        if( newSystem < Distance.lowerSystemBound )
                        {
                            newSystem += Distance.getUpperSystemBound(version);
                        }
                        else if( newSystem > Distance.getUpperSystemBound(version) )
                        {
                            newSystem -= Distance.getUpperSystemBound(version);
                        }
                        placeHolder = new Planet( null, new Position( originPosition.getGalaxy(), newSystem, 0 ) );
                        savePlanet = configData.getPlanet( placeHolder );
                        fav = savePlanet;
                        if( savePlanet == null || !savePlanet.isMine() )
                        {
                            newSystem = originPosition.getSystem() + systemDiff;
                            // check for wrap around
                            if( newSystem < Distance.lowerSystemBound )
                            {
                                newSystem += Distance.getUpperSystemBound(version);
                            }
                            else if( newSystem > Distance.getUpperSystemBound(version) )
                            {
                                newSystem -= Distance.getUpperSystemBound(version);
                            }

                            savePlanet = new Planet( null, new Position( originPosition.getGalaxy(), newSystem, 0 ) );
                            if( configData.hasPlanet( savePlanet ) )
                            {
                                savePlanet = configData.getPlanet( savePlanet );
                            }
                            else if (fav != null )
                            {
                                savePlanet = fav;
                            }
                        }
                    }
                    newTarget = new SaveTarget( origin, savePlanet, selectedTarget.isLongDistance(), selectedTarget.getSpeed(), selectedTarget.getVelocity(), selectedTarget.getUnit(), selectedTarget.getDuration(), selectedTarget.getRequestedDuration(), selectedTarget.getAccount() );
                }
                allSaves.addRow( newTarget );
            }
        }
    }

    private static long interval = 120;
    private void addAll2()
    {
        int selectedRow = summaryTable.getSelectedRow();
        SaveTarget selectedTarget = currentSaves.getRow( selectedRow );

        for( Planet planet : configData.getMyPlanets() )
        {
            if( selectedTarget.getOrigin().equals( planet ) )
            {
                allSaves.addRow( selectedTarget );
            }
            else
            {
                // calculate similar flights for the rest
                FleetSpeed maxSpeed = selectedTarget.getSpeed();
                String unit = selectedTarget.getUnit();
                long duration = selectedTarget.getDuration();
                List<SaveTarget> targets = new ArrayList<SaveTarget>( calculator.getSaveTargets2( planet, maxSpeed, unit, duration, current ) );
                // sort by time and find appropriate flight
                Collections.sort( targets, new SaveTargetComparator( SaveTargetComparatorMode.SORT_BY_DELTA ) );
                if( targets.size() > 0 )
                {
                    // preselect best match regardless of time delta
                    SaveTarget best = targets.get( 0 );
                    int index = 0;
                    while( targets.size() > ++index )
                    {
                        // find a match with lower velocity inside the interval
                        SaveTarget target = targets.get( index );
                        if( target.getDelta() > interval )
                        {
                            break;
                        }
                        if( target.getVelocity() < best.getVelocity() )
                        {
                            best = target;
                        }
                    }
                    allSaves.addRow( best );
                }
            }
        }

    }

    protected void actionDeleteSaves()
    {
        int[] rows = selectedTargets.getSelectedRows();
        if( selectedTargets.isEditing() )
        {
            selectedTargets.removeEditor();
        }
        selectedTargets.clearSelection();
        allSaves.removeRows( rows );
    }

    protected void actionSelectClosest()
    {
        long delta = Long.MAX_VALUE;
        int rowIndex = -1;
        List<SaveTarget> data = currentSaves.getData();
        for( int i = 0; i < data.size(); i++ )
        {
            SaveTarget target = data.get( i );
            long l = target.getDelta();
            if( l < delta )
            {
                rowIndex = i;
                delta = l;
            }
            else if ( ( l == delta )
                            && ( rowIndex >= 0 )
                            && ( target.getVelocity() < data.get( rowIndex ).getVelocity() ) )
            {
                rowIndex = i;
            }
        }
        if( rowIndex >= 0 )
        {
            summaryTable.getSelectionModel().setSelectionInterval( rowIndex, rowIndex );
            // show newly selected row
            // since scrollRectToVisible() will just show the rect add some padding
            int lower = Math.max( rowIndex - 4, 0 );
            int upper = Math.min( rowIndex + 4, data.size() );
            Rectangle lRect = summaryTable.getCellRect( lower, 0, true );
            Rectangle uRect = summaryTable.getCellRect( upper, 0, true );
            Rectangle r = lRect.union( uRect );
            summaryTable.scrollRectToVisible(r);
        }
    }

    private void updateMyPlanets()
	{
        Object selection = myPlanets.getSelectedItem();
        myPlanets.removeAllItems();
        int size = 0;
        myPlanets.addItem( new Planet( "-- " + configData.getI18nText( "Savecalc.ownPlanets" ) + "  --", new Position( 0, 0, 0) ) );
        size++;
        List<Planet> ownPlanets = new ArrayList<Planet>( configData.getMyPlanets( true, false ) );
        for( Planet planet : ownPlanets )
        {
            myPlanets.addItem( planet );
            size++;
        }
        List<Planet> asteroids = new ArrayList<Planet>( configData.getMyPlanets( false, true ) );
        if( asteroids.size() > 0 )
        {
            myPlanets.addItem( new Planet( "-- " + configData.getI18nText( "MainFrame.asteroids" ) + "  --", new Position( 0, 0, 0) ) );
            size++;
            for( Planet asteroid : asteroids )
            {
                myPlanets.addItem( asteroid );
                size++;
            }
        }
        // restore selection
        if( selection != null )
        {
            myPlanets.setSelectedItem( selection );
        }
        else
        {
            myPlanets.setSelectedIndex( 0 );
        }
        myPlanets.setMaximumRowCount( size );
	}

	/**
     * @param e
     */
    protected void actionComboBoxChanged( ItemEvent e )
    {
        if( e.getStateChange() == ItemEvent.SELECTED )
        {
            Planet planet = ( Planet )e.getItem();
            Position position = planet.getPosition();
            launchPosition.setPosition( position );
            FleetSpeed lowestSpeed  = planet.getFleet().getFleetVelocity( configData.getVersion(), configData.getMyTech(), configData.getSpecialization(), 0 );
            configData.setVelocity( lowestSpeed );
            setVelocity( lowestSpeed );
        }
    }

    private void adjustTableColumns()
    {
        TableColumnModel model = summaryTable.getColumnModel();
        int tableColumnWidths[] = new int[] { 25, 50, 96, 94, 94 };
        // lets lock columns 2-6
        for( int column = 2; column < model.getColumnCount(); column++ )
        {
            TableColumn tableColumn = model.getColumn( column );
            int width = tableColumnWidths[column-2];
            tableColumn.setPreferredWidth( width );
            tableColumn.setMinWidth( width );
            tableColumn.setMaxWidth( width );
        }
        model = selectedTargets.getColumnModel();
        for( int column = 2; column < model.getColumnCount(); column++ )
        {
            TableColumn tableColumn = model.getColumn( column );
            int width = tableColumnWidths[column-2];
            tableColumn.setPreferredWidth( width );
            tableColumn.setMinWidth( width );
            tableColumn.setMaxWidth( width );
        }
    }

    /**
     *
     */
    protected void actionCalculate()
    {
        try
        {
            // get start planet
            Position position = launchPosition.getPosition();
            if( position.isRealPosition() )
            {
                Planet origin = configData.getPlanetOnPosition( position );
                Position astroPosition = new Position( position, false );
                astroPosition.setAsteroidPosition( true );
                Planet myPlanet = (Planet)myPlanets.getSelectedItem();
                if( astroPosition.equals( myPlanet.getPosition() ) )
                {
                    origin.setName( myPlanet.getName() );
                    origin.getPosition().setAsteroidPosition( true );
                }
                myPlanets.setSelectedItem( origin );
                FleetSpeed maxSpeed = configData.getVelocity();
                String unit = configData.getUnitFromMaxSpeed( maxSpeed );
                long duration = Calculator.parseTime( saveTime.getDay(), saveTime.getHour(), saveTime.getMinute() );
                // use only half the time b/c we need to get back again!
                duration /= 2;
                // set back the time for reading
                setDuration( duration );
                if( duration != 0 && maxSpeed.getVelocity() > 0 )
                {
                    currentSaves.setData( new ArrayList<SaveTarget>() );
                    Account current = configData.getActiveAccount();
                    List<SaveTarget> targets = new ArrayList<SaveTarget>();
                    if( configData.getVersion() != Version.v20 )
                    {
                        targets.addAll( calculator.getSaveTargets( origin, maxSpeed, unit, duration, current ) );
                    }
                    else
                    {
                        targets.addAll( calculator.getSaveTargets2( origin, maxSpeed, unit, duration, current ) );
                    }
                    Collections.sort( targets, new SaveTargetComparator( SaveTargetComparatorMode.SORT_BY_DURATION ) );
                    currentSaves.setData( targets );
                }
            }
        }
        catch( Exception ex )
        {
        	ex.printStackTrace();
        }

    }

    @Override
    public String getFileName()
    {
        StringBuilder sb = new StringBuilder();
        sb.append( "flights-" );
        sb.append( fileFormat.format( new Date() ) );
        return sb.toString();
    }

    @Override
    public String getExtension()
    {
        return extension;
    }

    @Override
    public File getCurrentDir()
    {
        return configData.getAccountDir();
    }

    @Override
    public void setCurrentDir( File currentDir )
    {
        configData.setAccountDir( currentDir );
    }

    @Override
    public Document getData( Parser xmlParser )
    {
        Document doc = null;
        if( xmlParser != null )
        {
            doc = xmlParser.getDocument();
            Element rootEl = doc.createElement( Parser.Tag_Module_SaveCalc );
            doc.appendChild( rootEl );

            List<SaveTarget> saves = allSaves.getData();
            for( SaveTarget target : saves )
            {
                Element saveTargetNode = doc.createElement( Parser.Tag_SaveTarget );

                saveTargetNode.setAttribute( Parser.Attribute_Origin, target.getOrigin().toString() );
                saveTargetNode.setAttribute( Parser.Attribute_Target, target.getPlanet().toString() );
                saveTargetNode.setAttribute( Parser.Attribute_Distance, Boolean.toString(target.isLongDistance() ) );
                saveTargetNode.setAttribute( Parser.Attribute_Duration, Long.toString( target.getDuration() ) );
                saveTargetNode.setAttribute( Parser.Attribute_Speed, Integer.toString( target.getVelocity() ) );
                saveTargetNode.setAttribute( Parser.Attribute_Ship, target.getUnit() );
                saveTargetNode.setAttribute( Parser.Attribute_Units, formatUnits( target.getSpeed().getShips() ) );
                saveTargetNode.setAttribute( Parser.Attribute_Account, target.getAccount().getId() );
                saveTargetNode.setAttribute( Parser.Attribute_Version, target.getAccount().getVersion().name() );
                saveTargetNode.setAttribute( Parser.Attribute_Specialization_Fleeter, Integer.toString(target.getAccount().getSpecialization().getFleeter() ) );
                saveTargetNode.setAttribute( Parser.Attribute_Rocket, Integer.toString(target.getAccount().getMyTech().getRocket() ) );
                saveTargetNode.setAttribute( Parser.Attribute_Diffusion, Integer.toString(target.getAccount().getMyTech().getDiffusion() ) );
                saveTargetNode.setAttribute( Parser.Attribute_Warp, Integer.toString(target.getAccount().getMyTech().getWarp() ) );

                rootEl.appendChild( saveTargetNode );
            }
        }
        return doc;
    }

    protected void actionLoad()
    {
        Document doc = configData.readData( this );
        if( doc != null )
        {
            NodeList targets = doc.getElementsByTagName( Parser.Tag_SaveTarget );
            if( targets != null )
            {
                for( int i = 0; i < targets.getLength(); i++ )
                {
                    Element saveNode = (Element)targets.item( i );
                    Planet origin = parsePlanet( saveNode.getAttribute( Parser.Attribute_Origin ) );
                    Planet target = parsePlanet( saveNode.getAttribute( Parser.Attribute_Target ) );
                    boolean longDistance = ConfigData.parseBooleanAttribute( saveNode.getAttribute( Parser.Attribute_Distance ) );
                    Ship[] ships = parseUnits( saveNode.getAttribute( Parser.Attribute_Units ) );
                    Version version = ConfigData.parseVersionAttribute( saveNode.getAttribute( Parser.Attribute_Version ) );
                    // sanity check
                    if(ships[FleetSpeed.PREPERATIONTIME] != null) {
                        version = Version.v20;
                    }
                    int percent = ConfigData.parseIntegerAttribute( saveNode.getAttribute( Parser.Attribute_Speed ) );
                    String id = saveNode.getAttribute( Parser.Attribute_Account );
                    String unit = saveNode.getAttribute( Parser.Attribute_Ship );
                    Specialization specialization = new Specialization();
                    specialization.setFleeter( ConfigData.parseIntegerAttribute( saveNode.getAttribute( Parser.Attribute_Specialization_Fleeter ) ) );
                    Technology tech = new Technology();
                    tech.setRocket(ConfigData.parseIntegerAttribute( saveNode.getAttribute( Parser.Attribute_Rocket ) ) );
                    tech.setDiffusion(ConfigData.parseIntegerAttribute( saveNode.getAttribute( Parser.Attribute_Diffusion ) ) );
                    tech.setWarp(ConfigData.parseIntegerAttribute( saveNode.getAttribute( Parser.Attribute_Warp ) ) );
                    Account current = configData.lookupAccount( id );
                    Accountable account = (current != null) ? current : new SaveTarget.AccountLight( id, version, tech, specialization );
                    FleetSpeed speed = parseFleetSpeed( ships, account );

                    long duration = 0;
                    long[] distance = getDistance( origin.getPosition(), target.getPosition(), version, current );
                    if( distance.length > 1)
                    {
                        duration = getDuration( speed, distance[longDistance ? 1 : 0], percent, account.getFleetSpeed(), version );
                    }
                    else
                    {
                        duration = getDuration( speed, distance[0], percent, account.getFleetSpeed(), version );
                    }
                    SaveTarget saveTarget = new SaveTarget( origin, target, longDistance, speed, percent, unit, duration, 0, account );
                    allSaves.addRow( saveTarget );
                }
            }
        }
    }

    protected void actionSave()
    {
        configData.saveData( this );
    }

    private static String delimiter = ":";
    private String formatUnits( Ship[] ships )
    {
        StringBuilder sb = new StringBuilder();
        sb.append( (ships[FleetSpeed.PREPERATIONTIME] != null) ? ships[FleetSpeed.PREPERATIONTIME].name() : "" ).append( delimiter );
        sb.append( (ships[FleetSpeed.ACCELERATION] != null) ? ships[FleetSpeed.ACCELERATION].name() : "" ).append( delimiter );
        sb.append( ships[FleetSpeed.VELOCITY].name() );
        return sb.toString();
    }

    private Ship[] parseUnits( String units )
    {
        String[] unit = units.split( delimiter );
        Ship[] ships = new Ship[unit.length];
        for( int i = 0; i < unit.length; i++ )
        {
            ships[i] = (unit[i].length() > 0)? Ship.valueOf( unit[i] ) : null;
        }
        return ships;
    }

    private Planet parsePlanet( String value )
    {
        Position position = new Position( value );

        String[] parts = value.split( "]" );
        String name = ( parts.length > 1 ) ? parts[1].trim() : "";
        return new Planet( name, position );
    }

    private FleetSpeed parseFleetSpeed( Ship[] ships, Accountable account )
    {
        Fleet fleet = new Fleet();
        for( Ship ship : ships )
        {
            if( ship != null )
            {
                fleet.setCount( ship, 1l );
            }
        }
        return fleet.getFleetVelocity( account.getVersion(), account.getMyTech(), account.getSpecialization(), 0 );
    }
}
