/****************************************
 *  COPYRIGHT (C) 2008-2024
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.Icon;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.EnumWrapper;
import sitool.components.ProductionBonus;
import sitool.components.Specialization;
import sitool.components.Version;
import sitool.i18n.Language;



/**
 * @author graf
 *
 */
public class AccountTableModel extends AbstractTableModel implements MouseListener
{
    private static final long serialVersionUID = -1405529512618782948L;

    List<Account> data = new ArrayList<Account>();
    private final Icon upArrow;
    private final Icon downArrow;

    protected MainFrame mainFrame;

    public final static int NAME = 0;
    public final static int DEFAULT = 1;
    public final static int LANGUAGE = 2;
    public final static int SPEED = 3;
    public final static int FLEETSPEED = 4;
    public final static int SPEEDUP = 5;
    public final static int SPECIALIZATION = 6;
    public final static int PRODUCTIONBONUS = 7;
    public final static int SPECTRAL = 8;
    public final static int VERSION = 9;
    public final static int OUTPOST = 10;
    public final static int IMPORT_PLANETS = 11;
    public final static int IMPORT_SPY_REPORTS = 12;
    public final static int DOWN = 13;
    public final static int UP = 14;
    private final static int LAST = UP;

    public AccountTableModel( MainFrame mainFrame )
    {
        this.mainFrame = mainFrame;
        upArrow = ConfigData.getIcon( "/button_up.gif" );
        downArrow = ConfigData.getIcon( "/button_down.gif" );
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        return LAST + 1;
    }


    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @Override
    public String getColumnName( int columnIndex )
    {
        String rv ="";
        ConfigData cd = mainFrame.getConfigData();

        switch( columnIndex )
        {
            case NAME:
                rv = cd.getI18nText( "Account.name" );
                break;
            case DEFAULT:
                rv = cd.getI18nText( "Account.isdefault" );
                break;
            case LANGUAGE:
                rv = cd.getI18nText( "Account.language" );
                break;
            case SPEED:
                rv = cd.getI18nText( "Account.speed" );
                break;
            case FLEETSPEED:
                rv = cd.getI18nText( "Account.fleetSpeed" );
                break;
            case SPEEDUP:
                rv = cd.getI18nText( "Account.speedUp" );
                break;
            case SPECIALIZATION:
                rv = cd.getI18nText( "Account.specialization" );
                break;
            case PRODUCTIONBONUS:
                rv = cd.getI18nText( "Account.productionBonus" );
                break;
            case SPECTRAL:
                rv = cd.getI18nText( "Account.spectral" );
                break;
            case VERSION:
                rv = cd.getI18nText( "Account.version" );
                break;
            case OUTPOST:
                rv = cd.getI18nText( "Account.outpost" );
                break;
            case IMPORT_PLANETS:
                rv = cd.getI18nText( "Account.importPlanets" );
                break;
            case IMPORT_SPY_REPORTS:
                rv = cd.getI18nText( "Account.importSpyReports" );
                break;
            case DOWN:
            case UP:
                rv = "";
                break;
        }
        return rv;
    }


    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return data.size();
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Account target = data.get( rowIndex );
        ConfigData cd = mainFrame.getConfigData();
        Object rv = null;
        switch( columnIndex )
        {
            case NAME:
                rv = target.getId();
                break;
            case DEFAULT:
                rv = target.isDefault();
                break;
            case LANGUAGE:
                rv = cd.getI18nText( target.getLanguage().name() );
                break;
            case SPEED:
                rv = target.getSpeedFactor();
                break;
            case FLEETSPEED:
                rv = target.getFleetSpeed();
                break;
            case SPEEDUP:
                rv = target.getSpeedUp();
                break;
            case SPECIALIZATION:
                rv = target.getSpecialization();
                break;
            case PRODUCTIONBONUS:
                rv = target.getProductionBonus();
                break;
            case SPECTRAL:
                rv = target.isSpectralMode();
                break;
            case VERSION:
                rv = cd.getI18nText( target.getVersion().name() );
                break;
            case OUTPOST:
                rv = target.isOutpost();
                break;
            case IMPORT_PLANETS:
                rv = target.isMayImportPlanetaryOverview();
                break;
            case IMPORT_SPY_REPORTS:
                rv = target.isMayImportSpyReports();
                break;
            case DOWN:
                if( rowIndex < data.size() - 1 )
                {
                    rv = downArrow;
                }
                break;
            case UP:
                if( rowIndex > 0 )
                {
                    rv = upArrow;
                }
                break;
        }
        return rv;
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        boolean rv = true;

        if( columnIndex == UP || columnIndex == DOWN )
        {
            rv = false;
        }
        if( columnIndex == SPECIALIZATION || columnIndex == PRODUCTIONBONUS || columnIndex == IMPORT_PLANETS
         || columnIndex == IMPORT_SPY_REPORTS || columnIndex == OUTPOST)
        {
            rv = (data.get( rowIndex ).getVersion() != Version.v20);
        }
        if(columnIndex == SPECTRAL) {
            rv = (data.get( rowIndex ).getVersion().isSpectral());
        }

        return rv;
    }


    @SuppressWarnings("unchecked")
    @Override
    public void setValueAt(Object value, int rowIndex, int columnIndex)
    {
        Account target = data.get( rowIndex );
        switch( columnIndex )
        {
            case NAME:
                target.setId( value.toString() );
                break;
            case DEFAULT:
                boolean newDefault = Boolean.parseBoolean( value.toString() );
                if( newDefault )
                {
                    // reset others
                    for( Account account : data )
                    {
                        account.setDefault( false );
                    }
                }
                target.setDefault( newDefault );
                fireTableDataChanged();
                break;
            case LANGUAGE:
                target.setLanguage( ( (EnumWrapper<Language>)value).getBase() );
                break;
            case SPEED:
                target.setSpeedFactor( ((Number)value).doubleValue() );
                break;
            case FLEETSPEED:
                target.setFleetSpeed( ((Number)value).doubleValue() );
                break;
            case SPEEDUP:
                target.setSpeedUp( ((Number)value).doubleValue() );
                break;
            case SPECIALIZATION:
                Specialization specialization = (Specialization)value;
                target.getSpecialization().setValue(specialization);
                break;
            case PRODUCTIONBONUS:
                ProductionBonus bonus = (ProductionBonus)value;
                target.getProductionBonus().setValue(bonus);
                break;
            case SPECTRAL:
                target.setSpectralMode((Boolean)value);
                break;
            case VERSION:
                target.setVersion( ( (EnumWrapper<Version>)value).getBase() );
                break;
            case OUTPOST:
                target.setOutpost((Boolean)value);
                break;
            case IMPORT_PLANETS:
                {
                    boolean b = ((Boolean)value).booleanValue();
                    target.setMayImportPlanetaryOverview(b);
                    if(b && target.isMayImportSpyReports()) {
                        target.setMayImportSpyReports(false);
                    }
                    fireTableCellUpdated(rowIndex, IMPORT_PLANETS);
                    fireTableCellUpdated(rowIndex, IMPORT_SPY_REPORTS);
                }
                break;
            case IMPORT_SPY_REPORTS:
            {
                boolean b = ((Boolean)value).booleanValue();
                target.setMayImportSpyReports(b);
                if(b && target.isMayImportPlanetaryOverview()) {
                    target.setMayImportPlanetaryOverview(false);
                }
                fireTableCellUpdated(rowIndex, IMPORT_PLANETS);
                fireTableCellUpdated(rowIndex, IMPORT_SPY_REPORTS);
            }
                break;
        }
    }

    public List<Account> getData()
    {
        return new ArrayList<Account>( data );
    }

    public void setData( List<Account> data )
    {
        this.data.clear();
        for( Account account : data )
        {
            this.data.add( new Account( account ) );
        }

        fireTableDataChanged();
    }

    public void addRow( Account target )
    {
        int lastRow = data.size();
        data.add( target );
        fireTableRowsInserted( lastRow, lastRow );
    }

    public void removeRows( int[] rows )
    {
        // since deleting rows will change the indices we'll get the objects first
        List<Account> rows2delete = new ArrayList<Account>();
        for( int row :rows )
        {
            rows2delete.add( data.get( row ) );
        }
        data.removeAll( rows2delete );
        fireTableDataChanged();
    }

    public List<Account> getRows( int[] rows )
    {
        List<Account> result = new ArrayList<Account>();
        for( int row :rows )
        {
            result.add( data.get( row ) );
        }
        return result;
    }

    public boolean hasDefault()
    {
        boolean rv = false;
        for( Account account : data )
        {
            if( account.isDefault() )
            {
                rv = true;
                break;
            }
        }
        return rv;
    }

    public void mouseClicked( MouseEvent e )
    {
        Object o = e.getSource();
        if( o instanceof JTable )
        {
            JTable table = (JTable)o;
            int row = table.rowAtPoint(e.getPoint());
            int col = table.columnAtPoint(e.getPoint());
            // convert to model
            col = table.convertColumnIndexToModel( col );
            if ( SwingUtilities.isLeftMouseButton(e) && e.getClickCount() == 1 )
            {
                if( col == UP || col == DOWN )
                {
                    int start = -1;
                    int stop = -1;
                    switch( col )
                    {
                        case UP:
                            start = row - 1;
                            stop = row + 1;
                            break;
                        case DOWN:
                            start = row;
                            stop = row + 2;
                            break;
                    }

                    if( start >= 0 && stop <= data.size() )
                    {
                        // switch them over
                        Collections.rotate( data.subList( start, stop ), 1 );
                        fireTableRowsUpdated( start, stop - 1 );
                        if( col == UP)
                        {
                            table.getSelectionModel().setSelectionInterval( start, start );
                        }
                        else
                        {
                            table.getSelectionModel().setSelectionInterval( stop - 1, stop - 1 );
                        }
                    }
                }
            }
        }
    }

    public void mouseEntered( MouseEvent e )
    {}

    public void mouseExited( MouseEvent e )
    {}

    public void mousePressed( MouseEvent e )
    {}

    public void mouseReleased( MouseEvent e )
    {}
}
