/****************************************
 *  COPYRIGHT (C) 2008-2017
 *  Holger Graf
 ****************************************/
package sitool.table;

import sitool.MainFrame;
import sitool.components.Building;
import sitool.components.Cost;
import sitool.components.Planet;
import sitool.components.Resource;
import sitool.components.Stage;
import sitool.components.Version;
import sitool.resources.ResourceCalculator;

/**
 * @author graf
 *
 */
public class BuildingStatisticsTableModel extends StatisticsTableModel
{
    private static final long serialVersionUID = -327932871342205186L;

    private int maxRows = 0;
    private Building building = null;
    private int temperature = 0;
    private int devCenterLevel = 0;
    private int msaLevel = 0;
    private Planet planet;
    private boolean astro;
    private boolean isConstructorEnabled = false;

    public BuildingStatisticsTableModel( MainFrame mainFrame )
    {
        super( mainFrame );
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        int rv = 1;
        if( building != null )
        {
            switch( building )
            {
                case researchLab:
                    rv = 4;
                    break;
                case fusionPlant:
                case ironStorage:
                case metalStorage:
                case kryptoniteStorage:
                case spiceStorage:
                case developmentCenter:
                case fleetBase:
                case weaponsFactory:
                case msa:
                case tunnel:
                case superComputer:
                    rv = 5;
                    break;
                case ironMine:
                case kryptoniteMine:
                case spiceMine:
                    rv = 6;
                    break;
                case furnace:
                case thermosolarPlant:
                case particleCannon:
                case particleShield:
                case teleporter:
                case scanner:
                    rv = 7;
                    break;
            }
        }

        return rv;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @SuppressWarnings("incomplete-switch")
    @Override
    public String getColumnName( int column )
    {
        String rv = "";
        if( column == 0 )
        {
            rv = formatHeader( configData.getI18nText( "Level" ) );
        }
        else if( building != null )
        {
            Cost cost = building.getCost();
            String parts[];
            Resource resource;
            switch( column )
            {
                case 1:
                    resource = getFirstResource( cost );
                    if( resource != null )
                    {
                        parts = configData.getI18nText( resource.name() ).split( " ", 2 );
                        rv = formatHeader( parts );
                    }
                    break;
                case 2:
                    resource = getSecondResource( cost );
                    if( resource != null )
                    {
                        parts = configData.getI18nText( resource.name() ).split( " ", 2 );
                        rv = formatHeader( parts );
                    }
                    break;
                case 3:
                    resource = getThirdResource( cost );
                    if( resource != null )
                    {
                        parts = configData.getI18nText( resource.name() ).split( " ", 2 );
                        rv = formatHeader( parts );
                    }
                    else
                    {
                        parts = configData.getI18nText( "Buildtime" ).split( " ", 2 );
                        rv = formatHeader( parts );
                    }
                    break;
                case 4:
                    resource = getFourthResource( cost );
                    if( resource != null )
                    {
                        parts = configData.getI18nText( resource.name() ).split( " ", 2 );
                        rv = formatHeader( parts );
                    }
                    else
                    {
                        switch( building )
                        {
                            case ironMine:
                            case kryptoniteMine:
                                parts = configData.getI18nText( "Production.p.h" ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                            case spiceMine:
                                parts = configData.getI18nText( "Production.p.h.temp", temperature ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                            case fusionPlant:
                                parts = configData.getI18nText( "Energyproduction.p.h" ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                            case furnace:
                                parts = configData.getI18nText( "Metalproduction.p.h" ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                            case thermosolarPlant:
                            case developmentCenter:
                            case weaponsFactory:
                            case fleetBase:
                            case msa:
                            case tunnel:
                            case superComputer:
                                parts = configData.getI18nText( "Buildtime" ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                            case ironStorage:
                            case metalStorage:
                            case kryptoniteStorage:
                            case spiceStorage:
                                parts = configData.getI18nText( "Capacity" ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                        }
                    }
                    break;
                case 5:
                    switch( building )
                    {
                        case ironMine:
                        case kryptoniteMine:
                        case spiceMine:
                            parts = configData.getI18nText( "Energyconsumption.p.h" ).split( " ", 2 );
                            rv = formatHeader( parts );
                            break;
                        case furnace:
                            parts = configData.getI18nText( "Ironconsumption.p.h" ).split( " ", 2 );
                            rv = formatHeader( parts );
                            break;
                        case thermosolarPlant:
                            parts = configData.getI18nText( "Energyproduction.p.h" ).split( " ", 2 );
                            rv = formatHeader( parts );
                            break;
                        case particleCannon:
                        case particleShield:
                        case teleporter:
                        case scanner:
                            parts = configData.getI18nText( "Buildtime" ).split( " ", 2 );
                            rv = formatHeader( parts );
                            break;
                    }
                    break;
                case 6:
                    switch( building )
                    {
                        case furnace:
                            parts = configData.getI18nText( "Energyconsumption.p.h" ).split( " ", 2 );
                            rv = formatHeader( parts );
                            break;
                        case thermosolarPlant:
                            parts = configData.getI18nText( "Spiceconsumption.p.h" ).split( " ", 2 );
                            rv = formatHeader( parts );
                            break;
                        case particleCannon:
                        case particleShield:
                        case teleporter:
                            parts = configData.getI18nText( "Particle.buffer" ).split( " ", 2 );
                            rv = formatHeader( parts );
                            break;
                        case scanner:
                            parts = configData.getI18nText( "Range" ).split( " ", 2 );
                            rv = formatHeader( parts );
                            break;
                    }
                    break;
            }
        }
        return rv;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return maxRows;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    @SuppressWarnings("incomplete-switch")
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Object rv = null;
        rv = "";

        if( building != null )
        {
            int level;
            Version version = configData.getVersion();
            switch( building )
            {
                case ironStorage:
                case metalStorage:
                case kryptoniteStorage:
                case spiceStorage:
                    if( rowIndex == 0 && columnIndex > 0 && columnIndex < 4 )
                    {
                        return "-";
                    }
                    level = rowIndex;
                    break;
                default:
                    level = rowIndex + 1;
                    break;
            }
            if( columnIndex == 0 )
            {
                rv = Integer.valueOf( level );
            }
            else
            {
                Cost cost = ResourceCalculator.calculateCost( building, level );
                Resource resource;
                switch( columnIndex )
                {
                    case 1:
                        resource = getFirstResource( cost );
                        if( resource != null )
                        {
                            rv = format( cost.getResource( resource ) );
                        }
                        break;
                    case 2:
                        resource = getSecondResource( cost );
                        if( resource != null )
                        {
                            rv = format( cost.getResource( resource ) );
                        }
                        break;
                    case 3:
                        resource = getThirdResource( cost );
                        if( resource != null )
                        {
                            rv = format( cost.getResource( resource ) );
                        }
                        else
                        {
                            rv = formatTime( ResourceCalculator.buildTime( cost, isConstructorEnabled, version ) / configData.getSpeedFactor(Building.class) );
                        }
                        break;
                    case 4:
                        resource = getFourthResource( cost );
                        if( resource != null )
                        {
                            rv = format( cost.getResource( resource ) );
                        }
                        else
                        {
                            switch( building )
                            {
                                case ironMine:
                                case kryptoniteMine:
                                case spiceMine:
                                    rv = format( Math.round( configData.getSpeedFactor(Resource.class) * ResourceCalculator.calculateProduction( building, level, temperature, version ) ) );
                                    break;
                                case fusionPlant:
                                    rv = format( Math.round( ResourceCalculator.calculateProduction( building, level, temperature, version ) ) );
                                    break;
                                case furnace:
                                    rv = format( Math.round( configData.getSpeedFactor(Resource.class) * ResourceCalculator.calculateProduction( building, level, 0, version ) ) );
                                    break;
                                case thermosolarPlant:
                                case developmentCenter:
                                case weaponsFactory:
                                case fleetBase:
                                case msa:
                                case tunnel:
                                    rv = formatTime( ResourceCalculator.buildTime( cost, isConstructorEnabled, version ) / configData.getSpeedFactor(Building.class) );
                                    break;
                                case superComputer:
                                    double constructor = (isConstructorEnabled) ? 0.9 : 1.0;
                                    rv = super.formatTime( constructor * 24.0 * 60.0 * 60.0 * 1000.0 / configData.getSpeedFactor(Building.class) );
                                    break;
                                case ironStorage:
                                case metalStorage:
                                case kryptoniteStorage:
                                case spiceStorage:
                                    rv = format( ResourceCalculator.calculateCapacity( building, level ) );
                                    break;
                            }
                        }
                        break;
                    case 5:
                        switch( building )
                        {
                            case ironMine:
                            case kryptoniteMine:
                            case spiceMine:
                                rv = format( Math.round( ResourceCalculator.calculateEnergyConsumption( building, level ) ) );
                                break;
                            case furnace:
                                rv = format( Math.round( configData.getSpeedFactor(Resource.class) * ResourceCalculator.calculateProduction( building, level, 0, version ) ) * 2l );
                                break;
                            case thermosolarPlant:
                                rv = format( Math.round( ResourceCalculator.calculateProduction( building, level, 0, version ) ) );
                                break;
                            case particleCannon:
                            case particleShield:
                            case teleporter:
                            case scanner:
                                rv = formatTime( ResourceCalculator.buildTime( cost, isConstructorEnabled, version ) / configData.getSpeedFactor(Building.class) );
                                break;
                        }
                        break;
                    case 6:
                        switch( building )
                        {
                            case furnace:
                                rv = format( Math.round(ResourceCalculator.calculateEnergyConsumption( building, level )) );
                                break;
                            case thermosolarPlant:
                                rv = format( Math.round( ResourceCalculator.calculateConsumption( building, level ) ) );
                                break;
                            case particleCannon:
                                rv = format( 20000L * level );
                                break;
                            case particleShield:
                                rv = format( 30000L * level );
                                break;
                            case teleporter:
                                rv = format( 50000L * level );
                                break;
                            case scanner:
                                if( rowIndex == 0 )
                                {
                                    rv = configData.getI18nText( "one.system" );
                                }
                                else
                                {
                                    int v = level * level / 2 - 1;
                                    if( v > 400 )
                                    {
                                        v = 400;
                                    }
                                    rv = "+/- " + Integer.valueOf( v );
                                }
                                break;
                        }
                        break;
                }
            }
        }
        return rv;
    }

    public boolean markResource( int row, int column )
    {
        int rv = 0;
        if( planet != null && building != null )
        {
            Stage stage = planet.getStage( building, astro );
            if( stage != null )
            {
                rv = stage.getLevel();
            }
            // bump up the stores by 1
            switch( building )
            {
                case ironStorage:
                case metalStorage:
                case kryptoniteStorage:
                case spiceStorage:
                    rv++;
                    break;
                default:
                    break;
            }
        }
        return ( rv == row + 1 );
    }

    public void setMaxRows( int maxRows )
    {
        this.maxRows = maxRows;
        fireTableDataChanged();
    }

    public void setBuilding( Building building )
    {
        this.building = building;
        fireTableStructureChanged();
    }


    public void setTemperature( int temperature )
    {
        this.temperature = temperature;
        fireTableStructureChanged();
    }

    public void setDevCenterLevel( int devCenterLevel )
    {
        this.devCenterLevel = devCenterLevel;
        fireTableDataChanged();
    }

    public void setMsaLevel( int msaLevel )
    {
        this.msaLevel = msaLevel;
        fireTableDataChanged();
    }

    public Planet getPlanet()
    {
        return planet;
    }

    public void setPlanet( Planet planet )
    {
        this.planet = planet;
    }

    public void setConstructor( boolean isConstructorEnabled )
    {
        this.isConstructorEnabled = isConstructorEnabled;
        fireTableDataChanged();
    }

    public void setAstro( boolean astro )
    {
        this.astro = astro;
    }

    protected String formatTime( double buildTime )
    {
        int activeDevCenterLevel = devCenterLevel + 1;
        // DevC in V2.0 is reduced by 1
        if( configData.getVersion() == Version.v20 && activeDevCenterLevel > 1 )
        {
            activeDevCenterLevel--;
        }
        buildTime /= ( activeDevCenterLevel );
        buildTime = Math.floor( buildTime );
        if(devCenterLevel >= 12) {
            buildTime /= Math.pow( 2, msaLevel );
            buildTime = Math.floor( buildTime );
        }

        return super.formatTime( buildTime );
    }
}
