/****************************************
 *  COPYRIGHT (C) 2012
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.table.AbstractTableModel;

import sitool.Account;
import sitool.ConfigData;
import sitool.components.Coordinates;
import sitool.components.Position;
import sitool.components.SystemData;

/**
 * @author graf
 *
 */
public class CoordinatorTableModel extends AbstractTableModel
{
    private static final long serialVersionUID = 2675717223585468107L;

    public static final int ROW = 0;
    public static final int POSITION = 1;
    private static final int COORDINATES = 2;
    private static final int TEMPERATURE = 3;

    private ConfigData configData;
    private List<Entry<Position, Coordinates>> data;

    private int galaxyTicks = 0;
    private int ticks = 0;;


    public CoordinatorTableModel( ConfigData configData )
    {
        this.configData = configData;
        data = new ArrayList<Entry<Position, Coordinates>>();
        Account current = configData.getActiveAccount();
        if( current != null )
        {
            galaxyTicks = current.getGalaxyTicks();
            ticks = current.getTicks();
        }
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    @Override
    public int getRowCount()
    {
        return data.size();
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    @Override
    public int getColumnCount()
    {
        return TEMPERATURE + 1;
    }


    @Override
    public Class<?> getColumnClass( int columnIndex )
    {
        if( columnIndex == 0 )
            return Number.class;
        return String.class;
    }

    @Override
    public String getColumnName( int columnIndex )
    {
        String rv ="";
        switch( columnIndex )
        {
            case POSITION:
                rv = configData.getI18nText( "Planet.position" );
                break;
            case TEMPERATURE:
                rv = configData.getI18nText( "Planet.temperature" );
                break;
            case COORDINATES:
                rv = configData.getI18nText( "Planet.coordinates" );
                break;
        }
        return rv;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    @Override
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Object rv = "";
        Entry<Position, Coordinates> entry = data.get( rowIndex );
        Coordinates translated;
        switch( columnIndex )
        {
            case ROW:
                rv = rowIndex + 1;
                break;
            case POSITION:
                rv = entry.getKey();
                break;
            case COORDINATES:
                translated = configData.translate( entry.getValue(), galaxyTicks, ticks );
                rv = translated;
                break;
            case TEMPERATURE:
                translated = configData.translate( entry.getValue(), galaxyTicks, ticks );
                rv = SystemData.getTemperature( translated );
                break;
        }
        return rv;
    }

    public void update()
    {
        data.clear();
        Account current = configData.getActiveAccount();
        if( current != null )
        {
            Map<Position, Coordinates> map = configData.getCoordinates();
            data.addAll( map.entrySet() );
            Collections.sort( data, configData.getEntryComparator() );
        }
        configData.clearTranslationCache();
        fireTableDataChanged();
    }

    public void setTicks( int galaxyTicks, int ticks )
    {
        this.galaxyTicks = galaxyTicks;
        this.ticks = ticks;
        configData.clearTranslationCache();
        fireTableDataChanged();
    }

    public void filter( String regex )
    {
        data.clear();
        Pattern pattern = Pattern.compile( regex );
        Matcher m;
        Account current = configData.getActiveAccount();
        if( current != null )
        {
            Map<Position, Coordinates> map = configData.getCoordinates();
            for( Entry<Position, Coordinates> entry : map.entrySet() )
            {
                m = pattern.matcher( entry.getValue().toFullString() );
                if( m.find() )
                {
                    data.add( entry );
                }
            }
            Collections.sort( data, configData.getEntryComparator() );
        }
        fireTableDataChanged();
    }

}
