/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Coordinates;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.Version;

/**
 * @author graf
 *
 */
public class PlanetTableModel extends AbstractTableModel implements MouseListener
{
    private static final long serialVersionUID = -1405529512618782948L;

    private MainFrame mainFrame;
    private Icon upArrow;
    private Icon downArrow;

    public final static int NAME = 0;
    public final static int POSITION = 1;
    public final static int SIZE = 2;
    public final static int TEMPERATURE = 3;
    public final static int OWN = 4;
    public final static int ASTROSIZE = 5;
    public final static int COORDINATES = 6;
    public final static int DOWN = 7;
    public final static int UP = 8;

    public PlanetTableModel( MainFrame mainFrame )
    {
        this.mainFrame = mainFrame;
        upArrow = ConfigData.getIcon( "/button_up.gif" );
        downArrow = ConfigData.getIcon( "/button_down.gif" );
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        return UP + 1;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @Override
    public String getColumnName( int columnIndex )
    {
        ConfigData configData = mainFrame.getConfigData();
        String rv ="";
        switch( columnIndex )
        {
            case NAME:
                rv = configData.getI18nText( "Planet.name" );
                break;
            case POSITION:
                rv = configData.getI18nText( "Planet.position" );
                break;
            case SIZE:
                rv = configData.getI18nText( "Planet.size" );
                break;
            case TEMPERATURE:
                rv = configData.getI18nText( "Planet.temperature" );
                break;
            case OWN:
                rv = configData.getI18nText( "Planet.own" );
                break;
            case ASTROSIZE:
                rv = configData.getI18nText( "Planet.astrosize" );
                break;
            case COORDINATES:
                rv = configData.getI18nText( "Planet.coordinates" );
                break;
            case DOWN:
            case UP:
                rv = "";
                break;
        }
        return rv;
    }


    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return mainFrame.getConfigData().getPlanets().size();
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        ConfigData configData = mainFrame.getConfigData();
        List<Planet> data = configData.getPlanets();
        Planet target = data.get( rowIndex );
        Object rv = "";
        switch( columnIndex )
        {
            case NAME:
                rv = target.getName();
                break;
            case POSITION:
                rv = target.getPosition();
                break;
            case SIZE:
                rv = target.getSize();
                break;
            case TEMPERATURE:
                rv = target.getTemperature(configData.getVersion());
                break;
            case OWN:
                rv = target.isMine();
                break;
            case ASTROSIZE:
                rv = target.getAstroSize();
                break;
            case COORDINATES:
                rv = target.getCoordinates();
                break;
            case DOWN:
                if( rowIndex < data.size() - 1 )
                {
                    rv = downArrow;
                }
                break;
            case UP:
                if( rowIndex > 0 )
                {
                    rv = upArrow;
                }
                break;
        }
        return rv;
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        boolean rv;
        List<Planet> data = mainFrame.getConfigData().getPlanets();
        Planet target = data.get( rowIndex );
        // name, size and ownership are editable
        rv = ( columnIndex != COORDINATES && columnIndex != TEMPERATURE && columnIndex < DOWN );
        if( columnIndex == SIZE || columnIndex == ASTROSIZE )
        {
            rv = target.isMine();
        }
        return rv;
    }


    @Override
    public void setValueAt(Object value, int rowIndex, int columnIndex)
    {
        List<Planet> data = mainFrame.getConfigData().getPlanets();
        Planet target = data.get( rowIndex );
        switch( columnIndex )
        {
            case NAME:
                target.setName( value.toString() );
                break;
            case POSITION:
                target.setPosition(new Position(value.toString()));
                adjustCoordinates(target);
                fireTableRowsUpdated(rowIndex, rowIndex);
                break;
            case SIZE:
                target.setSize( Integer.parseInt( value.toString() ) );
                break;
            case OWN:
                target.setMine( Boolean.parseBoolean( value.toString() ) );
                break;
            case ASTROSIZE:
                target.setAstroSize( Integer.parseInt( value.toString() ) );
                break;
        }
    }

    public Planet addOrUpdateRow( Planet target )
    {
        List<Planet> data = mainFrame.getConfigData().getPlanets();
        int updatedRow;
        Planet rv = null;
        if( data.contains( target ) )
        {
            updatedRow = 0;
            for( Planet planet : data )
            {
                if( planet.equals( target ) )
                {
                    // update
                    rv = planet;
                    if( target.getAstroSize() == 0 )
                    {
                        planet.setName( target.getName() );
                    }
                    else
                    {
                        planet.setAstroSize( target.getAstroSize() );
                    }
                    planet.setSize( target.getSize() );
                    planet.setMine( target.isMine() );
                    fireTableRowsUpdated( updatedRow, updatedRow );
                    break;
                }
                updatedRow++;
            }
        }
        else
        {
            rv = target;
            adjustCoordinates( target );
            updatedRow = data.size();
            data.add( target );
            fireTableRowsInserted( updatedRow, updatedRow );
        }

        return rv;
    }

    public void removeRows( int[] rows )
    {
        List<Planet> data = mainFrame.getConfigData().getPlanets();
        // since deleting rows will change the indices we'll get the objects first
        List<Planet> rows2delete = new ArrayList<Planet>();
        for( int row :rows )
        {
            rows2delete.add( data.get( row ) );
        }
        data.removeAll( rows2delete );
        fireTableDataChanged();
    }

    public void mouseClicked( MouseEvent e )
    {
        Object o = e.getSource();
        if( o instanceof JTable )
        {
            JTable table = (JTable)o;
            int row = table.rowAtPoint(e.getPoint());
            int col = table.columnAtPoint(e.getPoint());
            // convert to model
            col = table.convertColumnIndexToModel( col );
            if ( SwingUtilities.isLeftMouseButton(e) && e.getClickCount() == 1 )
            {
                if( col == UP || col == DOWN )
                {
                    List<Planet> data = mainFrame.getConfigData().getPlanets();
                    int start = -1;
                    int stop = -1;
                    switch( col )
                    {
                        case UP:
                            start = row - 1;
                            stop = row + 1;
                            break;
                        case DOWN:
                            start = row;
                            stop = row + 2;
                            break;
                    }

                    if( start >= 0 && stop <= data.size() )
                    {
                        // switch them over
                        Collections.rotate( data.subList( start, stop ), 1 );
                        fireTableRowsUpdated( start, stop - 1 );
                        if( col == UP)
                        {
                            table.getSelectionModel().setSelectionInterval( start, start );
                        }
                        else
                        {
                            table.getSelectionModel().setSelectionInterval( stop - 1, stop - 1 );
                        }
                    }
                }
            }
        }
    }

    public void mouseEntered( MouseEvent e )
    {}

    public void mouseExited( MouseEvent e )
    {}

    public void mousePressed( MouseEvent e )
    {}

    public void mouseReleased( MouseEvent e )
    {}

    public void update() {
        fireTableDataChanged();
    }

    private void adjustCoordinates( Planet planet )
    {
        ConfigData configData = mainFrame.getConfigData();
        if( configData.getVersion() == Version.v20 )
        {
            Account account = configData.getActiveAccount();
            Map<Position, Coordinates> map = configData.getCoordinates();
            if( map.size() > 0 )
            {
                int ticks = account.getTicks();
                int galaxyTicks = account.getGalaxyTicks();
                Coordinates coordinates = map.get( planet.getPosition() );
                if( coordinates != null )
                {
                    // enter current coordinates
                    planet.setCoordinates( configData.translate( coordinates, galaxyTicks, ticks ) );
                }
                else
                {
                    planet.setCoordinates( new Coordinates( "" ) );
                }
            }
        }
    }
}
