/****************************************
 *  COPYRIGHT (C) 2008-2022
 *  Holger Graf
 ****************************************/
package sitool.table;

import sitool.Account;
import sitool.MainFrame;
import sitool.components.Cost;
import sitool.components.Research;
import sitool.components.Resource;
import sitool.components.Technology;
import sitool.technology.TechCalculator;

/**
 * @author graf
 *
 */
public class ResearchStatisticsTableModel extends StatisticsTableModel
{
    private static final long serialVersionUID = -327932871342205186L;

    private static final long speedupCutoff = 1000l * 60l * 60l * 24l * 80l;

    private Research research;
    private int maxRows;
    private int recLabLevel;

    public ResearchStatisticsTableModel( MainFrame mainFrame )
    {
        super( mainFrame );
        this.maxRows = 10;
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        int rv = 1;
        if( research != null )
        {
            switch( research )
            {
                case drive:
                case structure:
                    rv = 4;
                    break;
                case mining:
                case graviton:
                case shields:
                case target:
                case espionage:
                case weapons:
                case particle:
                case teleportation:
                case blackhole:
                    rv = 5;
                    break;
                case energy:
                case rocket:
                case diffusion:
                case warp:
                    rv = 6;
                    break;
            }
        }

        return rv;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @SuppressWarnings("incomplete-switch")
    @Override
    public String getColumnName( int column )
    {
        String rv = "";
        if( column == 0 )
        {
            rv = formatHeader( configData.getI18nText( "Level" ) );
        }
        else if( research != null )
        {
            Cost cost = research.getCost();
            Resource resource;
            String parts[];
            switch( column )
            {
                case 1:
                    resource = getFirstResource( cost );
                    if( resource != null )
                    {
                        parts = configData.getI18nText( resource.name() ).split( " ", 2 );
                        rv = formatHeader( parts );
                    }
                    break;
                case 2:
                    resource = getSecondResource( cost );
                    if( resource != null )
                    {
                        parts = configData.getI18nText( resource.name() ).split( " ", 2 );
                        rv = formatHeader( parts );
                    }
                    else
                    {
                        switch( research )
                        {
                            case structure:
                                parts = configData.getI18nText( "Researchtime" ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                        }
                    }
                    break;
                case 3:
                    resource = getThirdResource( cost );
                    if( resource != null )
                    {
                        parts = configData.getI18nText( resource.name() ).split( " ", 2 );
                        rv = formatHeader( parts );
                    }
                    else
                    {
                        switch( research )
                        {
                            case mining:
                            case drive:
                            case weapons:
                            case shields:
                                parts = configData.getI18nText( "Researchtime" ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                            case structure:
                                parts = configData.getI18nText( "header.bonus.structure" ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                        }
                    }
                    break;
                case 4:
                    resource = getFourthResource( cost );
                    if( resource != null )
                    {
                        parts = configData.getI18nText( resource.name() ).split( " ", 2 );
                        rv = formatHeader( parts );
                    }
                    else
                    {
                        switch( research )
                        {
                            case mining:
                                parts = configData.getI18nText( "header.bonus.mining" ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                            case energy:
                            case rocket:
                            case diffusion:
                            case warp:
                            case espionage:
                            case target:
                            case particle:
                            case graviton:
                            case teleportation:
                            case blackhole:
                                parts = configData.getI18nText( "Researchtime" ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                            case weapons:
                                parts = configData.getI18nText( "header.bonus.weapons" ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                            case shields:
                                parts = configData.getI18nText( "header.bonus.shields" ).split( " ", 2 );
                                rv = formatHeader( parts );
                                break;
                        }
                    }
                    break;
                case 5:
                    switch( research )
                    {
                        case energy:
                            parts = configData.getI18nText( "header.bonus.energy" ).split( " ", 2 );
                            rv = formatHeader( parts );
                            break;
                        case rocket:
                        case diffusion:
                        case warp:
                            parts = configData.getI18nText( "header.bonus.drive" ).split( " ", 2 );
                            rv = formatHeader( parts );
                            break;
                    }
                    break;
            }
        }
        return rv;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return maxRows;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    @SuppressWarnings("incomplete-switch")
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Object rv = null;
        int level = rowIndex + 1;
        rv = "";
        if( columnIndex == 0 )
        {
            rv = Integer.valueOf( level );
        }
        else if( research != null )
        {
            Cost cost = TechCalculator.currentCost( level, research.getCost() );
            Resource resource;
            switch( columnIndex )
            {
                case 1:
                    resource = getFirstResource( cost );
                    if( resource != null )
                    {
                        rv = format( cost.getResource( resource ) );
                    }
                    break;
                case 2:
                    resource = getSecondResource( cost );
                    if( resource != null )
                    {
                        rv = format( cost.getResource( resource ) );
                    }
                    else
                    {
                        switch( research )
                        {
                            case structure:
                                rv = formatTime( TechCalculator.researchTime( cost ) );
                                break;
                        }
                    }
                    break;
                case 3:
                    resource = getThirdResource( cost );
                    if( resource != null )
                    {
                        rv = format( cost.getResource( resource ) );
                    }
                    else
                    {
                        switch( research )
                        {
                            case mining:
                            case drive:
                            case weapons:
                            case shields:
                                rv = formatTime( TechCalculator.researchTime( cost ) );
                                break;
                            case structure:
                                rv = "+" + 10 * level + "%";
                                break;
                        }
                    }
                    break;
                case 4:
                    resource = getFourthResource( cost );
                    if( resource != null )
                    {
                        rv = format( cost.getResource( resource ) );
                    }
                    else
                    {
                        switch( research )
                        {
                            case mining:
                                rv = "+" + 2 * level + "%";
                                break;
                            case energy:
                            case rocket:
                            case diffusion:
                            case warp:
                            case espionage:
                            case target:
                            case particle:
                            case graviton:
                            case teleportation:
                            case blackhole:
                                rv = formatTime( TechCalculator.researchTime( cost ) );
                                break;
                            case weapons:
                            case shields:
                                rv = "+" + 10 * level + "%";
                                break;
                        }
                    }
                    break;
                case 5:
                    switch( research )
                    {
                        case energy:
                            rv = "+" + level + "%";
                            break;
                        case rocket:
                        case diffusion:
                        case warp:
                            rv = "+" + 10 * level + "%";
                            break;
                    }
                    break;
            }
        }
        return rv;
    }

    public boolean markResource( int row, int column )
    {
        int rv = 0;
        Technology technology = mainFrame.getConfigData().getMyTech();
        rv = technology.getTech( research );
        return ( rv == row + 1 );
    }

    public void setMaxRows( int maxRows )
    {
        this.maxRows = maxRows;
        fireTableDataChanged();
    }

    public void setResearch( Research research )
    {
        this.research = research;
        fireTableStructureChanged();
    }

    public void setRecLabLevel( int level )
    {
        this.recLabLevel = level;
        fireTableDataChanged();
    }

    protected String formatTime( double buildTime )
    {
        double speedFactor = 1.0;
        double speedUp = 1.0;
        Account current = configData.getActiveAccount();
        if( current != null )
        {
            speedFactor = current.getSpeedFactor();
            speedUp = current.getSpeedUp();
        }
        if(buildTime < speedupCutoff) {
            buildTime /= speedUp;
        }
        buildTime /= speedFactor;
        if( recLabLevel > 0 )
        {
            buildTime /= recLabLevel;
            buildTime = Math.floor( buildTime );
        }
        long researchCap = getResearchCap();
        buildTime = (researchCap > 0 && buildTime > researchCap) ? researchCap : Math.floor( buildTime );

        return super.formatTime( buildTime );
    }
}
