/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import sitool.Account;
import sitool.MainFrame;
import sitool.components.Building;
import sitool.components.Cost;
import sitool.components.Defense;
import sitool.components.Percent;
import sitool.components.Planet;
import sitool.components.ProductionBonus;
import sitool.components.Resource;
import sitool.components.Ship;
import sitool.components.Specialization;
import sitool.components.Stage;
import sitool.components.Technology;
import sitool.components.TotalPointDisplay;
import sitool.components.Version;
import sitool.resources.ResourceCalculator;

/**
 * @author graf
 *
 */
public class ResourceTableModel extends StatisticsTableModel implements AlienTableModelExtension
{
    private static final long serialVersionUID = -1405529512618782948L;

    private double time_multiplier;
    private double multipleProduction = 1.0;
    private int allianceBonus = 0;
    private int allianceSpecialization = 0;
    private int[] productionBonus = new int[Resource.values().length + 1];
    private boolean u_mod;
    private Map<Planet, int[]> utilization = new HashMap<Planet, int[]>();
    private boolean[] link;
    private Map<Planet, Boolean> aliens = new HashMap<Planet, Boolean>();
    private Technology tech;
    private StockpileTableModel stocks = null;

    private static final int IRON = 1;
    private static final int METAL = 3;
    private static final int KRYPTONITE = 5;
    private static final int SPICE = 7;
    private static final int SKIP = 9;
    private static final int THERMO = 10;
    private static final int ENERGY = 11;

    static final String EclipsedValue = "-  ";

    public ResourceTableModel( MainFrame mainFrame )
    {
    	super(mainFrame);
        link = new boolean[Resource.values().length + 1];
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        return 2 * Resource.values().length + 4;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @Override
    public String getColumnName( int columnIndex )
    {
        String rv ="";
        switch (columnIndex) {
            case 0:
            case SKIP:
                break;
            case ENERGY:
                rv = configData.getI18nText( "Energy" );
                break;
            case THERMO:
                rv = configData.getI18nText( Building.thermosolarPlant.name() );
                break;
            default:
                if( columnIndex % 2 == 0) {
                    rv = configData.getI18nText( "Efficiency" );
                }
                else {
                    rv = configData.getI18nText( Resource.values()[columnIndex / 2].name() );
                }
        }
        return rv;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return configData.getMyPlanets().size() + 3;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        List<Planet> data = configData.getMyPlanets();
        Planet planet;
        long production = 0;
        Object rv = "";

        if( columnIndex == 0 )
        {
            if( rowIndex == data.size() )
            {
                rv = configData.getI18nText( "Sum" );
            }
            else if( rowIndex == data.size() + 1 )
            {
                rv = configData.getI18nText( "resourceequivalenz" );
            }
            else if( rowIndex == data.size() + 2 )
            {
                rv = configData.getI18nText( "Resources.productionBonus" );
            }
            else
            {
                planet = data.get( rowIndex );
                rv = planet.toString();
            }
        }
        else if( columnIndex == getColumnCount() - 1 )
        {
            if( rowIndex < data.size() )
            {
                planet = data.get( rowIndex );
                rv = format( getEnergy( planet, rowIndex ) );
            }
            else if( rowIndex == data.size() + 2 )
            {
                rv = productionBonus[productionBonus.length - 1];
            }
        }
        else if( columnIndex % 2 == 0 )
        {
            if( rowIndex == data.size() )
            {
                rv = link[columnIndex/2-1];
            }
            else if( rowIndex == data.size() + 1 )
            {
                rv = "";
            }
            else if( rowIndex == data.size() + 2 )
            {
                if(columnIndex <= SPICE + 1)
                {
                    rv = productionBonus[columnIndex/2-1];
                }
                else if (columnIndex == ENERGY)
                {
                    rv = productionBonus[productionBonus.length - 1];
                }
            }
            else
            {
                if( isBuildingPresent( rowIndex, columnIndex ) )
                {
                    planet = data.get( rowIndex );
                    int[] uArray = getUArray( planet );
                    rv = new Percent( uArray[columnIndex/2-1] );
                }
                else
                {
                    rv = EclipsedValue;
                }
            }
        }
        else if( columnIndex / 2 < Resource.values().length )
        {
            int planetCount = data.size();
            if( rowIndex == planetCount )
            {
                // sum
                production = getTotalProduction( columnIndex );
                rv = format( production );
            }
            else if( rowIndex == planetCount + 1 )
            {
                long iron = getTotalProduction( IRON );
                long metal = getTotalProduction( METAL );
                long kryptonite = getTotalProduction( KRYPTONITE );
                long spice = getTotalProduction( SPICE );
                // sum
                double value = getWeightedResourceEquivalenz( new Cost( iron, metal, kryptonite, spice ) );
                int weight = Resource.values()[columnIndex/2].getWeight();
                rv = format( Math.round( weight * value ) );
            }
            else if( rowIndex == planetCount + 2 )
            {
                rv = "";
            }
            // row 0 in stockpile is in flight resources
            else if( StockpileTableModel.EclipsedValue.equals( stocks.getValueAt( rowIndex + 1, columnIndex ) ) )
            {
                rv = EclipsedValue;
            }
            else
            {
                production = getProduction( rowIndex, columnIndex );
                rv = format( production );
            }
        }
        return rv;
    }

    @Override
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
        boolean rv = false;
        if( columnIndex > 0 && columnIndex % 2 == 0 && columnIndex < getColumnCount() - 1 && rowIndex < getRowCount() - 1 )
        {
            if( rowIndex == configData.getMyPlanets().size() )
            {
                rv = true;
            }
            else
            {
                rv = isBuildingPresent( rowIndex, columnIndex );
            }
        }
        if( rowIndex == getRowCount() - 1 )
        {
            rv = (columnIndex == getColumnCount() - 1) || (columnIndex <= SPICE + 1);
        }
        return rv;
	}


    @Override
    public void setValueAt( Object value, int rowIndex, int columnIndex )
    {
        if( rowIndex == getRowCount() - 1)
        {
            if(( columnIndex % 2 == 0 ) && ( columnIndex <= SPICE + 1 ) )
            {
                productionBonus[columnIndex/2 - 1] = ((Number)value).intValue();
            }
            else if( columnIndex == getColumnCount() - 1 )
            {
                productionBonus[productionBonus.length - 1] = ((Number)value).intValue();
            }
            fireTableDataChanged();
        }
        else if( columnIndex % 2 == 0 )
        {
            List<Planet> data = configData.getMyPlanets();
            int planetCount = data.size();
            Planet planet;
            if( rowIndex == planetCount )
            {
                link[columnIndex/2-1] = Boolean.valueOf( value.toString() );
            }
            else
            {
                int percent = ((Percent)value).getValue();
                if( link[columnIndex/2-1] )
                {
                    for( int i = 0; i < planetCount; i++ )
                    {
                        planet = data.get( i );
                        int[] uArray = getUArray( planet );
                        uArray[columnIndex/2-1] = percent;
                    }
                }
                else
                {
                    planet = data.get( rowIndex );
                    int[] uArray = getUArray( planet );
                    uArray[columnIndex/2-1] = percent;
                }
                updateUtilization();
                fireTableDataChanged();
            }
        }
    }

    public void setMultiplier( double multiplier, boolean u_mod )
    {
        this.time_multiplier = multiplier;
        if( this.u_mod != u_mod )
        {
            this.u_mod = u_mod;
            adjustUtilization();
        }
        updateUtilization();
        fireTableDataChanged();
    }

    public void setMultipleProduction( double multipleProduction, int productionBonus, int allianceSpecialization )
    {
        this.multipleProduction = multipleProduction;
        this.allianceBonus = productionBonus;
        this.allianceSpecialization = allianceSpecialization;
        updateUtilization();
        fireTableDataChanged();
    }

    public void update()
    {
        tech = configData.getMyTech();
        updateUtilization();
        updateProductionBonus();
        fireTableStructureChanged();
    }

    private void updateUtilization() {
        double speedUp = configData.getUtilizationSpeedUp();
        // utilization counts all in SI2
        if(configData.getVersion() == Version.v20) {
            speedUp = powerUtilizationFactor(Version.v20);
        }
        for( Planet planet : utilization.keySet() ) {
            int[] uArray = getUArray( planet );
            int maxUtil = (u_mod) ? 25 : (int)(speedUp * (isAlien(planet) ? 80.0 : 100.0));
            for(int index = 0; index < uArray.length; index++) {
                int util = Math.min(uArray[index], maxUtil);
                uArray[index] = util;
            }
        }
    }

    public void setStocks( StockpileTableModel stocks )
    {
        this.stocks = stocks;
    }

    protected long getTotalProduction( int columnIndex )
    {
        long production = 0;
        for( int i = 0; i < configData.getMyPlanetsAndAsteroids().size() + 1; i++ )
        {
            if( StockpileTableModel.EclipsedValue.equals( stocks.getValueAt( i, columnIndex ) ) )
            {
                continue;
            }
            long p;
            // row 0 in stockpile is in flight resources
            if( i > 0 && i <= configData.getMyPlanets().size() )
            {
                p = getProduction( i - 1, columnIndex );
            }
            else
            {
                // since asteroids don't produce anything simply add them to the bottom line
                p = getValue( stocks.getValueAt( i, columnIndex ) );
            }
            production += p;
        }
        return production;
    }

    protected double getProduction(int rowIndex, int columnIndex, boolean productionOnly) {
        final Planet planet = configData.getMyPlanets().get( rowIndex );
        final Resource resource = Resource.values()[columnIndex/2];
        final Version version = configData.getVersion();
        final Account current = configData.getActiveAccount();
        final Specialization specialization = configData.getSpecialization();
        final int temperature = planet.getTemperature(version);
        final int spectral = configData.isSpectralMode() ? 4 : 1;

        Building building;
        Stage stage;
        double production;
        double baseProduction = 0.0;
        double consumption;
        double baseSpeed = 1.0;
        double bonusSpeed = 0.0;
        boolean mineTech = true;
        int utilization = getUArray(planet)[resource.ordinal()];

        if( current != null )
        {
            baseSpeed = current.getSpeedFactor();
            // any kind of multi-production above base speed is handled with utilization in SI2
            if(version != Version.v20) {
                // multiple production is additive, not multiplicative
                // 5x bonus (= 1 + 4) and 2x speedup (= 1 + 1) == 6x (1 + 5)
                bonusSpeed = current.getSpeedUp() - 1.0 + multipleProduction - 1.0;
            }
        }
        // get stock (row 0 in stockpile is in flight resources)
        long stock = getValue( stocks.getValueAt( rowIndex + 1, columnIndex ) );
        switch(resource) {
            case iron:
                building = Building.ironMine;
                // pig iron has a base production in Classic
                baseProduction = (version != Version.v20) ? 30.0 : 0;
                break;
            case metal:
                building = Building.furnace;
                mineTech = false;
                break;
            case kryptonite:
                building = Building.kryptoniteMine;
                // kryptonite has a base production in Classic
                baseProduction = (version != Version.v20) ? 15.0 : 0;
                break;
            case spice:
                building = Building.spiceMine;
                break;
            default:
                building = null;
                break;
        }
        // base production
        baseProduction *= baseSpeed;
        stage = planet.getStage( building, false );
        double raw_production = (stage != null) ? stage.getProduction(temperature, version) : 0.0;
        production = baseSpeed * raw_production;
        if(version != Version.v20) {
            production = baseSpeed * Math.round(raw_production);
            production = Math.round(production);
            raw_production = production / baseSpeed;
        }
        // mine tech
        if(mineTech) {
            double techBonus = raw_production * 2.0 * tech.getMining() / 100.0;
            if(version != Version.v20) {
                techBonus = Math.floor(techBonus);
            }
            production += baseSpeed * techBonus;
        }
        // base production, comes after mine tech
        production += baseProduction;
        // uni speed
        production *= (1 + bonusSpeed);
        // utilization
        production = production * utilization / 100.0;
        // boni, anyone?
        // double arithmetic is imprecise, shift to integer
        long bonus = ((long)production) * (
                                            specialization.getProductionbonus()
                                            + allianceBonus
                                            + ((mineTech) ? allianceSpecialization : 0)
                                            + productionBonus[resource.ordinal()]
                                           ) / 100;
        production += bonus;
        // how much is it?
        production = Math.floor(time_multiplier * production / spectral);
        // add stock
        production += stock;

        //consumption/overuse
        switch(resource) {
            case iron:
                double metalProduction = getProduction(rowIndex, METAL, true);
                production -= 2* metalProduction;
                break;
            case metal:
                if(!productionOnly) {
                    double ironProduction = getProduction(rowIndex, IRON, true);
                    if( ironProduction < 0 ) {
                        production += ironProduction / 2;
                    }
                }
                else {
                    // metal productionOnly is w/o stock
                    production -= stock;
                }
                break;
            case kryptonite:
                break;
            case spice:
                building = Building.thermosolarPlant;
                stage = planet.getStage( building, false );
                consumption = (stage != null) ? stage.getConsumption() : 0.0;
                if(version != Version.v20) {
                    consumption = Math.round(consumption);
                }
                // thermosolar utilization
                utilization = getUArray(planet)[Resource.values().length];
                consumption *= utilization / 100.0;
                consumption = time_multiplier * Math.floor( consumption ) / spectral;
                production -= consumption;
        }
        if(!productionOnly && production < 0.0) {
            production = 0.0;
        }
        return production;
    }

    protected long getProduction( int rowIndex, int columnIndex )
    {
        return (long)getProduction(rowIndex, columnIndex, false);
    }

    protected long getEnergy( Planet planet, int rowIndex )
    {
        double rv = 0;
        int[] uArray = getUArray( planet );
        Building building;
        Stage stage;
        double bonus;
        double utilization;

        Version version = configData.getVersion();
        int temperature = planet.getTemperature(version);

        // sum of energy
        // fusion plant
        building = Building.fusionPlant;
        stage = planet.getStage( building, false );
        if( stage != null )
        {
            rv = stage.getProduction( temperature, version );
            if( u_mod )
            {
                rv *= 0.25;
            }
            else {
                if( isAlien( planet ) )
                {
                    rv *= 0.80;
                }
                rv *= powerUtilizationFactor(version);
            }
        }
        // energotech applies only to the unaugmented fusion power
        // keep the bonus for later
        bonus = Math.floor(rv * productionBonus[productionBonus.length-1] / 100.0);
        rv = Math.floor(rv);
        // thermosolar plant
        building = Building.thermosolarPlant;
        stage = planet.getStage( building, false );
        if( stage != null )
        {
            utilization = uArray[Resource.values().length] / 100.0;
            double solar = stage.getProduction( temperature, version );
            rv += Math.floor( solar * utilization );
        }
        // add solsats
        long solarCells;
        long solarCellEnergy;
        if(version != Version.v20)
        {
            solarCells = planet.getFleet( false ).getCount( Ship.solarCell );
            solarCellEnergy = solarCells * ResourceCalculator.calculateSolarSatProduction( temperature, version );
        }
        else
        {
            solarCells = planet.getFleet( false ).getCount( Defense.solarCell );
            solarCellEnergy = (long)(solarCells * ResourceCalculator.calculateSolarSatProduction( temperature, version ) * powerUtilizationFactor(version));
        }
        rv += solarCellEnergy;
        // research bonus
        rv *= ( 1.0 + tech.getEnergy() / 100.0 );
        // now apply the bonus
        rv += bonus;
        rv = Math.floor(rv);

        // mine consumption
        double consumption;
        building = Building.ironMine;
        stage = planet.getStage( building, false );
        if( stage != null )
        {
            utilization = uArray[Resource.iron.ordinal()] / 100.0;
            consumption = ResourceCalculator.calculateEnergyConsumption(building, stage.getLevel() );
            rv -= Math.floor( consumption * utilization );
        }
        building = Building.furnace;
        stage = planet.getStage( building, false );
        if( stage != null )
        {
            utilization = uArray[Resource.metal.ordinal()] / 100.0;
            consumption = ResourceCalculator.calculateEnergyConsumption(building, stage.getLevel() );
            rv -= Math.floor( consumption * utilization );
        }
        building = Building.kryptoniteMine;
        stage = planet.getStage( building, false );
        if( stage != null )
        {
            utilization = uArray[Resource.kryptonite.ordinal()] / 100.0;
            consumption = ResourceCalculator.calculateEnergyConsumption(building, stage.getLevel() );
            rv -= Math.floor( consumption * utilization );
        }
        building = Building.spiceMine;
        stage = planet.getStage( building, false );
        if( stage != null )
        {
            utilization = uArray[Resource.spice.ordinal()] / 100.0;
            consumption = ResourceCalculator.calculateEnergyConsumption(building, stage.getLevel() );
            rv -= Math.floor( consumption * utilization );
        }

        return (long)Math.floor(rv);
    }

    // SI2 power is scaled by multiproduction
    public double powerUtilizationFactor( Version version ) {
        double utilization = 1.0;
        if(version == Version.v20) {
            utilization = getSpeedFactor(configData.getSpeedFactor(Resource.class),  configData.getUtilizationSpeedUp());
        }
        return utilization;
    }

    protected String format( long value )
    {
        return TotalPointDisplay.format( value );
    }

    public boolean markResource( int row, int column )
    {
        boolean rv = false;
        if( row < configData.getMyPlanets().size() )
        {
            List<Planet> data = configData.getMyPlanets();
            Planet planet = data.get( row );
            if( column == getColumnCount() - 1 )
            {
                rv = ( getEnergy(planet, row) < 0 );
            }
            else if( column % 2 != 0 &&  column / 2 < Resource.values().length )
            {
                Resource resource = Resource.values()[column/2];
                long production = getProduction( row, column );
                long capacity;
                Stage stage = null;
                switch( resource )
                {
                    case iron:
                        stage = planet.getStage( Building.ironStorage, false );
                        break;
                    case metal:
                        stage = planet.getStage( Building.metalStorage, false );
                        break;
                    case kryptonite:
                        stage = planet.getStage( Building.kryptoniteStorage, false );
                        break;
                    case spice:
                        stage = planet.getStage( Building.spiceStorage, false );
                        break;
                }
                capacity = ( stage != null ) ? stage.getCapacity() : 100000;
                rv = ( production >= capacity );
            }
        }
        return rv;
    }

    public int[] getUtilization(Planet planet)
    {
        return getUArray(planet);
    }

    public void setUtilization( Map<Planet, int[]> utilization )
    {
        this.utilization = utilization;
        updateUtilization();
        fireTableDataChanged();
    }

    private void updateProductionBonus() {
        ProductionBonus bonus = new ProductionBonus();
        Account account = configData.getActiveAccount();
        if(account != null) {
            bonus = account.getProductionBonus();
        }
        this.productionBonus[0] = bonus.getIron();
        this.productionBonus[1] = bonus.getMetal();
        this.productionBonus[2] = bonus.getKryptonite();
        this.productionBonus[3] = bonus.getSpice();
        this.productionBonus[4] = bonus.getEnergy();
        fireTableDataChanged();
    }

    private long getValue( Object value )
    {
        long rv = 0;
        StringBuffer sb = new StringBuffer();
        sb.append( value.toString() );
        for( int i = 0; i < sb.length(); i++ )
        {
            char ch = sb.charAt( i );
            if( !Character.isDigit( ch ) )
            {
                sb.deleteCharAt( i );
            }
        }
        if( sb.length() > 0 )
        {
            rv = Long.valueOf( sb.toString() );
        }
        return rv;
    }

    private void adjustUtilization()
    {
        double speedUp = configData.getUtilizationSpeedUp();
        for( int[] uArray : utilization.values() )
        {
            fillUtilization(uArray, speedUp, false);
        }
    }

    /**
     * Return and, if necessary initialize utilization array for planet
     * @param planet
     * @return
     */
    private int[] getUArray( Planet planet )
    {
        int[] uArray = utilization.get( planet );
        if( uArray == null )
        {
            uArray = new int[Resource.values().length + 1];
            fillUtilization(uArray, configData.getUtilizationSpeedUp(), isAlien(planet));
            utilization.put( planet, uArray );
        }
        return uArray;
    }

    private void fillUtilization(int[] uArray, double speedUp, boolean isAlien) {
        Arrays.fill( uArray, (u_mod) ? 25 : (int)( ((isAlien) ? 80 : 100) * speedUp));
    }

    private boolean isBuildingPresent( int rowIndex, int columnIndex )
    {
        boolean rv = false;
        List<Planet> data = mainFrame.getConfigData().getMyPlanets();
        if( columnIndex > 0 && columnIndex % 2 == 0 && rowIndex < data.size() )
        {
            Planet planet = data.get( rowIndex );
            Stage stage = null;
            int level;
            switch( columnIndex / 2 - 1 )
            {
                case 0:
                    stage = planet.getStage( Building.ironMine, false );
                    break;
                case 1:
                    stage = planet.getStage( Building.furnace, false );
                    break;
                case 2:
                    stage = planet.getStage( Building.kryptoniteMine, false );
                    break;
                case 3:
                    stage = planet.getStage( Building.spiceMine, false );
                    break;
                case 4:
                    stage = planet.getStage( Building.thermosolarPlant, false );
                    break;
            }
            level = ( stage != null ) ? stage.getLevel() : 0;
            rv = (level > 0);
        }
        return rv;
    }

    @Override
    public boolean isAlien( int rowIndex )
    {
        Planet planet = configData.getMyPlanets().get( rowIndex );
        return isAlien( planet );
    }

    private boolean isAlien( Planet planet )
    {
        Boolean alien = aliens.get( planet );
        return ( alien != null ) ? alien.booleanValue() : false;
    }

    @Override
    public void toggleAlien( int rowIndex )
    {
        Planet planet = configData.getMyPlanets().get( rowIndex );
        boolean alien = !isAlien( planet );
        aliens.put( planet,  alien);
        if( alien )
        {
            double speedUp = powerUtilizationFactor(configData.getVersion());
            int util[] = utilization.get( planet );
            for( int i = 0; i < util.length; i++ )
            {
                util[i] = Math.min( (int)(speedUp * 80), util[i] );
            }
        }
        fireTableRowsUpdated( rowIndex, rowIndex );
    }

    public Map<Planet, Boolean> getAliens()
    {
        return aliens;
    }

    public void setAliens( Map<Planet, Boolean> aliens )
    {
        this.aliens = aliens;
        fireTableDataChanged();
    }

    private double getSpeedFactor(double speedFactor, double speedUp) {
        // multiple production is additive, not multiplicative
        // 5x bonus (= 1 + 4) and 2x speedup (= 1 + 1) == 6x (1 + 5)
        double bonus = speedUp + multipleProduction - 1.0;
        return speedFactor * bonus;
    }
}
