/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.SystemData;
import sitool.components.Version;
import sitool.savecalc.Calculator;
import sitool.savecalc.SaveTarget;


/**
 * @author graf
 *
 */
public class SaveTargetTableModel extends AbstractTableModel implements SortableTableModel, ResourceMarker
{
    private static final long serialVersionUID = -806190722320456592L;
    private MainFrame mainFrame;
    private ConfigData configData;
    private Calculator calculator;
    List<SaveTarget> data = new ArrayList<SaveTarget>();
    private boolean edit;
    private int sortColumn = -1;
    private SortOrder sortOrder = SortOrder.None;
    private SaveTargetComparator comparator = new SaveTargetComparator( SaveTargetComparatorMode.SORT_BY_DURATION );

    private final RowHeaderTableModel rowHeaderModel;
    private final JTable rowHeaderTable;

    public SaveTargetTableModel( MainFrame mainFrame, boolean edit )
    {
        this.mainFrame = mainFrame;
        this.configData = mainFrame.getConfigData();
        this.edit = edit;
        calculator = new Calculator( mainFrame.getConfigData() );

        final int rowHeaderWidth = 26;
        rowHeaderModel = new RowHeaderTableModel();
        rowHeaderTable = new TooltipTable(rowHeaderModel) {
            private static final long serialVersionUID = -8707317721680383319L;
            @Override
            public Dimension getPreferredScrollableViewportSize()
            {
                Dimension size = getPreferredSize();
                size.width = rowHeaderWidth;
                return size;
            }
        };
        rowHeaderTable.setAutoResizeMode( JTable.AUTO_RESIZE_OFF );
        TableColumnModel columnModel = rowHeaderTable.getColumnModel();
        TableColumn column;
        column = columnModel.getColumn( 0 );
        column.setMinWidth( rowHeaderWidth );
        column.setMaxWidth( rowHeaderWidth );
        column.setPreferredWidth( rowHeaderWidth );
        rowHeaderTable.setDefaultRenderer( Integer.class, new NumberTableCellRenderer() );
    }

    /* synchronize our row count with row header */
    @Override
    public void fireTableChanged( TableModelEvent e ) {
        rowHeaderModel.setRows(getRowCount());
        super.fireTableChanged(e);
    }

    public JTable getRowHeaderTable() {
        return rowHeaderTable;
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        return 7;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @Override
    public String getColumnName( int columnIndex )
    {
        ConfigData configData = mainFrame.getConfigData();
        String rv ="";
        switch( columnIndex )
        {
            case 0:
                rv = configData.getI18nText( "Savecalc.table.start" );
                break;
            case 1:
                rv = configData.getI18nText( "Savecalc.table.end" );
                break;
            case 2:
                rv = configData.getI18nText( "Savecalc.table.longdistance" );
                break;
            case 3:
                rv = configData.getI18nText( "Savecalc.table.factor" );
                break;
            case 4:
                rv = configData.getI18nText( "Savecalc.table.speed" );
                break;
            case 5:
                rv = configData.getI18nText( "Savecalc.table.oneway" );
                break;
            case 6:
                rv = configData.getI18nText( "Savecalc.table.total" );
                break;

        }
        return rv;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return data.size();
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        SaveTarget target = data.get( rowIndex );
        Object rv = null;
        switch( columnIndex )
        {
            case 0:
                rv = target.getOrigin();
                break;
            case 1:
                rv = getPlanets(target);
                break;
            case 2:
                rv = target.isLongDistance();
                break;
            case 3:
                rv = Integer.toString( target.getVelocity() ) + "% ";
                break;
            case 4:
                rv = target.getUnit();
                break;
            case 5:
                rv = calculator.formatTime( target.getDuration() / 2.0 );
                break;
            case 6:
                rv = calculator.formatTime( target.getDuration() );

        }
        return rv;
    }


    /**
     * @see javax.swing.table.AbstractTableModel#isCellEditable(int, int)
     */
    @Override
    public boolean isCellEditable( int rowIndex, int columnIndex )
    {
        boolean rv = false;
        if( columnIndex == 1 && edit )
        {
            SaveTarget target = data.get( rowIndex );
            rv = !target.getPlanet().getPosition().isRealPosition();
        }
        return rv;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#setValueAt(java.lang.Object, int, int)
     */
    @Override
    public void setValueAt( Object value, int rowIndex, int columnIndex )
    {
        if( columnIndex == 1 )
        {
            SaveTarget target = data.get( rowIndex );
            Position position = ((Planet)value).getPosition();
            target.getPlanet().setPosition( position );
        }
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
     */
    @Override
    public Class<?> getColumnClass( int columnIndex )
    {
        // we return String and use our own cell renderer
        return String.class;
    }

    public List<SaveTarget> getData()
    {
        return new ArrayList<SaveTarget>( data );
    }

    public void setData( List<SaveTarget> data )
    {
        this.data.clear();
        this.data.addAll( data );

        if( sortColumn >= 0 )
        {
            switch( sortOrder )
            {
                case Ascending:
                    Collections.sort( this.data, comparator );
                    break;
                case Descending:
                    Collections.sort( this.data, Collections.reverseOrder( comparator ) );
                    break;
                default:
                    break;
            }
        }
        fireTableDataChanged();
    }

    public void addRow( SaveTarget target )
    {
        int insertRow = data.size();
        Comparator<SaveTarget> c = null;
        if( sortColumn >= 0 )
        {
            switch( sortOrder )
            {
                case Ascending:
                    c = comparator;
                    break;
                case Descending:
                    c = Collections.reverseOrder( comparator );
                    break;
                default:
                    break;
            }
        }

        if( c != null )
        {
            // list is sorted, insert at right position
            for( int i = 0; i < data.size(); i++ )
            {
                if( c.compare( data.get( i ), target ) > 0 )
                {
                    insertRow = i;
                    break;
                }
            }
        }

        if( insertRow < data.size() )
        {
            data.add( insertRow, target );
        }
        else
        {
            // just append
            data.add( target );
        }
        fireTableRowsInserted( insertRow, insertRow );
    }

    public SaveTarget getRow( int rowIndex )
    {
        return data.get( rowIndex );
    }

    public void removeAll()
    {
        data.clear();
        fireTableDataChanged();
    }

    public void removeRow( int rowIndex )
    {
        data.remove( rowIndex );
        fireTableRowsDeleted( rowIndex, rowIndex );
    }

    public void removeRows( int[] rows )
    {
        // delete back to front to preserve indices
        Arrays.sort( rows );
        for( int i = rows.length - 1; i >= 0; i-- )
        {
            data.remove( rows[i] );
        }
        fireTableDataChanged();
    }

    public int getSortColumn()
    {
        int rv = -1;
        if( sortOrder != SortOrder.None )
        {
            rv = sortColumn;
        }
        return rv;
    }

    public SortOrder getSortOrder()
    {
        return sortOrder;
    }

    public void setSortColumn(int columnIndex, SortOrder sortOrder )
    {
        if( columnIndex >= 0 && columnIndex != sortColumn )
        {
            sortColumn = columnIndex;
            SaveTargetComparatorMode mode;
            switch( sortColumn )
            {
                case 0:
                    mode = SaveTargetComparatorMode.SORT_BY_ORDER;
                    comparator.setOrder( mainFrame.getConfigData().getMyPlanets() );
                    break;
                case 1:
                    mode = SaveTargetComparatorMode.SORT_BY_PLANET;
                    break;
                case 2:
                    mode = SaveTargetComparatorMode.SORT_BY_DISTANCE;
                    break;
                case 3:
                    mode = SaveTargetComparatorMode.SORT_BY_VELOCITY;
                    break;
                case 4:
                    mode = SaveTargetComparatorMode.SORT_BY_UNIT;
                    break;
                case 5:
                case 6:
                default:
                    mode = SaveTargetComparatorMode.SORT_BY_DURATION;
                    break;
            }
            comparator.setMode( mode );
            if( sortOrder != null )
            {
                this.sortOrder = sortOrder;
            }
            switch( this.sortOrder )
            {
                case Ascending:
                    Collections.sort( data, comparator );
                    break;
                case Descending:
                    Collections.sort( data, Collections.reverseOrder( comparator ) );
                    break;
                default:
                    break;
            }
            fireTableDataChanged();
        }
    }

    public void setSortOrder(SortOrder sortOrder)
    {
        if( this.sortOrder != sortOrder )
        {
            this.sortOrder = sortOrder;
            if( sortColumn >= 0 )
            {
                switch( sortOrder )
                {
                    case Ascending:
                        Collections.sort( data, comparator );
                        break;
                    case Descending:
                        Collections.sort( data, Collections.reverseOrder( comparator ) );
                        break;
                    default:
                        break;
                }
                fireTableDataChanged();
            }
        }
    }

    @Override
    public boolean markResource( int row, int column )
    {
        boolean rv = false;

        if( configData.getVersion() == Version.v20 )
        {
            SaveTarget target = data.get( row );
            rv = !target.getOrigin().getPosition().isRealPosition() || !target.getPlanet().getPosition().isRealPosition();
        }
        return rv;
    }

    private Object getPlanets( SaveTarget target ) {
        Object rv = target.getPlanet();
        if(edit) {
            Version version = target.getAccount().getVersion();
            switch(version) {
                case genesis:
                case sirius:
                case infinity:
                    Position position = target.getPlanet().getPosition();
                    if(position.getPlanet() == 0) {
                        if(position.getSystem() == 0) {
                            // need a system for map lookup, use the one from origin
                            position = new Position(position.getGalaxy(), target.getOrigin().getPosition().getSystem(), 0);
                        }
                        rv = SystemData.getVisiblePlanets(version, position);
                    }
                    break;
                default:
                    break;
            }
        }
        return rv;
    }

}
