/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Cost;
import sitool.components.Planet;
import sitool.components.Resource;
import sitool.components.TotalPointDisplay;

/**
 * @author graf
 *
 */
public class StockpileTableModel extends AbstractTableModel implements MouseListener
{
    private static final long serialVersionUID = -1405529512618782948L;
    private ConfigData configData;
    private Map<Planet, boolean[]> use = new HashMap<Planet, boolean[]>();
    private Map<Planet, Cost> values = new HashMap<Planet, Cost>();
    private Cost flightResources = new Cost(0, 0, 0, 0);
    private boolean[] flightSettings = new boolean[Resource.values().length + 1];

    static final String EclipsedValue = "-";

    public StockpileTableModel( MainFrame mainFrame )
    {
        configData = mainFrame.getConfigData();
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        return 2 * Resource.values().length + 1;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @Override
    public String getColumnName( int columnIndex )
    {
        String rv ="";
        if( columnIndex > 0 )
        {
            if( columnIndex % 2 == 0)
            {
                rv = "";
            }
            else
            {
                if( columnIndex / 2 < Resource.values().length )
                {
                    rv = configData.getI18nText( Resource.values()[columnIndex / 2].name() );
                }
            }
        }
        return rv;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return configData.getMyPlanetsAndAsteroids().size() + 1;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Planet planet = planetForRow(rowIndex);
        boolean[] useArray = getUseArray( planet );
        Object rv = "";

        if( columnIndex == 0 )
        {
            rv = (planet != null) ? planet.toString() : configData.getI18nText("Resources.inFlight");
        }
        else if( columnIndex % 2 == 0 )
        {
            rv = Boolean.valueOf( useArray[columnIndex/2-1] );
        }
        else
        {
            if( useArray[columnIndex/2] )
            {
                Resource resource = Resource.values()[columnIndex/2];
                Cost cost = getCost( planet );
                rv = format( cost.getResource( resource ) );
            }
            else
            {
                rv = EclipsedValue;
            }
        }
        return rv;
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        boolean rv = false;
        if( columnIndex > 0 )
        {
            if( columnIndex % 2 == 0 )
            {
                rv = true;
            }
            else
            {
                Planet planet = planetForRow(rowIndex);
                boolean[] useArray = getUseArray( planet );
                rv = useArray[columnIndex/2];
            }
        }
        return rv;
    }


    @Override
    public void setValueAt( Object value, int rowIndex, int columnIndex )
    {
        Planet planet = planetForRow(rowIndex);
        boolean[] useArray = getUseArray( planet );
        if( columnIndex % 2 == 0 )
        {
            useArray[columnIndex/2-1] = Boolean.valueOf( value.toString() );
        }
        else
        {
            StringBuffer sb = new StringBuffer();
            sb.append( value.toString() );
            for( int i = sb.length() - 1; i >= 0; i-- )
            {
                char ch = sb.charAt( i );
                if( !Character.isDigit( ch ) )
                {
                    sb.deleteCharAt( i );
                }
            }
            // if nothing is left, use '0'
            if( sb.length() == 0 )
            {
                sb.append( '0' );
            }
            Resource resource = Resource.values()[columnIndex/2];
            Cost cost = getCost( planet );
            cost.setResource( resource, Long.valueOf( sb.toString() ) );
        }
        updateRow( rowIndex );
    }

    public void setData( Map<Planet, boolean[]> use, Map<Planet, Cost> values )
    {
        this.use = use;
        this.values = values;
        fireTableDataChanged();
    }

    public Map<Planet, Cost> getValues()
    {
        return values;
    }

    public Map<Planet, boolean[]> getUse()
    {
        return use;
    }

    public Cost getFlightResources() {
        return flightResources;
    }

    public boolean[] getFlightSettings() {
        return flightSettings;
    }

    public void update()
    {
        fireTableStructureChanged();
    }

    protected String format( long value )
    {
        return TotalPointDisplay.format( value );
    }

    /**
     * Returns the planet for the given row index or <code>null</code> for
     * the in flight row
     * @param rowIndex
     * @return
     */
    private Planet planetForRow( int rowIndex) {
        List<Planet> data = configData.getMyPlanetsAndAsteroids();
        Planet planet = (rowIndex != 0) ? data.get( rowIndex - 1) : null;
        return planet;
    }

    /**
     * Return and, if necessary initialize use array for planet
     * @param planet
     * @return
     */
    private boolean[] getUseArray( Planet planet )
    {
        boolean[] uArray;
        if(planet != null) {
            uArray = use.get( planet );
            if( uArray == null )
            {
                uArray = new boolean[Resource.values().length + 1];
                Arrays.fill( uArray, true );
                use.put( planet, uArray );
            }
        }
        else {
            uArray = flightSettings;
        }
        return uArray;
    }

    private Cost getCost( Planet planet )
    {
        Cost cost;
        if(planet != null) {
            cost = values.get( planet );
            if( cost == null )
            {
                cost = new Cost( 0, 0, 0, 0 );
                values.put( planet, cost );
            }
        }
        else {
            cost = flightResources;
        }
        return cost;
    }

    private void updateRow( int rowIndex )
    {
        fireTableRowsUpdated( (rowIndex > 0) ? rowIndex - 1 : rowIndex, rowIndex );
        // update summary lines in ResourceTable
        int last = configData.getMyPlanets().size();
        fireTableRowsUpdated( last, last + 1 );
    }

    public void mouseClicked( MouseEvent e )
    {
        Object o = e.getSource();
        if( o instanceof JTable )
        {
            JTable table = (JTable)o;
            int row = table.rowAtPoint(e.getPoint());
            int col = table.columnAtPoint(e.getPoint());
            // convert to model
            int rowIndex = table.convertRowIndexToModel( row );
            int columnIndex = table.convertColumnIndexToModel( col );
            if ( SwingUtilities.isLeftMouseButton(e) && e.getClickCount() == 1 )
            {
                if( columnIndex == 0 )
                {
                    Planet planet = planetForRow(rowIndex);
                    boolean[] uArray = getUseArray( planet );
                    boolean uValue = false;
                    for( int i = 0; !uValue && i < uArray.length; i++ )
                    {
                        uValue |= uArray[i];
                    }
                    Arrays.fill( uArray, !uValue );
                    updateRow( rowIndex );
                }
            }
        }
    }

    public void mouseEntered( MouseEvent e )
    {}

    public void mouseExited( MouseEvent e )
    {}

    public void mousePressed( MouseEvent e )
    {}

    public void mouseReleased( MouseEvent e )
    {}
}
