/****************************************
 *  COPYRIGHT (C) 2022
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.JTable;
import javax.swing.SwingUtilities;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Building;
import sitool.components.Cost;
import sitool.components.ListItem;
import sitool.components.Planet;
import sitool.components.Research;
import sitool.components.Resource;
import sitool.components.Ship;
import sitool.components.Stage;
import sitool.technology.TechCalculator;
import sitool.technology.TechtreeDependency;

/**
 * @author graf
 *
 */
public class TechBuilderTableModel extends StatisticsTableModel implements MouseListener
{
    private static final long serialVersionUID = -3137885733068377331L;
    private static final long speedupCutoff = 1000l * 60l * 60l * 24l * 80l;

    public final static int RESEARCH = 0;
    public final static int LEVEL = RESEARCH + 1;
    public final static int IRON = LEVEL + 1;
    public final static int METAL = IRON + 1;
    public final static int KRYPTONITE = METAL + 1;
    public final static int SPICE = KRYPTONITE + 1;
    public final static int TIME = SPICE + 1;
    public final static int TRANSMITTER = TIME + 1;
    public final static int POINTS = TRANSMITTER + 1;
    public final static int DOWN = POINTS + 1;
    public final static int UP = DOWN + 1;

    private static Icon upArrow;
    private static Icon downArrow;

    private List<ListItem<Research>> data = new ArrayList<ListItem<Research>>();
    private Map<ListItem<Research>, Double> times = new HashMap<>();

    public TechBuilderTableModel( MainFrame mainFrame )
    {
        super( mainFrame );
        upArrow = ConfigData.getIcon( "/button_up.gif" );
        downArrow = ConfigData.getIcon( "/button_down.gif" );
    }

    public int getColumnCount()
    {
        return UP + 1;
    }

    public int getRowCount()
    {
        return data.size() + 2;
    }

    public String getColumnName( int columnIndex )
    {
        String rv ="";
        switch( columnIndex )
        {
            case RESEARCH:
                rv = formatHeader( configData.getI18nText( "Research" ) );
                break;
            case LEVEL:
                rv = formatHeader( configData.getI18nText( "Level" ) );
                break;
            case IRON:
            case METAL:
            case KRYPTONITE:
            case SPICE:
                rv = formatHeader( configData.getI18nText( Resource.values()[columnIndex - IRON].name() ) );
                break;
            case TIME:
                rv = formatHeader( configData.getI18nText( "Researchtime" ).split( " ", 2 ) );
                break;
            case TRANSMITTER:
                rv =  formatHeader( configData.getI18nText( "Transmitters" ).split( " ", 2 ) );
                break;
            case POINTS:
                rv =  formatHeader( configData.getI18nText( "Points" ).split( " ", 2 ) );
                break;
        }
        return rv;
    }

    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Object rv = "";
        if( rowIndex < data.size() )
        {
            ListItem<Research> item = data.get( rowIndex );
            Cost cost = TechCalculator.calculateCost( item.getItem(), item.getLevel() );
            switch( columnIndex )
            {
                case RESEARCH:
                    rv = configData.getI18nText( item.getItem().name() );
                    break;
                case LEVEL:
                    rv = Integer.valueOf( item.getLevel() );
                    break;
                case IRON:
                case METAL:
                case KRYPTONITE:
                case SPICE:
                    rv = format( cost.getResource( Resource.values()[columnIndex - IRON] ) );
                    break;
                case TIME:
                    // as a side effect the build time is saved to <code>times</code>
                    rv = formatTime( rowIndex, TechCalculator.researchTime(cost) );
                    break;
                case TRANSMITTER:
                    long l = cost.getIron() + cost.getMetal() + cost.getKryptonite() + cost.getSpice();
                    if( l > 0 )
                    {
                        int transmittercapacity = (100 + configData.getSpecialization().getCapacitybonus()) * Ship.transmitter.getLoad() / 100;
                        l /= transmittercapacity;
                        l++;
                    }
                    rv = format( l );
                    break;
                case POINTS:
                    rv = format( (long)cost.points() );
                    break;
                case DOWN:
                    if( rowIndex < data.size() - 1 )
                    {
                        rv = downArrow;
                    }
                    break;
                case UP:
                    if( rowIndex > 0 )
                    {
                        rv = upArrow;
                    }
                    break;
             }
        }
        else
        {
            if( rowIndex == data.size() )
            {
                switch( columnIndex )
                {
                    case IRON:
                    case METAL:
                    case KRYPTONITE:
                    case SPICE:
                    case TRANSMITTER:
                    case POINTS:
                        Cost cost = new Cost( 0, 0, 0, 0 );
                        for( int index = 0; index < data.size(); index++ )
                        {
                            ListItem<Research> item = data.get( index );
                            cost.add( TechCalculator.calculateCost( item.getItem(), item.getLevel() ) );
                        }
                        if( columnIndex <= SPICE )
                        {
                            rv = format( cost.getResource( Resource.values()[columnIndex - IRON] ) );
                        }
                        else if( columnIndex == TRANSMITTER )
                        {
                            long l = cost.getIron() + cost.getMetal() + cost.getKryptonite() + cost.getSpice();
                            if( l > 0 )
                            {
                                int transmittercapacity = (100 + configData.getSpecialization().getCapacitybonus()) * Ship.transmitter.getLoad() / 100;
                                l /= transmittercapacity;
                                l++;
                            }
                            rv = format( l );
                        }
                        else  if( columnIndex == POINTS )
                        {
                            rv = format( (long)cost.points() );
                        }
                        break;
                    case TIME:
                        double total = 0;
                        for(Double time : times.values()) {
                            total += time.doubleValue();
                        }
                        if(total > 0) {
                            rv = formatTime(total);
                        }
                        break;
                }
            }
            else
            {
                switch( columnIndex )
                {
                    case RESEARCH:
                        rv = configData.getI18nText( "resourceequivalenz" );
                        break;
                    case IRON:
                    case METAL:
                    case KRYPTONITE:
                    case SPICE:
                        Cost cost = new Cost( 0, 0, 0, 0 );
                        for( int index = 0; index < data.size(); index++ )
                        {
                            ListItem<Research> item = data.get( index );
                            cost.add( TechCalculator.calculateCost( item.getItem(), item.getLevel() ) );
                        }
                        double value = getWeightedResourceEquivalenz( cost );
                        int weight = Resource.values()[columnIndex - IRON].getWeight();
                        rv = format( Math.round( weight * value ) );
                        break;
                    case TRANSMITTER:
                    case POINTS:
                    case TIME:
                        break;
                }
            }
        }
        return rv;
    }

    public void addItem( ListItem<Research> item )
    {
        data.add( item );
        fireTableDataChanged();
    }

    public void removeAll()
    {
        data.clear();
        times.clear();
        fireTableDataChanged();
    }

    public void removeRow( int row )
    {
        ListItem<Research> item = data.remove( row );
        times.remove(item);
        fireTableDataChanged();
    }

    @Override
    public boolean markResource( int row, int column ) {
        return false;
    }

    public void mouseClicked( MouseEvent e )
    {
        Object o = e.getSource();
        if( o instanceof JTable )
        {
            JTable table = (JTable)o;
            int row = table.rowAtPoint(e.getPoint());
            int col = table.columnAtPoint(e.getPoint());
            // convert to model
            col = table.convertColumnIndexToModel( col );
            if ( SwingUtilities.isLeftMouseButton(e) && e.getClickCount() == 1 )
            {
                if( col == UP || col == DOWN )
                {
                    int start = -1;
                    int stop = -1;
                    switch( col )
                    {
                        case UP:
                            start = row - 1;
                            stop = row + 1;
                            break;
                        case DOWN:
                            start = row;
                            stop = row + 2;
                            break;
                    }

                    if( start >= 0 && stop <= data.size() )
                    {
                        if( enableSwap( start, stop - 1 ) )
                        {
                            // switch them over
                            Collections.rotate( data.subList( start, stop ), 1 );
                            if( col == UP)
                            {
                                table.getSelectionModel().setSelectionInterval( start, start );
                            }
                            else
                            {
                                table.getSelectionModel().setSelectionInterval( stop - 1, stop - 1 );
                            }
                            fireTableDataChanged();
                        }
                    }
                }
            }
        }
    }

    public List<ListItem<Research>> getData()
    {
        return data;
    }

    public void setData( List<ListItem<Research>> data )
    {
        this.data.clear();
        if( data != null )
        {
            this.data.addAll( data );
        }
        fireTableDataChanged();
    }

    public void mouseEntered( MouseEvent e )
    {}

    public void mouseExited( MouseEvent e )
    {}

    public void mousePressed( MouseEvent e )
    {}

    public void mouseReleased( MouseEvent e )
    {}

    private boolean enableSwap( int pos1, int pos2 )
    {
        boolean rv = true;
        ListItem<Research> i1 = data.get( pos1 );
        ListItem<Research> i2 = data.get( pos2 );
        if( i1.getPlanet().equals( i2.getPlanet() ) )
        {
            // don't allow higher values of same building before lower
            if( i1.getItem() == i2.getItem() && i1.getLevel() < i2.getLevel() )
            {
                rv = false;
            }
            else
            {
                // check tech dependency up to pos1 - 1
                TechtreeDependency techtreeDependency = new TechtreeDependency( configData );
                List<ListItem<Research>> buildListItems = data.subList( 0, pos1 );
                techtreeDependency.updateTechnology(buildListItems);
                rv = techtreeDependency.isEnabled( i2.getItem() );
            }
        }
        return rv;
    }

    private String formatTime( int rowIndex, double buildTime )
    {
        double speedFactor = 1.0;
        double speedUp = 1.0;
        long researchCap = getResearchCap();
        Account current = configData.getActiveAccount();
        if( current != null )
        {
            speedFactor = current.getSpeedFactor();
            speedUp = current.getSpeedUp();
        }
        if(buildTime < speedupCutoff) {
            buildTime /= speedUp;
        }
        buildTime /= speedFactor;
        ListItem<Research> item = data.get( rowIndex );
        Planet planet = configData.getPlanet( item.getPlanet() );
        Stage  stage = planet.getStage( Building.researchLab, false );
        int labLevel = ( stage != null ) ? stage.getLevel() : 0;
        labLevel = (labLevel == 0) ? 1 : labLevel;
        buildTime /= labLevel;
        buildTime = (researchCap > 0 && buildTime > researchCap) ? researchCap : Math.floor( buildTime );

        times.put(item, buildTime);

        return formatTime( buildTime );
    }
}
