/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool.technology;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;

import com.fasterxml.jackson.jr.ob.JSON;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Research;
import sitool.components.Technology;
import sitool.components.Version;
import sitool.datatransfer.DataImport;
import sitool.json.IntergalacticOverviewBean;
import sitool.json.JSONTags;


/**
 *
 * @author graf
 *
 */
public class ResearchBox extends JDialog implements ActionListener
{
    private static final long serialVersionUID = -4586875011135823902L;

    private JTextArea textArea = new JTextArea();
    private ConfigData configData;
    private Version version;

    private MainFrame mainFrame = null;
    private Pattern levelPattern = Pattern.compile( "\\(\\D*(\\d+)\\D*\\)" );
    private Pattern levelPattern2 = Pattern.compile( "\\[(\\d+)\\]\\s+(\\[.+:\\s+(\\d+)\\])?" );

    // these Scanboxes are all pretty much the same, they just need a different scanner...
    public ResearchBox( MainFrame parent )
    {
	    super( parent );
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        version = configData.getVersion();

        enableEvents( AWTEvent.WINDOW_EVENT_MASK );
        this.setTitle( configData.getI18nText( "TechBox.title" ) );
        JPanel contentPane = ( JPanel )getContentPane();
        contentPane.setLayout( new BorderLayout() );
        contentPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel center = createCenterPanel();
        center.setPreferredSize( new Dimension( 640, 520 ) );

        JPanel lower = new JPanel( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
        JButton button = new JButton( configData.getI18nText( "OK" ) );
        button.addActionListener( this );
        lower.add( button );
        getRootPane().setDefaultButton( button );

        contentPane.add( center, BorderLayout.CENTER );
        contentPane.add( lower, BorderLayout.SOUTH );
        pack();

        // by popular demand, add paste clipboard to RMB
        MouseAdapter l = new MouseAdapter()
        {

            @Override
            public void mouseClicked( MouseEvent e )
            {
                if( SwingUtilities.isRightMouseButton( e ) )
                {
                    String input = "";
                    Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
                    Transferable contents = clipboard.getContents( null );
                    boolean hasTransferableText = ( ( contents != null )
                                    && contents.isDataFlavorSupported( DataFlavor.stringFlavor ) );
                    if( hasTransferableText )
                    {
                        try
                        {
                            input = ( String )contents.getTransferData( DataFlavor.stringFlavor );
                        }
                        catch( UnsupportedFlavorException ex )
                        {
                            System.out.println( ex );
                            ex.printStackTrace();
                        }
                        catch( IOException ex )
                        {
                            System.out.println( ex );
                            ex.printStackTrace();
                        }
                    }
                    textArea.setText( input );
                }
            }

        };

        textArea.addMouseListener( l );
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );
        center.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10) );

        JScrollPane scrollPane = new JScrollPane( textArea,
                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                        JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        center.add( scrollPane, BorderLayout.CENTER );

        return center;
    }

    // Overridden so we can exit when window is closed
    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    protected void processWindowEvent( WindowEvent e )
    {
        if( e.getID() == WindowEvent.WINDOW_CLOSING )
        {
            cancel();
        }
        super.processWindowEvent( e );
    }

    // Close the dialog

    /***************************************************************************
     * Description of the Method
     **************************************************************************/
    void cancel()
    {
        dispose();
    }

    // Close the dialog on a button event

    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    public void actionPerformed( ActionEvent e )
    {
        String input = textArea.getText();
        if( input.indexOf(JSONTags.reportType_PlanetOverview) > -1 ) {
            scanIntergalacticPlanetOverviewJSON(input);
        }
        else {
            if( version != Version.v20 )
            {
                // remove current research, if any
                int pos;
                String currentMarker = configData.getI18nText( "cancel" ) + '\n';
                if( ( pos = input.indexOf( currentMarker ) ) >= 0 )
                {
                    input = input.substring( pos + currentMarker.length() );
                }
            }
            BufferedReader br = new BufferedReader( new StringReader( input ) );
            String line;
            try
            {
                Technology myTech = configData.getMyTech();
                while( (line = br.readLine() ) != null )
                {
                    if( version != Version.v20 )
                    {
                        scanLine( myTech, line );
                    }
                    else
                    {
                        scanLine2( myTech, line );
                    }
                }
                br.close();
                configData.setMyTech( myTech );
            }
            catch( IOException e1 )
            {}
        }
        mainFrame.getDisplay().update();
        mainFrame.actionUpdatePane();
        dispose();
    }

    private void scanIntergalacticPlanetOverviewJSON( String input ) {
        try {
            IntergalacticOverviewBean overview = JSON.std.beanFrom(IntergalacticOverviewBean.class, input);
            Technology tech = DataImport.createTechnologyFromJSON(overview.getResearches());
            Account account = mainFrame.getConfigData().getActiveAccount();
            DataImport.updateConfiguration(account, null, tech, null, null);
        }
        catch(IOException e) {
            e.printStackTrace();
        }

    }

    private void scanLine( Technology myTech, String line )
    {
        // scan for Tech
        for( Research research : Research.values() )
        {
            if( line.indexOf( configData.getI18nText( research.name() ) ) >= 0 )
            {
                Matcher m = levelPattern.matcher( line );
                if( m.find() )
                {
                    int level = Integer.parseInt( m.group( 1 ) );
                    myTech.setTech( research, level );
                }
            }
        }
    }

    private void scanLine2( Technology myTech, String line )
    {
        // scan for Tech
        for( Research research : Research.values() )
        {
            if( line.indexOf( configData.getI18nText( research.name() ) ) >= 0 )
            {
                Matcher m = levelPattern2.matcher( line );
                if( m.find() )
                {
                    int level = Integer.parseInt( m.group( 1 ) );
                    if( m.group( 3 ) != null )
                    {
                        level -= Integer.parseInt( m.group( 3 ) );
                    }
                    myTech.setTech( research, level );
                }
            }
        }
    }
}
